#include <stdio.h>
#include <stdlib.h>
#include <math.h>
#include <string.h>
#include <unistd.h>
#ifdef LINUX
#include "linux.h"
#else
#include "sun.h"
#endif

typedef float imtype[800][800];
typedef char fntype[20];

int *DMAX,*DMIN,ODMAX,ODMIN,N,TEMP;
imtype *data,final;
double *mult,EXP,EXP0,GAIN,RN,EPOCH;
fntype FILT;

int ferr(FILE *f,char *str) {
   printf("%s\n",str);
   fclose(f);
   exit(1);
   return -1;
}

FILE *readfits(char *fn,int n) {
   FILE *f;
   char str[82];
   int MODE=0,line=0,tmp,HPP=0;

   if ((f=fopen(fn,"rb"))==NULL) {
      printf("File \"%s\" not found\n",fn);
      exit(1);
   }
   while ((!feof(f)) && ((!MODE) || (line%36))) {
      fgets(str,81,f);
      line++;
      if (!MODE) {
	 if (!strncmp(str,"END     ",8)) MODE=1;
	 else if ((!strncmp(str,"SIMPLE  =",9)) && (str[29]!='T')) ferr(f,"Illegal format type");
	 else if ((!strncmp(str,"BITPIX  =",9)) && (strncmp(str+27,"-32",3))) ferr(f,"Illegal bits per pixel");
	 else if ((!strncmp(str,"NAXIS   =",9)) && (strncmp(str+27,"  3",3))) ferr(f,"Illegal number of axes");
	 else if (!strncmp(str,"NAXIS",5)) {
	    tmp=atoi(str+9);
	    if (str[5]=='1' && tmp!=800) ferr(f,"Illegal X axis size");
	    else if (str[5]=='2' && tmp!=800) ferr(f,"Illegal Y axis size");
	    else if (str[5]=='3' && tmp!=4) ferr(f,"Illegal Z axis size");
	 }
	 else if (!strncmp(str,"ATODGAIN=",9) && n==0) GAIN=atof(str+9);
	 else if (!strncmp(str,"RNOISE  =",9) && n==0) RN=atof(str+9);
	 else if (!strncmp(str,"EPOCH   =",9) && n==0) EPOCH=atof(str+9);
	 else if (!strncmp(str,"EXPTIME =",9) && n==0) EXP=atof(str+9);
	 else if (!strncmp(str,"EXP0    =",9) && n==0) EXP0=atof(str+9);
	 else if (!strncmp(str,"RSDPFILL=",9)) DMIN[n]=atoi(str+9);
	 else if (!strncmp(str,"SATURATE=",9)) DMAX[n]=atoi(str+9);
	 else if (!strncmp(str,"TEMP    =",9) && n==0) TEMP=atoi(str+9);
	 else if (!strncmp(str,"FILTNAM1=",9) && n==0) {
	    int i=0;
	    while (i<19 && str[i+11]!=32) {
	       FILT[i]=str[i+11];
	       i++;
	    }
	 }
	 else if (!strncmp(str,"HSTPHOT =",9)) HPP=1;
      }
   }
   if (!HPP) ferr(f,"Not pre-processed by mask");
   printf("Reading FITS file %s\n",fn);
   return f;
}

FILE *writefits(char*fn) {
   FILE *f;
   char str[81];
   int line;

   if ((f=fopen(fn,"wb"))==NULL) {
      printf("Error on output file\n");
      exit(1);
   }
   fputs("SIMPLE  =                    T / Standard FITS format                           ",f);
   fputs("BITPIX  =                  -32 / 32 bit IEEE floating point numbers             ",f);
   fputs("NAXIS   =                    3 / Number of axes                                 ",f);
   fputs("NAXIS1  =                  800 /                                                ",f);
   fputs("NAXIS2  =                  800 /                                                ",f);
   fputs("NAXIS3  =                    4 /                                                ",f);
   fputs("EXTEND  =                    T / There may be standard extensions               ",f);
   sprintf(str,"ATODGAIN=       %14f / Analog to Digital Gain (Electrons/DN)          ",GAIN);
   fputs(str,f);
   sprintf(str,"RNOISE  =       %14f / Readout Noise (DN)                             ",RN);
   fputs(str,f);
   sprintf(str,"EPOCH   =     %16.8f / exposure average time (Modified Julian Date)   ",EPOCH);
   fputs(str,f);
   sprintf(str,"EXPTIME =       %14f / exposure duration (seconds)--calculated        ",EXP);
   fputs(str,f);
   sprintf(str,"EXP0    =       %14f / weighted average initial exposure time         ",EXP0);
   fputs(str,f);
   sprintf(str,"RSDPFILL=       %14d / bad data fill value for calibrated images      ",ODMIN+1);
   fputs(str,f);
   sprintf(str,"SATURATE=       %14d / Data value at which saturation occurs          ",ODMAX-1);
   fputs(str,f);
   sprintf(str,"TEMP    =       %14d / Temperature (0=cold, 1=warm)                   ",TEMP);
   fputs(str,f);
   sprintf(str,"FILTNAM1= \'%-18s\' / first filter name                              ",FILT);
   fputs(str,f);
   fputs("HSTPHOT =                    T / Preprocessed by HSTphot/mask                   ",f);
   fputs("END                                                                             ",f);
   for (line=18;line<36;line++) fputs("                                                                                ",f);
   return f;
}

void arith(void) {
   int x,y,i,sat;
   for (x=0;x<800;x++) for (y=0;y<800;y++) {
      int pOK=1;
      final[x][y]=0;
      sat=0;
      for (i=0;i<N;i++) {
	 if (data[i][x][y]>DMIN[i] && data[i][x][y]<DMAX[i]) final[x][y]+=mult[i]*data[i][x][y];
	 else {
	    pOK=0;
	    if (data[i][x][y]>=DMAX[i]) sat=1;
	 }
      }
      if (!pOK) {
	 if (sat) final[x][y]=ODMAX+1;
	 else final[x][y]=ODMIN-1;
      }
   }
   return;
}

int main(int argc,char**argv) {
   int i,fld;
   FILE **fdata,*fout;
   double tmax=0,tmin=0;

   if (argc<2 || argc%2) {
      printf("Usage: %s <output> (<fits> <scale>) * nimg\n",*argv);
      return 1;
   }
   N=(argc-2)/2;
   fdata=(FILE**)calloc(N,sizeof(FILE*));
   data=(imtype*)calloc(N,sizeof(imtype));
   mult=(double*)calloc(N,sizeof(double));
   DMAX=(int*)calloc(N,sizeof(int));
   DMIN=(int*)calloc(N,sizeof(int));
   if (!fdata || !data || !mult || !DMAX || !DMIN) {
      printf("Memory allocation error\n");
      return 1;
   }
   for (i=0;i<N;i++) {
      fdata[i]=readfits(argv[2+2*i],i);
      mult[i]=atof(argv[3+2*i]);
      if (mult[i]>=0.) {
	 tmax+=mult[i]*DMAX[i];
	 tmin+=mult[i]*DMIN[i];
      }
      else {
	 tmin+=mult[i]*DMAX[i];
	 tmax+=mult[i]*DMIN[i];
      }
   }
   ODMAX=(int)tmax+1;
   ODMIN=(int)tmin-1;
   fout=writefits(argv[1]);
   for (fld=0;fld<4;fld++) {
      for (i=0;i<N;i++) ffread(data[i],800*800,4,fdata[i]);
      arith();
      ffwrite(final,800*800,4,fout);
   }
   for (i=0;i<N;i++) fclose(fdata[i]);
   fclose(fout);
   return 0;
}
