#include <stdio.h>
#include <stdlib.h>
#include <math.h>
#ifdef LINUX
#include "linux.h"
#else
#include "sun.h"
#endif
#include "filters.h"

typedef char ftype[81];
typedef int fpsftype[4];
typedef float dpsftype[4][2];
typedef struct {double m,dm,ct,bg,wt;} clustype;

int ssky=0,Nfilt,*filt,*circ[2];
float *RN,****opsf,***npsf,***isig2,***data,**cwt,*tct,*dtct;
fpsftype *fpsf;
dpsftype *dpsf;
clustype *clus;
int s_EXT;

int IMG,fld,rpsf=6,sub=5,rphot=3;
float rap=3;
float ssh,std_ct;

void merr(void) {
   printf("Memory allocation error\n");
   exit(-1);
}

int getfunc(char*str) {
   if (!strcasecmp(str,"gauss") || !strcasecmp(str,"g")) return 0;
   else if (!strcasecmp(str,"hubble") || !strcasecmp(str,"h")) return 1;
   else if (!strcasecmp(str,"moffat10") || !strcasecmp(str,"m10")) return 1;
   else if (!strcasecmp(str,"moffat15") || !strcasecmp(str,"m15")) return 2;
   else if (!strcasecmp(str,"moffat20") || !strcasecmp(str,"m20")) return 3;
   else if (!strcasecmp(str,"moffat25") || !strcasecmp(str,"m25")) return 4;
   else {
      printf("Unknown psf shape\n");
      exit(-1);
   }
}

float **alloc_psf(int f) {
   float **ptr;
   int r=6,i;

   if (f) r=(r*2+1)/3;
   ptr=(float**)calloc(sizeof(float*),2*r+1);
   if (!ptr) merr();
   ptr+=r;
   for (i=-r;i<=r;i++) {
      ptr[i]=(float*)calloc(sizeof(float),2*r+1);
      if (!ptr[i]) merr();
      ptr[i]+=r;
   }
   return ptr;
}

void init_pars(char*fn) {
   char str[321],*ptr;
   int i,j;
   FILE *f;

   initfilters();
   if ((f=fopen(fn,"r"))==NULL) {
      printf("Cannot open %s\n",fn);
      exit(-1);
   }
   fscanf(f,"%d",&Nfilt);
   filt=(int*)calloc(sizeof(int),Nfilt);
   RN=(float*)calloc(sizeof(float),Nfilt);
   tct=(float*)calloc(sizeof(float),Nfilt);
   dtct=(float*)calloc(sizeof(float),Nfilt);
   fpsf=(fpsftype*)calloc(sizeof(fpsftype),Nfilt);
   dpsf=(dpsftype*)calloc(sizeof(dpsftype),Nfilt);
   clus=(clustype*)calloc(sizeof(clustype),Nfilt);
   opsf=(float****)calloc(sizeof(float***),Nfilt);
   npsf=(float***)calloc(sizeof(float**),Nfilt);
   isig2=(float***)calloc(sizeof(float**),Nfilt);
   data=(float***)calloc(sizeof(float**),Nfilt);
   if (!filt || !RN || !tct || !dtct || !fpsf || !dpsf || !clus || !opsf || !npsf || !isig2 || !data) merr();
   fgets(str,321,f);
   for (i=0;i<Nfilt;i++) {
      opsf[i]=(float***)calloc(sizeof(float**),Nfilt);
      if (!opsf[i]) merr();
      fgets(str,321,f);
      str[strlen(str)-1]=0;
      filt[i]=findfilt(str);
      fscanf(f,"%f",RN+i);
      for (j=0;j<4;j++) {
	 fscanf(f,"%f %f",dpsf[i][j],dpsf[i][j]+1);
	 fgets(str,321,f);
	 str[strlen(str)-1]=0;
	 for (ptr=str;*ptr==' ';ptr++);
	 fpsf[i][j]=getfunc(ptr);
	 opsf[i][j]=alloc_psf(j);
      }
      npsf[i]=alloc_psf(0);
      isig2[i]=alloc_psf(0);
      data[i]=alloc_psf(0);
   }
   fclose(f);
}

void getcirc(void) {
   int r=3,i,j;
   double rap=3.;

   if (fld) {
      rpsf=4;
      sub=10;
      rap=2.;
      rphot=2;
   }
   else {
      rpsf=6;
      sub=5;
      rap=3.;
      rphot=3;
   }
   circ[0]=(int*)calloc(r*2+1,sizeof(int));
   circ[1]=(int*)calloc(r*2+1,sizeof(int));
   cwt=(float**)calloc(r*2+1,sizeof(float*));
   if (!circ[0] || !circ[1] || !cwt) merr();
   circ[0]+=r;
   circ[1]+=r;
   cwt+=r;
   for (i=-r;i<=r;i++) {
      circ[0][i]=50;
      circ[1][i]=-50;
      cwt[i]=(float*)calloc(r*2+1,sizeof(float));
      if (!cwt[i]) merr();
      cwt[i]+=r;
      for (j=-r;j<=r;j++) {
	 cwt[i][j]=rap+.465-sqrt(i*i+j*j);
	 if (cwt[i][j]<0) cwt[i][j]=0;
	 else if (cwt[i][j]>1) cwt[i][j]=1;
	 if (cwt[i][j]>0 && j<circ[0][i]) circ[0][i]=j;
	 if (cwt[i][j]>0 && j>circ[1][i]) circ[1][i]=j;
      }
   }
}

double psffunc(int t,double r2) {
   if (t==0) return exp(-r2);
   if (t==1) return 1./(1.+r2);
   if (t==2) return pow(1.+r2,-1.5);
   if (t==3) {double x=1.+r2; return 1./x/x;}
   if (t==4) return pow(1.+r2,-2.5);
   printf("PSF func error!!\n");
   exit(0);
   return 0.;
}

void normpsf(int f,float**psf) {
   double av=0.,r2,r=6.;
   int N=0,x,y,rph=3,rp=6;

   if (f) {
      r=4.;
      rph=2;
      rp=4;
   }
   if (r>r+2) r=r+2;
   for (x=-r;x<=r;x++) for (y=-r;y<=r;y++) {
      r2=sqrt(x*x+y*y);
      if (r2>=rph+1 && r2<=r) {
	 av+=psf[x][y];
	 N++;
      }
   }
   av/=N;
   for (x=-rp;x<=rp;x++) for (y=-rp;y<=rp;y++) psf[x][y]-=av;
}

void getpsf(void) {
   FILE *pf,*pf2;
   char fn[80];
   int f,tf,x,y,x1,y1,rad,r,px,py;
   float z;

   for (f=0;f<Nfilt;f++) {
      sprintf(fn,"%s/psfs/%s.6.5.psf",BASEDIR,filters[filt[f]].suffix);
      if ((pf=fopen(fn,"rb"))==NULL) {
	 printf("****File error: %s\n",fn);
	 exit(-1);
      }
      sprintf(fn,"%s/psfs/%s.6.5.dpsf",BASEDIR,filters[filt[f]].suffix);
      if ((pf2=fopen(fn,"rb"))==NULL) printf("No default PSF residual for %s\n",filters[filt[f]].name);
      for (tf=0;tf<4;tf++) {
	 if (tf==0) r=6;
	 else r=4;
	 if (tf==0) rad=3;
	 else rad=6;
	 for (px=0;px<8;px++) for (py=0;py<8;py++) for (x=-rad;x<=rad;x++) for (y=-rad;y<=rad;y++) for (x1=-r;x1<=r;x1++) for (y1=-r;y1<=r;y1++) {
	    ffread(&z,1,sizeof(float),pf);
	    if (abs(x)!=rad && abs(y)!=rad) opsf[f][tf][x1][y1]+=z;
	 }
	 for (x1=-r;x1<=r;x1++) for (y1=-r;y1<=r;y1++) opsf[f][tf][x1][y1]/=64*(2*rad-1)*(2*rad-1);
	 if (pf2) {
	    float z;
	    for (x=-r;x<=r;x++) for (y=-r;y<=r;y++) {
	       ffread(&z,sizeof(float),1,pf2);
	       opsf[f][tf][x][y]+=z;
	    }
	 }
	 if (fpsf[f][tf]>=0) {
	    double wmlt[5]={2.7725887,4.0000000,2.3496042,1.6568542,1.2780316};
	    float w,w0,r2,t=0,t0=0;

	    w0=wmlt[fpsf[f][tf]]/dpsf[f][tf][0]/dpsf[f][tf][0];
	    w=wmlt[fpsf[f][tf]]/dpsf[f][tf][1]/dpsf[f][tf][1];
	    for (x=-r;x<=r;x++) for (y=-r;y<=r;y++) {
	       r2=x*x+y*y;
	       t0+=psffunc(fpsf[f][tf],r2*w0);
	       t+=psffunc(fpsf[f][tf],r2*w);
	    }
	    for (x=-r;x<=r;x++) for (y=-r;y<=r;y++) {
	       r2=x*x+y*y;
	       r2=psffunc(fpsf[f][tf],r2*w)/t-psffunc(fpsf[f][tf],r2*w0)/t0;
	       opsf[f][tf][x][y]+=r2;
	       if (opsf[f][tf][x][y]<0) opsf[f][tf][x][y]=0;
	    }
	 }
	 normpsf(tf,opsf[f][tf]);
	 //printf("image %d, chip %d center: %f\n",f,tf,opsf[f][tf][0][0]);
	 //for (x=-r;x<=r;x++) {for (y=-r;y<=r;y++) printf(" %5d",(int)(100000*opsf[f][tf][x][y])); printf("\n");}
      }
      fclose(pf);
      if (pf2) fclose(pf2);
   }
}

//taken from numerical recipes;
#define ITMAX 1000
#define EPS 3.0e-8
#define SIGN(a,b) ((b) >= 0.0 ? fabs(a) : -fabs(a))
float zbrent(float (*func)(float), float x1, float x2, float tol)
{
	int iter;
	float a,b,c,d=0,e=0,min1,min2;
	float fa,fb,fc,p,q,r,s,tol1,xm;
	float calcsh_s(float);
	float calcdc(float);

	a=x1;
	b=c=x2;
	fa=(*func)(a);
	fb=(*func)(b);
	if (fa==0) return a;
	if (fb==0) return b;
	while (fa>0) {
	   a+=(a-b);
	   fa=(*func)(a);
	   if (fa>0 && func==&calcsh_s && a<=0) return 0;
	}
	while (fb<0) {
	   b+=(b-a);
	   fb=(*func)(b);
	   if (fb<0 && func==&calcsh_s && b>200) return 200.;
	}
	fc=fb;
	for (iter=1;iter<=ITMAX;iter++) {
		if ((fb > 0.0 && fc > 0.0) || (fb < 0.0 && fc < 0.0)) {
			c=a;
			fc=fa;
			e=d=b-a;
		}
		if (fabs(fc) < fabs(fb)) {
			a=b;
			b=c;
			c=a;
			fa=fb;
			fb=fc;
			fc=fa;
		}
		tol1=2.0*EPS*fabs(b)+0.5*tol;
		xm=0.5*(c-b);
		if (fabs(xm) <= tol1 || fb == 0.0) return b;
		if (fabs(e) >= tol1 && fabs(fa) > fabs(fb)) {
			s=fb/fa;
			if (a == c) {
				p=2.0*xm*s;
				q=1.0-s;
			} else {
				q=fa/fc;
				r=fb/fc;
				p=s*(2.0*xm*q*(q-r)-(b-a)*(r-1.0));
				q=(q-1.0)*(r-1.0)*(s-1.0);
			}
			if (p > 0.0) q = -q;
			p=fabs(p);
			min1=3.0*xm*q-fabs(tol1*q);
			min2=fabs(e*q);
			if (2.0*p < (min1 < min2 ? min1 : min2)) {
				e=d;
				d=p/q;
			} else {
				d=xm;
				e=d;
			}
		} else {
			d=xm;
			e=d;
		}
		a=b;
		fa=fb;
		if (fabs(d) > tol1)
			b += d;
		else
			b += SIGN(tol1,xm);
		fb=(*func)(b);
	}
	printf("Maximum number of iterations exceeded in zbrent\n");
	return 0.0;
}
#undef SIGN
#undef ITMAX
#undef EPS

//Elson, Fall, & Freeman 1987
//Elson's data consistent with single gamma=2.527773; chi^2_red=1.12404
//setting profile equal to the fractional amount of light in an annulus
//divided by area of that annulus
// deltaL = [(1+r2/a2)^(1-gam/2)-(1+r2/a2)^(1-gam/2)]

double profile(float rc,int x,int y) {
   double rval=0.;
   double a2=rc*rc*1.3688774;
   double r=sqrt(x*x+y*y);

   if (rc<=0) {
      if (x==0 && y==0) return 1.;
      return 0.;
   }
   if (x==0 && y==0) {
      return (1.-pow(1+0.31830989/a2,-0.2638865));
   }
   rval=(pow(1+(r-0.5)*(r-0.5)/a2,-0.2638865)-pow(1+(r+0.5)*(r+0.5)/a2,-0.2638865));
   rval/=3.1415927*((r+0.5)*(r+0.5)-(r-0.5)*(r-0.5));
   return rval;
}

void makenpsf(float rc) {
   int x,y,x2,y2,X,Y,img;
   float t;

   for (img=0;img<Nfilt;img++) for (x=-rpsf;x<=rpsf;x++) for (y=-rpsf;y<=rpsf;y++) npsf[img][x][y]=0.;
   //printf("%d %f\n",img,rc);
   for (x=-rpsf*2;x<=rpsf*2;x++) {for (y=-rpsf*2;y<=rpsf*2;y++) {
      t=profile(rc,x,y);
      //printf(" %5d",(int)(100000*t));
      for (x2=-rpsf;x2<=rpsf;x2++) for (y2=-rpsf;y2<=rpsf;y2++) if (abs(X=x+x2)<=rpsf && abs(Y=y+y2)<=rpsf) for (img=0;img<Nfilt;img++) npsf[img][X][Y]+=t*opsf[img][fld][x2][y2];
   }
   //printf("\n");
   }
   //printf("\n");
   for (img=0;img<Nfilt;img++) {
      if (ssky) normpsf(fld,npsf[img]);
      //for (x=-rpsf;x<=rpsf;x++) {for (y=-rpsf;y<=rpsf;y++) printf(" %5d",(int)(100000*npsf[img][x][y])); printf("\n");} printf("\n");
   }
}

float calcdc(float ct0) {
   int x,y,r3;
   float A=0.,B=0.,tx;

   srand(s_EXT+1);
   rand();
   for (x=-rpsf;x<=rpsf;x++) for (y=-rpsf;y<=rpsf;y++) {
      data[IMG][x][y]=npsf[IMG][x][y]*ct0;
      if (s_EXT>=0) {
	 if (data[IMG][x][y]>0) tx=sqrt(data[IMG][x][y]+clus[IMG].bg);
	 else tx=sqrt(clus[IMG].bg);
	 if ((r3=rand()%3)==0) data[IMG][x][y]+=tx;
	 else if (r3==2) data[IMG][x][y]-=tx;
      }
      if (data[IMG][x][y]>0) isig2[IMG][x][y]=1./(data[IMG][x][y]+clus[IMG].bg);
      else isig2[IMG][x][y]=1./clus[IMG].bg;
   }
   for (x=-rphot;x<=rphot;x++) for (y=circ[0][x];y<=circ[1][x];y++) {
      A+=data[IMG][x][y]*opsf[IMG][fld][x][y]*isig2[IMG][x][y]*cwt[x][y];
      B+=opsf[IMG][fld][x][y]*opsf[IMG][fld][x][y]*isig2[IMG][x][y]*cwt[x][y];
   }
   return A/B-std_ct;
}

float calcsh1(float rc,float*sh,float*ct0) {
   int x,y;
   float pm,A,tsh;

   makenpsf(rc);
   *sh=0.;
   //printf("p'-p=%f\n",npsf[0][0]-opsf[0][0]);
   for (IMG=0;IMG<Nfilt;IMG++) {
      std_ct=clus[IMG].ct;
      ct0[IMG]=zbrent(&calcdc,clus[IMG].ct*0.5,clus[IMG].ct*1.5,1.e-4);
      //printf("ct0=%e\n",ct0[IMG]);
      pm=A=0;
      for (x=-rphot;x<=rphot;x++) for (y=circ[0][x];y<=circ[1][x];y++) {
	 pm+=opsf[IMG][fld][x][y]*opsf[IMG][fld][x][y]*isig2[IMG][x][y]*cwt[x][y];
	 A+=opsf[IMG][fld][x][y]*isig2[IMG][x][y]*cwt[x][y];
      }
      pm/=A;
      //printf("<p>=%f\n",pm);
      tsh=A=0;
      for (x=-rphot;x<=rphot;x++) for (y=circ[0][x];y<=circ[1][x];y++) {
	 tsh+=(data[IMG][x][y]/clus[IMG].ct-opsf[IMG][fld][x][y])*(opsf[IMG][fld][x][y]-pm)*sqrt(isig2[IMG][x][y])*cwt[x][y];
	 A+=(opsf[IMG][fld][x][y]-pm)*(opsf[IMG][fld][x][y]-pm)*sqrt(isig2[IMG][x][y])*cwt[x][y];
      }
      (*sh)+=clus[IMG].wt*tsh/A;
      //printf("sh=%f\n",tsh);
      //printf("%f %f %f\n",rc,*ct0,tsh);
   }
   return (*sh);
}

float calcsh_s(float w) {
   float a=0;

   s_EXT=-1;
   return ssh-calcsh1(w,&a,tct);
}

float calcsh(float w,float*shav,float*shsd,float*ctav,float*ctsd,int NIT) {
   int i,j;
   float sh1,sh2,rval=0.,INIT;
   static float *ct1=NULL,*ct2=NULL;

   if (ct1==NULL) {
      ct1=(float*)calloc(sizeof(float),Nfilt);
      ct2=(float*)calloc(sizeof(float),Nfilt);
      if (!ct1 || !ct2) merr();
   }
   INIT=1./NIT;
   s_EXT=-1;
   *shav=*shsd=0.;
   for (j=0;j<Nfilt;j++) ctav[j]=ctsd[j]=0.;
   calcsh1(w,&sh1,ct1);
   for (i=0;i<NIT;i++) {
      s_EXT=i;
      rval+=calcsh1(w,&sh2,ct2)*INIT;
      (*shav)+=sh2*INIT;
      (*shsd)+=(sh2-sh1)*(sh2-sh1)*INIT;
      for (j=0;j<Nfilt;j++) {
	 ctav[j]+=ct2[j]*INIT;
	 ctsd[j]+=(ct2[j]-ct1[j])*(ct2[j]-ct1[j])*INIT;
      }
   }
   *shsd=sqrt(*shsd);
   for (j=0;j<Nfilt;j++) ctsd[j]=sqrt(ctsd[j]);
   //printf("%7.5f %7.4f %7.4f\n",w,*av,*sd);
   //printf("%f %f\n",w,rval);
   return rval;
}

int main(int argc,char**argv) {
   FILE *f;
   double x,y;
   int i;
   float w,dw,a=0,b=0,wm,wp,sh,shm,shp;

   if (argc!=3 && argc!=4) {
      printf("Usage: %s <info> <data> <<ssky>>\n",*argv);
      return -1;
   }
   if (argc==4) ssky=atoi(argv[3]);
   init_pars(argv[1]);
   getpsf();
   if ((f=fopen(argv[2],"r"))==NULL) {
      printf("Cannot open %s\n",argv[2]);
      return -1;
   }
   while (fscanf(f,"%d %lf %lf %f",&fld,&x,&y,&sh)!=EOF) {
      getcirc();
      w=0.;
      for (i=0;i<Nfilt;i++) {
	 fscanf(f,"%lf %lf %lf %lf %lf",&(clus[i].m),&(clus[i].dm),&(clus[i].ct),&(clus[i].bg),&(clus[i].wt));
	 if (clus[i].ct<1.e-20) clus[i].ct=1.e-20;
	 if (clus[i].bg<0) clus[i].bg=0;
	 clus[i].wt*=clus[i].wt/clus[i].ct;
	 w+=clus[i].wt;
	 clus[i].bg+=RN[i]*RN[i];
      }
      for (i=0;i<Nfilt;i++) clus[i].wt/=w;
      if (sh>0) shm=-0.01;
      if (sh>-0.01) shm=sh-0.01;
      else if (sh>-0.05) shm=sh*2;
      else shm=sh-0.05;
      if (shm<-1.) shm=-1.;
      if (sh>0.0) shp=0.;
      else if (sh>-0.1) shp=sh/2.;
      else shp=sh+0.05;
      ssh=shm;
      wm=zbrent(&calcsh_s,0.,1.,0.0001);
      ssh=shp;
      wp=zbrent(&calcsh_s,0.,1.,0.0001);
      ssh=sh;
      w=zbrent(&calcsh_s,0.,1.,0.0001);
      calcsh(w,&a,&b,tct,dtct,100);
      dw=-(wp-wm)/(shp-shm)*b;
      printf("%d %6.2f %6.2f %7.3f %6.3f %6.3f %6.3f %5.3f",fld,x,y,w,dw,ssh,a,b);
      for (i=0;i<Nfilt;i++) {
	 if (clus[i].m<90) printf(" %6.3f %5.3f %6.3f %5.3f",clus[i].m,clus[i].dm,-2.5*log10(tct[i]/clus[i].ct),2.5*dtct[i]/tct[i]/log(10));
	 else printf(" %6.3f %5.3f %6.3f %5.3f",99.999,0.,0.,0.);
      }
      printf("\n");
   }
   return 0;
}
