#include <stdio.h>
#include <stdlib.h>
#include <math.h>
#include <string.h>
#ifdef LINUX
#include "linux.h"
#else
#include "sun.h"
#endif

typedef float imtype[800][800];
typedef char fntype[20];

int *DMAX,*DMIN,*TEMP,N,TDMAX,TDMIN,TTEMP,nrej[4]={0,0,0,0},*BITS;
double *EXP,*EXP0,*GAIN,*RN,*EPOCH,TEXP,TGAIN,TRN,TEPOCH,TEXP0,*scale,*zero;
imtype *data,clean;
fntype *FILT,TFILT;

int ferr(FILE *f,char *str) {
   printf("%s\n",str);
   fclose(f);
   exit(1);
   return -1;
}

FILE *readfits(char *fn,int n) {
   FILE *f;
   char str[82];
   int MODE=0,line=0,tmp,HPP=0;

   if ((f=fopen(fn,"rb"))==NULL) {
      printf("File \"%s\" not found\n",fn);
      exit(1);
   }
   scale[n]=1;
   zero[n]=0;
   while ((!feof(f)) && ((!MODE) || (line%36))) {
      fgets(str,81,f);
      line++;
      if (!MODE) {
	 if (!strncmp(str,"END     ",8)) MODE=1;
	 else if ((!strncmp(str,"SIMPLE  =",9)) && (str[29]!='T')) ferr(f,"Illegal format type");
	 else if (!strncmp(str,"BITPIX  =",9)) {
	    BITS[n]=atoi(str+9);
	    if (BITS[n]!=32 && BITS[n]!=-32) ferr(f,"Illegal bits per pixel");
	 }
	 else if ((!strncmp(str,"NAXIS   =",9)) && (strncmp(str+27,"  3",3))) ferr(f,"Illegal number of axes");
	 else if (!strncmp(str,"NAXIS",5)) {
	    tmp=atoi(str+9);
	    if ((str[5]=='1' && tmp!=800) || (str[5]=='2' && tmp!=800) || (str[5]=='3' && tmp!=4)) {
	       printf("Illegal Axis Size\n");
	       exit(1);
	    }
	 }
	 else if (!strncmp(str,"BSCALE  =",9)) scale[n]=atof(str+9);
	 else if (!strncmp(str,"BZERO   =",9)) zero[n]=atof(str+9);
	 else if (!strncmp(str,"ATODGAIN=",9)) GAIN[n]=atof(str+9);
	 else if (!strncmp(str,"RNOISE  =",9)) RN[n]=atof(str+9);
	 else if (!strncmp(str,"EPOCH   =",9)) EPOCH[n]=atof(str+9);
	 else if (!strncmp(str,"EXPTIME =",9)) EXP[n]=atof(str+9);
	 else if (!strncmp(str,"EXP0    =",9)) EXP0[n]=atof(str+9);
	 else if (!strncmp(str,"RSDPFILL=",9)) DMIN[n]=atoi(str+9);
	 else if (!strncmp(str,"SATURATE=",9)) {
	    DMAX[n]=atoi(str+9);
	 }
	 else if (!strncmp(str,"TEMP    =",9)) TEMP[n]=atoi(str+9);
	 else if (!strncmp(str,"FILTNAM1=",9)) {
	    int i=0;
	    while ((i<19) && (str[i+11]!=32)) {
	       FILT[n][i]=str[i+11];
	       i++;
	    }
	 }
	 else if (!strncmp(str,"HSTPHOT =",9)) HPP=1;
      }
   }
   if (!HPP) ferr(f,"Not pre-processed by mask");
   printf("Reading FITS file %s:\n",fn);
   if (GAIN[n]==7) RN[n]=5/GAIN[n];
   else if (GAIN[n]==14) RN[n]=7.5/GAIN[n];
   else if (GAIN[n]==15) {
      GAIN[n]=14;
      RN[n]=7.5/GAIN[n];
   }
   else {
      printf("Illegal Gain\n");
      exit(1);
   }
   printf("Exp time=%f s, Gain=%f e/ADU, Noise=%f ADU\n",EXP[n],GAIN[n],RN[n]);
   printf("Saturation at %d, bad value=%d\n\n",DMAX[n],DMIN[n]);
   return f;
}

void readimg(FILE *f,int n) {
   int j,k;
   if (BITS[n]==-32) {
      ffread(data[n],800*800,4,f);
      for (j=0;j<800;j++) for (k=0;k<800;k++) data[n][j][k]=data[n][j][k]*scale[n]+zero[n];
   }
   else for (j=0;j<800;j++) for (k=0;k<800;k++) {
      long ii;
      fread(&ii,1,4,f);
      data[n][j][k]=ii*scale[n]+zero[n];
   }
}

FILE *writefits(char*fn) {
   FILE *f;
   char str[81];
   int line;

   if ((f=fopen(fn,"wb"))==NULL) {
      printf("Error on output file\n");
      exit(1);
   }
   fputs("SIMPLE  =                    T / Standard FITS format                           ",f);
   fputs("BITPIX  =                  -32 / 32 bit IEEE floating point numbers             ",f);
   fputs("NAXIS   =                    3 / Number of axes                                 ",f);
   fputs("NAXIS1  =                  800 /                                                ",f);
   fputs("NAXIS2  =                  800 /                                                ",f);
   fputs("NAXIS3  =                    4 /                                                ",f);
   fputs("EXTEND  =                    T / There may be standard extensions               ",f);
   sprintf(str,"ATODGAIN=       %14f / Analog to Digital Gain (Electrons/DN)          ",TGAIN);
   fputs(str,f);
   sprintf(str,"RNOISE  =       %14f / Readout Noise (DN)                             ",TRN);
   fputs(str,f);
   sprintf(str,"EPOCH   =     %16.8f / exposure average time (Modified Julian Date)   ",TEPOCH);
   fputs(str,f);
   sprintf(str,"EXPTIME =       %14f / exposure duration (seconds)--calculated        ",TEXP);
   fputs(str,f);
   sprintf(str,"EXP0    =       %14f / weighted average initial exposure time         ",TEXP0);
   fputs(str,f);
   sprintf(str,"RSDPFILL=       %14d / bad data fill value for calibrated images      ",TDMIN);
   fputs(str,f);
   sprintf(str,"SATURATE=       %14d / Data value at which saturation occurs          ",TDMAX);
   fputs(str,f);
   sprintf(str,"TEMP    =       %14d / Temperature (0=cold, 1=warm)                   ",TTEMP);
   fputs(str,f);
   sprintf(str,"FILTNAM1= \'%-18s\' / first filter name                              ",TFILT);
   fputs(str,f);
   fputs("HSTPHOT =                    T / Preprocessed by HSTphot/mask                   ",f);
   fputs("END                                                                             ",f);
   for (line=18;line<36;line++) fputs("                                                                                ",f);
   return f;
}

void setpars(void) {
   int i;
   float x,y;
   TGAIN=14;
   for (i=0;i<N;i++) if (GAIN[i]==7) TGAIN=7;
   for (i=0;i<N;i++) if (GAIN[i]!=TGAIN) {
      DMAX[i]*=2.006;
      DMIN[i]*=2.006;
      RN[i]*=2;
   }
   x=DMAX[0]/(float)EXP[0];
   y=DMIN[0]/(float)EXP[0];
   TDMIN=DMIN[0];
   TTEMP=TEMP[0];
   strcpy(TFILT,FILT[0]);
   TEXP=TRN=TEPOCH=TEXP0=0;
   for (i=0;i<N;i++) {
      TEPOCH+=EXP[i]*EPOCH[i];
      TEXP0+=EXP[i]*EXP0[i];
      TEXP+=EXP[i];
      TRN+=RN[i]*RN[i];
      if (x<DMAX[i]/(float)EXP[i]) x=DMAX[i]/(float)EXP[i];
      if (y>DMIN[i]/(float)EXP[i]) y=DMIN[i]/(float)EXP[i];
      if (strcmp(FILT[i],TFILT)) {
	 printf("Filters don't match\n");
	 exit(-1);
      }
      else if (TEMP[i]!=TTEMP) {
	 printf("Temperatures don't match\n");
	 exit(-1);
      }
   }
   TEPOCH/=TEXP;
   TEXP0/=TEXP;
   TRN=sqrt(TRN);
   TDMAX=(int)(x*TEXP+.5);
   TDMIN=(int)(y*TEXP-.5);
   for (i=0;i<N;i++) if (GAIN[i]!=TGAIN) RN[i]/=2;
}

void cleanit(int fld) {
   int i,j,k,l;
   double gr[4]={1.987,2.003,2.006,1.955};

   for (i=0;i<N;i++) if (GAIN[i]!=TGAIN) {
      for (j=0;j<800;j++) for (k=0;k<800;k++) if (data[i][j][k]*2.007<=DMIN[i]) data[i][j][k]=DMIN[i]-1;
      else if (data[i][j][k]*2.007>DMAX[i]*0.855) data[i][j][k]=DMAX[i]+1;
      else data[i][j][k]*=gr[fld];
   }
   for (k=0;k<800;k++) for (l=0;l<800;l++) {
      int sat=0,n=0;
      float c=0.,s=0.;

      for (i=0;i<N;i++) if (data[i][k][l]>DMIN[i] && data[i][k][l]<DMAX[i]*0.855) {
	 c+=data[i][k][l];
	 s+=EXP[i];
	 n++;
      }
      else if (data[i][k][l]>DMIN[i]) sat=1;
      if (n==0 && !sat) clean[k][l]=TDMIN;
      else if (n==0) clean[k][l]=TDMAX;
      else {
	 if (s>0) clean[k][l]=c/s*TEXP;
	 else clean[k][l]=TDMIN;
      }
   }
}

int main(int argc,char**argv) {
   int i,fld;
   FILE **fin,*fout;

   if (argc<3) {
      printf("Usage: %s <fits files> <fits output>\n",*argv);
      return 1;
   }
   N=argc-2;
   fin=(FILE**)calloc(N,sizeof(FILE*));
   DMAX=(int*)calloc(N,sizeof(int));
   DMIN=(int*)calloc(N,sizeof(int));
   BITS=(int*)calloc(N,sizeof(int));
   EXP=(double*)calloc(N,sizeof(double));
   EXP0=(double*)calloc(N,sizeof(double));
   GAIN=(double*)calloc(N,sizeof(double));
   RN=(double*)calloc(N,sizeof(double));
   EPOCH=(double*)calloc(N,sizeof(double));
   scale=(double*)calloc(N,sizeof(double));
   zero=(double*)calloc(N,sizeof(double));
   TEMP=(int*)calloc(N,sizeof(int));
   data=(imtype*)calloc(N,sizeof(imtype));
   FILT=(fntype*)calloc(N,sizeof(fntype));
   if (!fin || !DMAX || !DMIN || !BITS || !EXP || !EXP0 || !GAIN || !RN || !EPOCH || !scale || !zero || !TEMP || !data || !FILT) {
      printf("Memory allocation error\n");
      return 1;
   }
   for (i=0;i<N;i++) fin[i]=readfits(argv[i+1],i);
   setpars();
   fout=writefits(argv[argc-1]);
   for (fld=0;fld<4;fld++) {
      for (i=0;i<N;i++) readimg(fin[i],i);
      cleanit(fld);
      ffwrite(clean,800*800,4,fout);
   }
   printf("\n");
   for (i=0;i<N;i++) fclose(fin[i]);
   fclose(fout);
   return 0;
}
