#include <stdio.h>
#include <stdlib.h>
#include <math.h>
#include <string.h>
#include <unistd.h>
#ifdef LINUX
#include "linux.h"
#else
#include "sun.h"
#endif

typedef float dtype[800][800];

dtype data,sky;
int X,Y,Z,DMAX,DMIN,TEMP,nrej=0,fld;
double EXP,GAIN,RN,EPOCH,EXP0;
char FILT[20]={0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0};

void merr(void) {
   printf("Mem error\n");
   exit(0);
}

void readsky(char *fn) {
   int f;
   FILE *sf;

   if ((sf=fopen(fn,"rb"))==NULL) {
      printf("****Cannot open file %s\n",fn);
      exit(-1);
   }
   for (f=0;f<=fld;f++) ffread(sky,800*800,4,sf);
   fclose(sf);
}

void zap(void) {
   int k,l,m,n;

   for (k=0;k<800;k++) for (l=0;l<800;l++) if (data[k][l]>DMIN && (k==0 || data[k][l]>data[k-1][l]) && (l==0 || data[k][l]>data[k][l-1]) && (k==799 || data[k][l]>data[k+1][l]) && (l==799 || data[k][l]>data[k][l+1])) {
      float av=0,sd=0,slist[8];
      int Ns=0;
      for (m=k-1;m<=k+1;m++) for (n=l-1;n<=l+1;n++) if ((m!=k || n!=l) && m>=0 && n>=0 && m<800 && n<800 && data[m][n]>DMIN && data[m][n]<DMAX) av+=(slist[Ns++]=data[m][n]-sky[m][n]);
      if (Ns) {
	 av/=Ns;
	 for (m=0;m<Ns;m++) sd+=(slist[m]-av)*(slist[m]-av);
	 sd=sqrt(sd/Ns);
	 if (data[k][l]-sky[k][l]>av*10 && data[k][l]-sky[k][l]>av+sd*7) {
	    data[k][l]=DMIN;
	    nrej++;
	 }
      }
   }
   return;
}

void ferr(FILE *f,char *str) {
   printf("%s\n",str);
   fclose(f);
   exit(1);
}

int main(int argc,char**argv) {
   FILE *f,*fo;
   char str[321],fn[81];
   int MODE=0,line=0,tmp,HPP=0;
   if (argc!=2) {
      printf("Usage: %s <fits base>\n",*argv);
      return 1;
   }
   sprintf(str,"%s.fits",argv[1]);
   if ((f=fopen(str,"rb"))==NULL) {
      printf("%s not found\n",str);
      return 1;
   }
   while ((!feof(f)) && ((!MODE) || (line%36))) {
      fgets(str,81,f);
      line++;
      if (!MODE) {
	 if (!strncmp(str,"END     ",8)) MODE=1;
	 else if ((!strncmp(str,"SIMPLE  =",9)) && (str[29]!='T')) ferr(f,"Illegal format type");
	 else if ((!strncmp(str,"BITPIX  =",9)) && (strncmp(str+27,"-32",3))) ferr(f,"Illegal bits per pixel");
	 else if ((!strncmp(str,"NAXIS   =",9)) && (strncmp(str+27,"  3",3))) ferr(f,"Illegal number of axes");
	 else if (!strncmp(str,"NAXIS",5)) {
	    tmp=atoi(str+9);
	    if (str[5]=='1') X=tmp;
	    else if (str[5]=='2') Y=tmp;
	    else if (str[5]=='3') Z=tmp;
	 }
	 else if (!strncmp(str,"ATODGAIN=",9)) GAIN=atof(str+9);
	 else if (!strncmp(str,"RNOISE  =",9)) RN=atof(str+9);
	 else if (!strncmp(str,"EPOCH   =",9)) EPOCH=atof(str+9);
	 else if (!strncmp(str,"EXPTIME =",9)) EXP=atof(str+9);
	 else if (!strncmp(str,"EXP0    =",9)) EXP0=atof(str+9);
	 else if (!strncmp(str,"RSDPFILL=",9)) DMIN=atoi(str+9);
	 else if (!strncmp(str,"SATURATE=",9)) DMAX=atoi(str+9);
	 else if (!strncmp(str,"TEMP    =",9)) TEMP=atoi(str+9);
	 else if (!strncmp(str,"FILTNAM1=",9)) {
	    int i=0;
	    while ((i<19) && (str[i+11]!=32)) {
	       FILT[i]=str[i+11];
	       i++;
	    }
	 }
	 else if (!strncmp(str,"HSTPHOT =",9)) HPP=1;
      }
   }
   if (!HPP) ferr(f,"Not pre-processed by mask");
   if ((X!=800) || (Y!=800) || (Z!=4)) ferr(f,"Illegal axis size");
   sprintf(fn,"temp.%d",(int)getpid());
   if ((fo=fopen(fn,"wb"))==NULL) {
      printf("Error on output file\n");
      exit(1);
   }
   fputs("SIMPLE  =                    T / Standard FITS format                           ",fo);
   fputs("BITPIX  =                  -32 / 32 bit IEEE floating point numbers             ",fo);
   fputs("NAXIS   =                    3 / Number of axes                                 ",fo);
   fputs("NAXIS1  =                  800 /                                                ",fo);
   fputs("NAXIS2  =                  800 /                                                ",fo);
   fputs("NAXIS3  =                    4 /                                                ",fo);
   fputs("EXTEND  =                    T / There may be standard extensions               ",fo);
   sprintf(str,"ATODGAIN=       %14f / Analog to Digital Gain (Electrons/DN)          ",GAIN);
   fputs(str,fo);
   sprintf(str,"RNOISE  =       %14f / Readout Noise (DN)                             ",RN);
   fputs(str,fo);
   sprintf(str,"EPOCH   =     %16.8f / exposure average time (Modified Julian Date)   ",EPOCH);
   fputs(str,fo);
   sprintf(str,"EXPTIME =       %14f / exposure duration (seconds)--calculated        ",EXP);
   fputs(str,fo);
   sprintf(str,"EXP0    =       %14f / weighted average initial exposure time         ",EXP0);
   fputs(str,fo);
   sprintf(str,"RSDPFILL=       %14d / bad data fill value for calibrated images      ",DMIN);
   fputs(str,fo);
   sprintf(str,"SATURATE=       %14d / Data value at which saturation occurs          ",DMAX);
   fputs(str,fo);
   sprintf(str,"TEMP    =       %14d / Temperature (0=cold, 1=warm)                   ",TEMP);
   fputs(str,fo);
   sprintf(str,"FILTNAM1= \'%-18s\' / first filter name                              ",FILT);
   fputs(str,fo);
   fputs("HSTPHOT =                    T / Preprocessed by HSTphot/mask                   ",fo);
   fputs("END                                                                             ",fo);
   for (line=18;line<36;line++) fputs("                                                                                ",fo);
   sprintf(str,"%s.sky",argv[1]);
   for (fld=0;fld<4;fld++) {
      ffread(data,800*800,4,f);
      readsky(str);
      zap();
      ffwrite(data,800*800,4,fo);
   }
   fclose(f);
   fclose(fo);
   printf("Rejected %d pixels\n",nrej);
   //sprintf(str,"mv %s.fits %s.old.fits ; mv %s %s.fits",argv[1],argv[1],fn,argv[1]);
   sprintf(str,"mv %s %s.fits",fn,argv[1]);
   if ((f=popen(str,"w"))!=NULL) pclose(f);
   else printf("Cannot replace old fits file\n");
   return 0;
}
