#include <stdio.h>
#include <stdlib.h>
#include <math.h>
#include <string.h>
#include <time.h>
#ifdef LINUX
#include "linux.h"
#else
#include "sun.h"
#endif
#include "filters.h"

#define NMAX 65000
#define NINDX 2

typedef float dtype[800][800];
typedef char rantype[800];
typedef struct{
   int flag,type;
   float x,y,s,ss,chi,sh,sky;
   float *is,*iss,*icm;
} stype;
typedef float ****psftype;
typedef psftype spsftype[8][8];
typedef float **dpsftype;
typedef dpsftype sdpsftype[8][8];
typedef char fntype[20];
typedef float dpostype[2];
typedef float rtype[800];
typedef float sigtype[2][4];
typedef int sigftype[4];

//parameters
int XMIN,XMAX,YMIN,YMAX,XMIN2=50,XMAX2=800,YMIN2=50,YMAX2=800,MMIN=0,MMAX=3,OPTS,WARM=0;
float fmult[4],flim[2][2];
int ffilt[2],NO_SKY=0;

//input file info
int *DMAX,*DMIN,*TEMP,*filt,*IGN,Nimg,Timg;
float *EXP,*GAIN,*RN,*YEAR,*EXP0,*apcor;
double *EPOCH;
fntype *FILT;
dpostype *dpos,*dpos0;
dtype *data,*sky,*res;
sigtype *sigpsf;
sigftype *sigfpsf;

//PSF info
spsftype *psflib;
sdpsftype *dpsf;
float **tpsf,**ttpsf;
rtype *rows;

//photometry variables
stype *stars;
unsigned short indx[800][800][NINDX+1];
rantype ran[800];
dtype snmap;
float rmin[4],rap[4],****cwt[4],***poff,pixmap[80][80][2],exmap[80][80],*gs_rval;
float outsig,findsig,minsig;
int rphot[4],rpsf[4],sub[4],***circ[2][4],Nstars,fld,MaxIT=25,*gs_x,*gs_y;
float apsize[4]={10.979,5.020,5.021,5.018};
float smapsize[4]={3,2,2,2};
float dnoise[4]={0.0361,0.0625,0.0625,0.0625};
float corr[2][4]={{0.5066,0.5017,0.4985,0.5122},{1.0066,1.0050,1.0000,1.0013}};
float smapcor0[4]={-0.016,-0.019,-0.021,-0.019};
float smappsfm[4]={-0.005,-0.019,-0.070,-0.017};
float smapprior=800.;  //prior = 1/sigma from apcor vs. PSF fit
float apcor0[4]={-0.007,-0.006,-0.074,-0.026};
float appsfm[4]={0.238,0.734,1.060,0.763};
float apprior=50.;  //prior = 1/sigma from apcor vs. PSF fit

int DEBUG=0;
int NOSUBTRACT=0;

inline double SQR(double x) {return x*x;}

inline void merr(void) {
   printf("****Mem error\n");
   exit(-1);
}

inline int ppixOK(int i,int x,int y) {
   return (x>=XMIN && x<XMAX && y>=YMIN && y<YMAX && data[i][x][y]>DMIN[i] && data[i][x][y]<DMAX[i]);
}

inline int ppixOK2(int i,int x,int y) {
   return (x>=50 && x<799 && y>=50 && y<799 && data[i][x][y]>DMIN[i] && data[i][x][y]<DMAX[i]);
}

inline int pixOK(int i,int x,int y) {
   return (data[i][x][y]>DMIN[i] && data[i][x][y]<DMAX[i]);
}

inline void setran(rantype* m) {
   memset(m[0],0,640000);
}

inline void setindx(void) {
   memset(indx[0][0],0,1280000*(NINDX+1));
}

void getpars(void) {
   int mx,my,x,y,f,rad,i;
   float *ptr;

   DMAX=(int*)calloc(Timg,sizeof(int));
   DMIN=(int*)calloc(Timg,sizeof(int));
   TEMP=(int*)calloc(Timg,sizeof(int));
   filt=(int*)calloc(Timg,sizeof(int));
   IGN=(int*)calloc(Timg,sizeof(int));
   poff=(float***)calloc(Timg,sizeof(float**));
   EXP=(float*)calloc(Timg,sizeof(float));
   GAIN=(float*)calloc(Timg,sizeof(float));
   RN=(float*)calloc(Timg,sizeof(float));
   YEAR=(float*)calloc(Timg,sizeof(float));
   EXP0=(float*)calloc(Timg,sizeof(float));
   apcor=(float*)calloc(Timg,sizeof(float));
   EPOCH=(double*)calloc(Timg,sizeof(double));
   dpos=(dpostype*)calloc(Timg,sizeof(dpostype));
   dpos0=(dpostype*)calloc(Timg,sizeof(dpostype));
   FILT=(fntype*)calloc(Timg,sizeof(fntype));
   rows=(rtype*)calloc(Timg,sizeof(rtype));
   psflib=(spsftype*)calloc(Timg,sizeof(spsftype));
   dpsf=(sdpsftype*)calloc(Timg,sizeof(sdpsftype));
   data=(dtype*)calloc(Timg,sizeof(dtype));
   sky=(dtype*)calloc(Timg,sizeof(dtype));
   res=(dtype*)calloc(Timg,sizeof(dtype));
   gs_rval=(float*)calloc(Timg,sizeof(float));
   gs_x=(int*)calloc(Timg,sizeof(int));
   gs_y=(int*)calloc(Timg,sizeof(int));
   if (!DMAX || !DMIN || !TEMP || !filt || !IGN || !poff || !EXP || !GAIN || !RN || !YEAR || !EXP0 || !apcor || !EPOCH || !dpos || !dpos0 || !FILT || !rows || !data || !sky || !res || !psflib || !dpsf || !gs_rval || !gs_x || !gs_y) merr();
   if (OPTS&256) {
      sigpsf=(sigtype*)calloc(Timg,sizeof(sigtype));
      sigfpsf=(sigftype*)calloc(Timg,sizeof(sigftype));
      if (!sigpsf || !sigfpsf) merr();
   }
   rpsf[1]=rpsf[2]=rpsf[3]=(rpsf[0]*2+1)/3;
   sub[1]=sub[2]=sub[3]=sub[0]*2;
   rap[1]=rap[2]=rap[3]=rap[0]*2./3.;
   for (f=0;f<4;f++) rphot[f]=(int)(rap[f]+0.99);
   rmin[1]=rmin[2]=rmin[3]=rmin[0]*2./3.;
   for (f=0;f<4;f++) {
      rad=(sub[f]+2)/2;
      circ[0][f]=(int***)calloc(rad*2+1,sizeof(int**));
      circ[1][f]=(int***)calloc(rad*2+1,sizeof(int**));
      cwt[f]=(float****)calloc(rad*2+1,sizeof(float***));
      if (!circ[0][f] || !circ[1][f] || !cwt[f]) merr();
      circ[0][f]+=rad;
      circ[1][f]+=rad;
      cwt[f]+=rad;
      for (mx=-rad;mx<=rad;mx++) {
	 circ[0][f][mx]=(int**)calloc(rad*2+1,sizeof(int*));
	 circ[1][f][mx]=(int**)calloc(rad*2+1,sizeof(int*));
	 cwt[f][mx]=(float***)calloc(rad*2+1,sizeof(float**));
	 if (!circ[0][f][mx] || !circ[1][f][mx] || !cwt[f][mx]) merr();
	 circ[0][f][mx]+=rad;
	 circ[1][f][mx]+=rad;
	 cwt[f][mx]+=rad;
	 for (my=-rad;my<=rad;my++) {
	    circ[0][f][mx][my]=(int*)calloc(rphot[f]*2+1,sizeof(int));
	    circ[1][f][mx][my]=(int*)calloc(rphot[f]*2+1,sizeof(int));
	    cwt[f][mx][my]=(float**)calloc(rphot[f]*2+1,sizeof(float*));
	    if (!circ[0][f][mx][my] || !circ[1][f][mx][my] || !cwt[f][mx][my]) merr();
	    circ[0][f][mx][my]+=rphot[f];
	    circ[1][f][mx][my]+=rphot[f];
	    cwt[f][mx][my]+=rphot[f];
	    for (x=-rphot[f];x<=rphot[f];x++) {
	       circ[0][f][mx][my][x]=50;
	       circ[1][f][mx][my][x]=-50;
	       cwt[f][mx][my][x]=(float*)calloc(rphot[f]*2+1,sizeof(float));
	       if (!cwt[f][mx][my][x]) merr();
	       cwt[f][mx][my][x]+=rphot[f];
	       for (y=-rphot[f];y<=rphot[f];y++) {
		  cwt[f][mx][my][x][y]=rap[f]+.465-sqrt((x-mx/(float)sub[f])*(x-mx/(float)sub[f])+(y-my/(float)sub[f])*(y-my/(float)sub[f]));
		  if (cwt[f][mx][my][x][y]<0) cwt[f][mx][my][x][y]=0;
		  else if (cwt[f][mx][my][x][y]>1) cwt[f][mx][my][x][y]=1;
		  if (cwt[f][mx][my][x][y]>0 && y<circ[0][f][mx][my][x]) circ[0][f][mx][my][x]=y;
		  if (cwt[f][mx][my][x][y]>0 && y>circ[1][f][mx][my][x]) circ[1][f][mx][my][x]=y;
	       }
	    }
	 }
      }
   }
   stars=(stype*)calloc(NMAX,sizeof(stype));
   ptr=(float*)calloc(Timg*3*NMAX,sizeof(float));
   if (!stars || !ptr) merr();
   for (i=0;i<NMAX;i++) {
      stars[i].is=ptr+(i*3)*Timg;
      stars[i].iss=ptr+(i*3+1)*Timg;
      stars[i].icm=ptr+(i*3+2)*Timg;
   }
}

//uses Holtzman et at (1995), PASP 107, 156
double CC[10][4]=
{{3.54356E+02,-8.12003E+02,-8.07068E+02,7.72904E+02},
 {1.00021E+00,1.95172E-02,-2.18757E+00,1.37619E-02},
 {9.79758E-04,-2.18805E+00,-8.51284E-03,2.18899E+00},
 {9.84222E-08,-1.51015E-06,-3.21458E-07,1.33326E-06},
 {-6.31327E-09,5.06693E-06,2.77459E-06,-4.00828E-06},
 {-7.19983E-07,1.04063E-06,-2.06874E-06,-5.71282E-07},
 {-3.73922E-08,-4.55094E-10,7.39223E-08,-2.00520E-09},
 {6.65101E-10,7.50249E-08,-1.18833E-09,-7.86673E-08},
 {-3.51470E-08,-1.51652E-09,7.73056E-08,-3.37491E-09},
 {-2.55003E-09,7.34568E-08,-3.72971E-10,-7.81490E-08}};
double DD[10][4]=
{{3.43646E+02,7.66592E+02,-7.71489E+02,-7.74638E+02},
 {1.00544E-03,2.18607E+00,7.40319E-03,-2.18688E+00},
 {9.99786E-01,2.02006E-02,-2.18554E+00,1.29831E-02},
 {-5.80870E-07,-3.79469E-06,-1.49841E-06,1.30814E-06},
 {-5.07221E-07,-2.99518E-06,2.96650E-06,2.01760E-06},
 {3.41574E-07,1.05789E-07,-2.02756E-07,-8.47656E-07},
 {-1.08091E-09,-7.47857E-08,-1.45720E-09,7.70408E-08},
 {-3.42070E-08,-1.67666E-09,7.72421E-08,-1.68514E-09},
 {1.17819E-09,-7.55302E-08,8.52633E-10,7.75257E-08},
 {-4.48966E-08,3.02579E-10,7.68023E-08,-3.69210E-10}};

void transformpix(int c,float x,float y,float*xc,float*yc) {
   double a,b;
   a=CC[1][c]*x+CC[2][c]*y+CC[3][c]*x*x+CC[4][c]*x*y+CC[5][c]*y*y+CC[6][c]*x*x*x+CC[7][c]*x*x*y+CC[8][c]*x*y*y+CC[9][c]*y*y*y;
   b=DD[1][c]*x+DD[2][c]*y+DD[3][c]*x*x+DD[4][c]*x*y+DD[5][c]*y*y+DD[6][c]*x*x*x+DD[7][c]*x*x*y+DD[8][c]*x*y*y+DD[9][c]*y*y*y;
   switch(c) {
   case 0:
      *xc=a/CC[1][0];
      *yc=b/DD[2][0];
      break;
   case 1:
      *xc=b/DD[1][1];
      *yc=a/CC[2][1];
      break;
   case 2:
      *xc=a/CC[1][2];
      *yc=b/DD[2][2];
      break;
   case 3:
      *xc=b/DD[1][3];
      *yc=a/CC[2][3];
      break;
   }
}

float pixdx(int c,int x,int y) {
   float dx=0,t,q;
   transformpix(c,x+.5,y,&dx,&q);
   transformpix(c,x-.5,y,&t,&q);
   return dx-t;
}

float pixdy(int c,int x,int y) {
   float dy=0,t,q;
   transformpix(c,x,y+.5,&q,&dy);
   transformpix(c,x,y-.5,&q,&t);
   return dy-t;
}

void initpixmap(void) {
   int x,y;
   double xx,yy;
   double ex[4][6]={
      {-0.019279,0.009223,0.010014,-0.024078,-0.025570,-0.016572},
      {0.026332,-0.004992,-0.004097,-0.040013,0.011779,-0.037071},
      {0.123705,-0.008796,0.056164,-0.021400,0.004539,0.029338},
      {0.037117,0.028335,0.021520,-0.007680,0.016204,-0.059367}
   };

   for (x=5;x<800;x+=10) for (y=5;y<800;y+=10) {
      pixmap[y/10][x/10][0]=pixdy(fld,x-400,y-400);
      pixmap[y/10][x/10][1]=pixdx(fld,x-400,y-400);
      xx=x/400.-1;
      yy=y/400.-1;
      exmap[y/10][x/10]=ex[fld][0]+ex[fld][1]*xx+ex[fld][2]*yy+ex[fld][3]*xx*xx+ex[fld][4]*xx*yy+ex[fld][5]*yy*yy;
   }
   //comment this (or delete) to re-enable PSF adjustments;
   //double av=0; int nn=0; for (x=55;x<800;x+=10) for (y=55;y<800;y+=10) {av+=exmap[y/10][x/10]; nn++;} av/=nn; for (x=5;x<800;x+=10) for (y=5;y<800;y+=10) exmap[y/10][x/10]=av;
   //double av=0,sig=0; int nn=0; for (x=55;x<800;x+=10) for (y=55;y<800;y+=10) {av+=exmap[y/10][x/10]; sig+=exmap[y/10][x/10]*exmap[y/10][x/10]; nn++;} av/=nn; sig/=nn; printf("%f %f\n",av,sqrt(sig-av*av));
   return;
}

float rowsize(int img,int x) {
   //Ron Gilliland, raw data by private communication
   int Nrows=32,rows[32]={64,98,132,166,200,201,234,235,268,269,302,303,337,371,405,439,440,473,474,507,508,541,542,576,610,644,678,712,746,747,780,781};
   float col0[32]={0.9789,0.9878,0.9675,0.9696,0.9829,0.9985,0.9806,0.9796,0.989,0.9803,0.9985,0.9855,0.9683,0.9849,0.9648,0.982,0.9978,0.9813,0.9851,0.9819,0.977,0.9979,0.9834,0.9674,0.9842,0.9667,0.9697,0.9801,0.9807,0.9799,0.9872,0.98};
   float dcol[32]={-3.42557e-06,-3.83428e-05,-3.92468e-06,-5.96548e-06,-3.72032e-05,-1.58276e-05,9.18357e-06,7.45086e-06,-2.89753e-07,-5.09053e-06,-1.21806e-05,-3.36296e-05,-2.00527e-06,-4.03303e-05,1.91426e-06,-2.69071e-05,-1.06506e-05,1.08298e-05,4.22073e-06,9.60101e-06,1.04708e-05,-1.01982e-05,-3.10483e-05,-1.51857e-06,-3.55187e-05,-4.69483e-06,-5.38823e-07,-3.14124e-05,1.22734e-05,3.24158e-06,1.91252e-07,-1.48993e-06};
   float lam0=568.104;
   int i;

   for (i=0;i<Nrows;i++) if (x==rows[i]) {
      float p;
      p=col0[i]+(atoi(FILT[img]+1)-lam0)*dcol[i];
      if (p>1.) return 1.;
      if (p<0.) return 0.;
      return p;
   }
   return 1.;
}

void initrows(void) {
   int img,y;
   for (img=0;img<Timg;img++) for (y=0;y<800;y++) rows[img][y]=rowsize(img,y);
}

inline int peak2(int img,int x,int y) {
   if (x>0 && data[img][x-1][y]>data[img][x][y] && data[img][x-1][y]<DMAX[img]) return 0;
   if (x<XMAX-1 && data[img][x+1][y]>data[img][x][y] && data[img][x+1][y]<DMAX[img]) return 0;
   if (y>0 && data[img][x][y-1]>data[img][x][y] && data[img][x][y-1]<DMAX[img]) return 0;
   if (y<YMAX-1 && data[img][x][y+1]>data[img][x][y] && data[img][x][y+1]<DMAX[img]) return 0;
   return 1;
}

void cleansat(int img) {
   int x,y,c=1;
   rantype* cran;
   float DDMAX=DMAX[img]*0.855;

   cran=(rantype*)calloc(800,sizeof(rantype));
   if (!cran) merr();
   for (x=0;x<800;x++) if (x>=XMIN-20 && x<XMAX+20) for (y=0;y<800;y++) if (y>=YMIN-20 && y<YMAX+20) {
      if (data[img][x][y]>=DDMAX) {
	 cran[x][y]=1;
	 data[img][x][y]=DMAX[img]*0.99;
      }
      else cran[x][y]=0;
   }
   while (c) {
      c=0;
      memcpy(ran,cran,640000);
      setran(cran);
      for (x=0;x<800;x++) if (x>=XMIN-20 && x<XMAX+20) for (y=0;y<800;y++) if (y>=YMIN-20 && y<YMAX+20 && pixOK(img,x,y) && ran[x][y] && peak2(img,x,y)) {
	 data[img][x][y]=DMAX[img];
	 if (x>0) cran[x-1][y]=1;
	 if (x<799) cran[x+1][y]=1;
	 if (y>0) cran[x][y-1]=1;
	 if (y<799) cran[x][y+1]=1;
	 c=1;
      }
   }
   setran(ran);
   free(cran);
}

void readsky(int img,char *fn) {
   int f,x,y;
   float z;
   FILE *sf;

   if ((sf=fopen(fn,"rb"))==NULL) {
      printf("****Cannot open file %s\n",fn);
      exit(-1);
   }
   for (f=0;f<=fld;f++) ffread(sky[img],800*800,4,sf);
   for (x=0;x<800;x++) for (y=0;y<800;y++) {
      z=pixmap[x/10][y/10][0]*pixmap[x/10][y/10][1]*rows[img][x];
      sky[img][x][y]*=z;
      if (pixOK(img,x,y)) {
	 data[img][x][y]*=z;
	 res[img][x][y]=data[img][x][y]-sky[img][x][y];
      }
      else res[img][x][y]=data[img][x][y];
   }
   fclose(sf);
}

inline float noise(int img,int x,int y) {
   return RN[img]+(((sky[img][x][y]>0)?sky[img][x][y]:0)+((data[img][x][y]>sky[img][x][y])?data[img][x][y]-sky[img][x][y]:0))/GAIN[img];
}

void getspsf(int img,int f) {
   int x,y,x2,y2,x1,y1,r,rad,N;
   float r2,av;

   if (!(OPTS&2)) return;
   r=rpsf[f];
   if (r>rphot[f]+2) r=rphot[f]+2;
   rad=(sub[f]+2)/2;
   for (x=0;x<8;x++) for (y=0;y<8;y++) for (x2=-rad;x2<=rad;x2++) for (y2=-rad;y2<=rad;y2++) {
      N=0;
      av=0.;
      for (x1=-r;x1<=r;x1++) for (y1=-r;y1<=r;y1++) {
	 r2=sqrt(x1*x1+y1*y1);
	 if (r2>=rphot[f]+1 && r2<=r) {
	    av+=psflib[img][x][y][x2][y2][x1][y1];
	    N++;
	 }
      }
      av/=N;
      for (x1=-rpsf[f];x1<=rpsf[f];x1++) for (y1=-rpsf[f];y1<=rpsf[f];y1++) psflib[img][x][y][x2][y2][x1][y1]-=av;
      dpsf[img][x][y][x2][y2]+=av;
   }
}

double psffunc(int t,double r2) {
   if (t==0) return exp(-r2);
   if (t==1) return 1./(1.+r2);
   if (t==2) return pow(1.+r2,-1.5);
   if (t==3) {double x=1.+r2; return 1./x/x;}
   if (t==4) return pow(1.+r2,-2.5);
   printf("PSF func error!!\n");
   exit(0);
   return 0.;
}

void getpsf(int f) {
   FILE *pf;
   char fn[80];
   int tf,x,y,x1,y1,rad,px,py,img,N;
   static int last=-1;
   float c[101];

   rad=(sub[f]+2)/2;
   if (OPTS&4) printf("Reading PSF residuals from default library\n");
   for (img=0;img<Timg;img++) {
      int off1,off2,off3,off4;
      float *ptr1,**ptr2,***ptr3,****ptr4,*dptr3,**dptr4;
      if (last>=0) {
	 int r2;
	 r2=(sub[last]+2)/2;
	 free(psflib[img][0][0][-r2][-r2][-rpsf[last]]-rpsf[last]);
	 free(psflib[img][0][0][-r2][-r2]-rpsf[last]);
	 free(psflib[img][0][0][-r2]-r2);
	 free(psflib[img][0][0]-r2);
	 free(dpsf[img][0][0][-r2]-r2);
	 free(dpsf[img][0][0]-r2);
      }
      sprintf(fn,"%s/psfs/%s.%d.%d.psf",BASEDIR,filters[filt[img]].suffix,rpsf[0],sub[0]);
      if ((pf=fopen(fn,"rb"))==NULL) {
	 printf("****File error: %s\n",fn);
	 exit(-1);
      }
      for (tf=0;tf<f;tf++) {
	 float buffer[200];
	 int r2;
	 r2=((sub[tf]+2)/2)*2+1;
	 N=64*r2*r2;
	 r2=rpsf[tf]*2+1;
	 for (x=0;x<N;x++) fread(buffer,sizeof(float),r2*r2,pf);
      }
      ptr1=(float*)calloc(64*(2*rad+1)*(2*rad+1)*(2*rpsf[f]+1)*(2*rpsf[f]+1),sizeof(float));
      ptr2=(float**)calloc(64*(2*rad+1)*(2*rad+1)*(2*rpsf[f]+1),sizeof(float*));
      ptr3=(float***)calloc(64*(2*rad+1)*(2*rad+1),sizeof(float**));
      ptr4=(float****)calloc(64*(2*rad+1),sizeof(float***));
      dptr3=(float*)calloc(64*(2*rad+1)*(2*rad+1),sizeof(float));
      dptr4=(float**)calloc(64*(2*rad+1),sizeof(float*));
      if (!ptr1 || !ptr2 || !ptr3 || !ptr4 || !dptr3 || !dptr4) merr();
      for (px=0;px<8;px++) for (py=0;py<8;py++) {
	 off4=(px*8+py)*(2*rad+1);
	 psflib[img][px][py]=ptr4+off4+rad;
	 dpsf[img][px][py]=dptr4+off4+rad;
	 for (x=-rad;x<=rad;x++) {
	    off3=(off4+x+rad)*(2*rad+1);
	    psflib[img][px][py][x]=ptr3+off3+rad;
	    dpsf[img][px][py][x]=dptr3+off3+rad;
	    for (y=-rad;y<=rad;y++) {
	       off2=(off3+y+rad)*(2*rpsf[f]+1);
	       psflib[img][px][py][x][y]=ptr2+off2+rpsf[f];
	       for (x1=-rpsf[f];x1<=rpsf[f];x1++) {
		  off1=(off2+x1+rpsf[f])*(2*rpsf[f]+1);
		  psflib[img][px][py][x][y][x1]=ptr1+off1+rpsf[f];
	       }
	       for (x1=-rpsf[f];x1<=rpsf[f];x1++) ffread(psflib[img][px][py][x][y][x1]-rpsf[f],2*rpsf[f]+1,sizeof(float),pf);
	    }
	 }
      }
      fclose(pf);
      if (OPTS&4) {
	 sprintf(fn,"%s/psfs/%s.%d.%d.dpsf",BASEDIR,filters[filt[img]].suffix,rpsf[0],sub[0]);
	 if ((pf=fopen(fn,"rb"))==NULL) printf("image %d: No default PSF residual for %s\n",img,filters[filt[img]].name);
	 else {
	    int tf;
	    for (tf=0;tf<f;tf++) {
	       rad=(sub[tf]+2)/2;
	       for (x=-rpsf[tf];x<=rpsf[tf];x++) ffread(c,sizeof(float),2*rpsf[tf]+1,pf);
	    }
	    rad=(sub[tf]+2)/2;
	    for (x=-rpsf[tf];x<=rpsf[tf];x++) {
	       ffread(c,sizeof(float),2*rpsf[tf]+1,pf);
	       for (y=-rpsf[tf];y<=rpsf[tf];y++) for (px=0;px<8;px++) for (py=0;py<8;py++) for (x1=-rad;x1<=rad;x1++) for (y1=-rad;y1<=rad;y1++) psflib[img][px][py][x1][y1][x][y]+=c[y+rpsf[tf]];
	       if (x==0) printf("image %d: %f\n",img,c[rpsf[tf]]);
	    }
	    fclose(pf);
	 }
      }
      if (OPTS&256 && img<Nimg) {
	 double wmlt[5]={2.7725887,4.0000000,2.3496042,1.6568542,1.2780316};
	 float w,w0,r2,r2t,t=0,t0=0;

	 w0=wmlt[sigfpsf[img][fld]]/sigpsf[img][0][fld]/sigpsf[img][0][fld];
	 w=wmlt[sigfpsf[img][fld]]/sigpsf[img][1][fld]/sigpsf[img][1][fld];
	 for (x=-rpsf[tf];x<=rpsf[tf];x++) for (y=-rpsf[tf];y<=rpsf[tf];y++) {
	    r2=x*x+y*y;
	    t0+=psffunc(sigfpsf[img][fld],r2*w0);
	    t+=psffunc(sigfpsf[img][fld],r2*w);
	 }
	 for (x=-rpsf[tf];x<=rpsf[tf];x++) for (y=-rpsf[tf];y<=rpsf[tf];y++) {
	    r2=x*x+y*y;
	    r2=psffunc(sigfpsf[img][fld],r2*w)/t-psffunc(sigfpsf[img][fld],r2*w0)/t0;
	    for (px=0;px<8;px++) for (py=0;py<8;py++) for (x1=-rad;x1<=rad;x1++) for (y1=-rad;y1<=rad;y1++) {
	       if (psflib[img][px][py][x1][y1][x][y]+r2<0) r2t=-psflib[img][px][py][x1][y1][x][y];
	       else r2t=r2;
	       psflib[img][px][py][x1][y1][x][y]+=r2t;
	       poff[img][x][y]+=r2t/64./(2*rad+1)/(2*rad+1);
	    }
	 }
      }
      getspsf(img,f);
   }
   last=f;
}

void addindx(int x,int y,int i) {
   if (indx[x][y][0]>=NINDX) {
      printf("****Index overflow\n");
      exit(-1);
   }
   indx[x][y][++indx[x][y][0]]=i;
}

void subindx(int x,int y,int i) {
   int j;
   if (indx[x][y][0]==0) {
      printf("****Index underflow\n");
      exit(-1);
   }
   for (j=1;j<=indx[x][y][0];j++) if (indx[x][y][j]==i) {
      indx[x][y][j]=indx[x][y][indx[x][y][0]--];
      return;
   }
   printf("****Subindx not found\n");
   exit(-1);
}

void repindx(int x,int y,int i,int ii) {
   int j;
   for (j=1;j<=indx[x][y][0];j++) if (indx[x][y][j]==i) {
      indx[x][y][j]=ii;
      return;
   }
   printf("****Repindx not found %d,%d\n",x,y);
   exit(-1);
}

void calc1psf(int img,int r,float x,float y,int*fx,int*fy) {
   int xx,yy;
   static int lastr=-1;
   static float *xoff;
   int x0,y0,xrg,yrg;
   float z,zz,mx,my,imx,imy;

   if (r>lastr) {
      if (lastr>=0) {
	 for (xx=-lastr;xx<=lastr;xx++) {
	    free(tpsf[xx]-lastr);
	    free(ttpsf[xx]-lastr);
	 }
	 free(tpsf-lastr);
	 free(ttpsf-lastr);
	 free(xoff-lastr);
      }
      tpsf=(float**)calloc(sizeof(float*),r*2+1);
      ttpsf=(float**)calloc(sizeof(float*),r*2+1);
      xoff=(float*)calloc(sizeof(float),r*2+2);
      if (!tpsf || !ttpsf || !xoff) merr();
      tpsf+=r;
      ttpsf+=r;
      xoff+=r;
      for (xx=-r;xx<=r;xx++) {
	 tpsf[xx]=(float*)calloc(sizeof(float),r*2+1);
	 ttpsf[xx]=(float*)calloc(sizeof(float),r*2+1);
	 if (!tpsf[xx] || !ttpsf[xx]) merr();
	 tpsf[xx]+=r;
	 ttpsf[xx]+=r;
      }
      lastr=r;
   }
   if (x<0) x=0;
   if (x>799.9) x=799.9;
   if (y<0) y=0;
   if (y>799.9) y=799.9;
   mx=(x-(int)x-0.5)*sub[fld];
   x0=(int)(mx+sub[fld])-sub[fld];
   if (fx!=NULL) *fx=(int)(mx+10.5+sub[fld])-sub[fld]-10;
   mx-=x0;
   imx=1-mx;
   my=(y-(int)y-0.5)*sub[fld];
   y0=(int)(my+sub[fld])-sub[fld];
   if (fy!=NULL) *fy=(int)(my+10.5+sub[fld])-sub[fld]-10;
   my-=y0;
   imy=1-my;
   xrg=(int)x/100; yrg=(int)y/100;
   for (xx=-r;xx<=r;xx++) for (yy=-r;yy<=r;yy++) tpsf[xx][yy]=psflib[img][yrg][xrg][x0][y0][xx][yy]*imx*imy+psflib[img][yrg][xrg][x0+1][y0][xx][yy]*mx*imy+psflib[img][yrg][xrg][x0][y0+1][xx][yy]*imx*my+psflib[img][yrg][xrg][x0+1][y0+1][xx][yy]*mx*my;
   zz=y-(int)y-0.5;
   z=pixmap[(int)x/10][(int)y/10][1]+exmap[(int)x/10][(int)y/10];
   xoff[1]=(1-z)*(0.5-zz);
   xoff[0]=-(1-z)*(0.5+zz);
   for (xx=2;xx<=r+1;xx++) xoff[xx]=xoff[xx-1]+(1-z);
   for (xx=-1;xx>=-r;xx--) xoff[xx]=xoff[xx+1]-(1-z);
   for (xx=-r;xx<=r;xx++) for (yy=-r;yy<=r;yy++) ttpsf[xx][yy]=0;
   for (xx=-r;xx<=r;xx++) for (yy=-r;yy<=r;yy++) {
      float q;
      ttpsf[xx][yy]+=(q=tpsf[xx][yy]);
      if (xoff[yy]<0) {
	 if (yy>-r) {
	    ttpsf[xx][yy-1]+=0.5*(q+tpsf[xx][yy-1])*(-xoff[yy]);
	    ttpsf[xx][yy]-=0.5*(q+tpsf[xx][yy-1])*(-xoff[yy]);
	 }
	 else ttpsf[xx][yy]-=0.75*q*(-xoff[yy]);
      }
      if (xoff[yy+1]>0) {
	 if (yy<r) {
	    ttpsf[xx][yy+1]+=0.5*(q+tpsf[xx][yy+1])*xoff[yy+1];
	    ttpsf[xx][yy]-=0.5*(q+tpsf[xx][yy+1])*xoff[yy+1];
	 }
	 else ttpsf[xx][yy]-=0.75*q*xoff[yy+1];
      }
      if (xoff[yy]>0 && yy==-r) ttpsf[xx][yy]+=0.75*q*xoff[yy];
      if (xoff[yy+1]<0 && yy==r) ttpsf[xx][yy]+=0.75*q*(-xoff[yy+1]);
   }
   zz=x-(int)x-0.5;
   z=pixmap[(int)x/10][(int)y/10][0]+exmap[(int)x/10][(int)y/10];
   xoff[1]=(1-z*(1-(1-rows[img][(int)x])*0.5))*(0.5-zz);
   xoff[0]=-(1-z*(1-(1-rows[img][(int)x])*0.5))*(0.5+zz);
   for (yy=2;yy<=r+1;yy++) if ((int)x+yy-1>=0 && (int)x+yy-1<800) xoff[yy]=xoff[yy-1]+(1-z*(1-(1-rows[img][(int)x+yy-1])*0.5));
   else xoff[yy]=xoff[yy-1]+(1-z);
   for (yy=-1;yy>=-r;yy--) if ((int)x+yy+1>=0 && (int)x+yy+1<800) xoff[yy]=xoff[yy+1]-(1-z*(1-(1-rows[img][(int)x+yy+1])*0.5));
   else xoff[yy]=xoff[yy+1]-(1-z);
   for (xx=-r;xx<=r;xx++) for (yy=-r;yy<=r;yy++) tpsf[xx][yy]=0;
   for (yy=-r;yy<=r;yy++) for (xx=-r;xx<=r;xx++) {
      float q;
      tpsf[xx][yy]+=(q=ttpsf[xx][yy]);
      if (xoff[xx]<0) {
	 if (xx>-r) {
	    tpsf[xx-1][yy]+=0.5*(q+ttpsf[xx-1][yy])*(-xoff[xx]);
	    tpsf[xx][yy]-=0.5*(q+ttpsf[xx-1][yy])*(-xoff[xx]);
	 }
	 else tpsf[xx][yy]-=0.75*q*(-xoff[xx]);
      }
      if (xoff[xx+1]>0) {
	 if (xx<r) {
	    tpsf[xx+1][yy]+=0.5*(q+ttpsf[xx+1][yy])*xoff[xx+1];
	    tpsf[xx][yy]-=0.5*(q+ttpsf[xx+1][yy])*xoff[xx+1];
	 }
	 else tpsf[xx][yy]-=0.75*q*xoff[xx+1];
      }
      if (xoff[xx]>0 && xx==-r) tpsf[xx][yy]+=0.75*q*xoff[xx];
      if (xoff[xx+1]<0 && xx==r) tpsf[xx][yy]+=0.75*q*(-xoff[xx+1]);
   }
}

void markstars(int x,int y) {
   int r,xx,yy,i;
   r=rphot[fld]+rpsf[fld];
   for (xx=-r;xx<=r;xx++) if (x+xx>=XMIN && x+xx<XMAX) for (yy=-r;yy<=r;yy++) if (y+yy>=YMIN && y+yy<YMAX) for (i=1;i<=indx[x+xx][y+yy][0];i++) stars[indx[x+xx][y+yy][i]].flag=1;
}

void cleargs(int IMG) {
   int img;

   for (img=0;img<Timg;img++) if (IMG<0 || img==IMG) gs_x[img]=-1000;
}

#define MINSMLT 1.35
float calcskyval(int n0,float*list,int img,float*unc) {
   int i,n,CONT=1;
   double R=0,RR=0,smlt=2.25,tsig;

   n=n0;
   while (CONT && n) {
      R=RR=0.;
      for (i=0;i<n;i++) {
	 R+=list[i];
	 RR+=list[i]*list[i];
      }
      R/=n;
      if (n>1) RR=smlt*sqrt(1.+(RR-R*R*n)/(n-1));
      else RR=smlt;
      if (R<0) tsig=sqrt(1.+RN[img]*RN[img])*3.5;
      else tsig=sqrt(1.+RN[img]*RN[img]+R/GAIN[img])*3.5;
      CONT=0;
      for (i=0;i<n;i++) if (fabs(list[i]-R)>RR) {
	 list[i--]=list[--n];
	 CONT=1;
      }
      if (!CONT && RR>tsig && smlt>MINSMLT) {
	 smlt*=0.9;
	 if (smlt<MINSMLT) smlt=MINSMLT;
	 CONT=1;
      }
   }
   if (!n) return 2.e30;
   if (unc!=NULL) *unc=RR/sqrt(n);
   return R;
}

float getsky_norm(int img,int x,int y) {
   static float *list=NULL;
   double s;
   int n=0,x1,y1,rsky,rin,ss;

   if (NO_SKY) return 0;
   if (!(OPTS&512)) return 0;
   if (abs(x-gs_x[img])<2 && abs(y-gs_y[img])<2) return gs_rval[img];
   if (fld==0) {rsky=36; rin=15; ss=2;}
   else {rsky=20; rin=8; ss=1;}
   if (list==NULL) {
      list=(float*)calloc((2*rsky+1)*(2*rsky+1),sizeof(float));
      if (!list) merr();
   }
   for (x1=x-rsky;x1<=x+rsky;x1+=ss) for (y1=y-rsky;y1<=y+rsky;y1+=ss) if (x1>=0 && x1<800 && y1>=0 && y1<800 && abs(x-x1)>rin && abs(y-y1)>rin && data[img][x1][y1]>DMIN[img] && data[img][x1][y1]<DMAX[img]*0.855) list[n++]=res[img][x1][y1]+sky[img][x1][y1];
   s=calcskyval(n,list,img,NULL);
   if (s>1.e30) s=0.;
   else s-=sky[img][x][y];
   gs_x[img]=x;
   gs_y[img]=y;
   gs_rval[img]=s;
   return s;
}

float getsky(int img,int x,int y) {
   int x1,y1,r,i,N=0,cont;
   float r2,av,sd;
   static float *list;
   static int first=1;

   if (!(OPTS&2)) return getsky_norm(img,x,y);
   r=rpsf[fld];
   if (r>rphot[fld]+2) r=rphot[fld]+2;
   if (first) {
      list=(float*)calloc(sizeof(float),4*r*r);
      if (!list) merr();
      first=0;
   }
   for (x1=x-r;x1<=x+r;x1++) for (y1=y-r;y1<=y+r;y1++) if (ppixOK2(img,x1,y1)) {
      r2=sqrt((x1-x)*(x1-x)+(y1-y)*(y1-y));
      if (r2>=rphot[fld]+1 && r2<=r) list[N++]=res[img][x1][y1]+sky[img][x1][y1];
   }
   cont=1;
   av=sd=0;
   if (N==1) av=list[0];
   while (cont && N>1) {
      av=sd=0;
      for (i=0;i<N;i++) av+=list[i];
      av/=N;
      for (i=0;i<N;i++) sd+=(list[i]-av)*(list[i]-av);
      sd=sqrt(sd/(N-1))*2.5;
      cont=0;
      for (i=0;i<N;)
	 if (fabs(list[i]-av)>sd) {
	    list[i]=list[--N];
	    cont=1;
	 }
	 else i++;
   }
   return av-sky[img][x][y];
}

inline void fixpos(int flt,float*x,float*y) {
   float dx,dy,r2,dps;
   dx=(*x)-400;
   dy=(*y)-400;
   if (fld) r2=(dx+6.2)*(dx+6.2)+(dy+6.2)*(dy+6.2);
   else r2=(dx+15.8)*(dx+15.8)+(dy+15.8)*(dy+15.8);
   dps=fdps[flt]*(1+(7.e-7)*r2);
   (*x)+=dx*dps;
   (*y)+=dy*dps;
}

void shift(int img,float x0,float y0,float *x,float *y) {
   float spars[4][2][2]={
      {{ 1.0000,-0.0002},{ 0.0002, 1.0000}},
      {{ 0.0042, 0.4577},{-0.4577, 0.0042}},
      {{-0.4577, 0.0019},{-0.0019,-0.4577}},
      {{ 0.0026,-0.4575},{ 0.4575, 0.0026}}
   };
   int ix,iy;
   static dpostype *spos=NULL,*ldpos=NULL;
   static int lastf=-1,sdpos;
   if (spos==NULL) {
      sdpos=sizeof(dpostype);
      spos=(dpostype*)calloc(Timg,sdpos);
      ldpos=(dpostype*)calloc(Timg,sdpos);
      if (!spos || !ldpos) merr();
   }
   if (lastf!=fld || dpos[img][0]!=ldpos[img][0] || dpos[img][1]!=ldpos[img][1]) {
      memcpy(ldpos[img],dpos[img],sdpos);
      spos[img][0]=spars[fld][1][1]*dpos[img][0]+spars[fld][1][0]*dpos[img][1];
      spos[img][1]=spars[fld][0][1]*dpos[img][0]+spars[fld][0][0]*dpos[img][1];
      lastf=fld;
   }
   ix=(int)x0/10; if (ix<0) ix=0; if (ix>79) ix=79;
   iy=(int)y0/10; if (iy<0) iy=0; if (iy>79) iy=79;
   *x=x0+spos[img][0]/(pixmap[ix][iy][0]*1.33-0.33);
   *y=y0+spos[img][1]/(pixmap[ix][iy][1]*1.33-0.33);
   fixpos(filt[img],x,y);
}

void centroid(int IMG,int x0,int y0,float *X,float *Y) {
   int img,x,y,xx,yy;
   float min=0,xc=0,yc=0,wt,twt=0;
   for (img=0;img<Timg;img++) if (img==IMG || (IMG<0 && img<Nimg)) {
      float tx,ty;
      wt=0;
      shift(img,x0,y0,&tx,&ty);
      x=(int)(tx+0.5);
      y=(int)(ty+0.5);
      for (xx=-2;xx<=2;xx++) for (yy=-2;yy<=2;yy++) if (abs(xx)+abs(yy)<4 && ppixOK2(img,x+xx,y+yy) && res[img][x+xx][y+yy]<min) min=res[img][x+xx][y+yy];
      for (xx=-2;xx<=2;xx++) for (yy=-2;yy<=2;yy++) if (abs(xx)+abs(yy)<4) {
	 if (ppixOK2(img,x+xx,y+yy)) {
	    if (res[img][x+xx][y+yy]>min) {
	       xc+=xx*(res[img][x+xx][y+yy]-min);
	       yc+=yy*(res[img][x+xx][y+yy]-min);
	       wt+=(res[img][x+xx][y+yy]-min);
	    }
	 }
	 else if (ppixOK2(img,x-xx,y-yy)) {
	    xc+=xx*(res[img][x-xx][y-yy]-min);
	    yc+=yy*(res[img][x-xx][y-yy]-min);
	    wt+=(res[img][x-xx][y-yy]-min);
	 }
      }
      twt+=wt;
      xc+=wt*(tx-x);
      yc+=wt*(ty-y);
   }
   if (twt>0) {
      *X=xc/twt+x0+0.5;
      *Y=yc/twt+y0+0.5;
   }
   else {
      *X=x0+0.5;
      *Y=y0+0.5;
   }
}

int starOK(int i,int IMG) {
   int img,nbad,ntot,x1,y1,ix,iy,fx,fy,anyOK=0;
   float x,y;

   for (img=0;img<Nimg;img++) if (IMG<0 || img==IMG) {
      shift(img,stars[i].x,stars[i].y,&x,&y);
      ix=(int)x; iy=(int)y;
      fx=(int)((x-ix-0.5)*sub[fld]+10.5+sub[fld])-sub[fld]-10;
      fy=(int)((y-iy-0.5)*sub[fld]+10.5+sub[fld])-sub[fld]-10;
      if (ix<=797 && iy<=797 && ix>=52 && iy>=52 && pixOK(img,ix,iy)) {
	 nbad=0;
	 if (!pixOK(img,ix+1,iy)) nbad+=2;
	 if (!pixOK(img,ix-1,iy)) nbad+=2;
	 if (!pixOK(img,ix,iy+1)) nbad+=2;
	 if (!pixOK(img,ix,iy-1)) nbad+=2;
	 if (!pixOK(img,ix+1,iy+1)) nbad++;
	 if (!pixOK(img,ix+1,iy-1)) nbad++;
	 if (!pixOK(img,ix-1,iy+1)) nbad++;
	 if (!pixOK(img,ix-1,iy-1)) nbad++;
	 if (nbad<4) {
	    nbad=ntot=0;
	    for (x1=ix-rphot[fld];x1<=ix+rphot[fld];x1++) for (y1=iy+circ[0][fld][fx][fy][x1-ix];y1<=iy+circ[1][fld][fx][fy][x1-ix];y1++) if (x1>=XMIN && x1<XMAX && y1>=YMIN && y1<YMAX) {
	       ntot++;
	       if (!pixOK(img,x1,y1)) nbad++;
	    }
	    if (nbad<ntot/4) anyOK=1;
	 }
      }
   }
   return anyOK;
}

float coldCTE(int img,float x,float y,float ct,float bg) {
   float mult,lct,lbg,yr,dm,cm1,cm2;

   if (OPTS&32) return 1.;
   if (ct<=0.) return pow(10,0.4*(fcorr[filt[img]][IGN[img]][fld]));
   //Convert to electrons in original image level;
   mult=EXP0[img]/EXP[img];
   if (GAIN[img]!=7) mult*=14;
   else mult*=7;
   bg-=ct*bgcorr[filt[img]][(fld==0)?0:1];
   bg*=mult;
   ct*=mult;
   //Soften background and signal;
   if (bg<0) bg=1.;
   else bg=sqrt(1.+bg*bg);
   if (ct<0) ct=1.;
   else ct=sqrt(1.+ct*ct);
   lbg=log(bg)-1;
   //bg-=10;
   yr=YEAR[img]+0.001767;
   dm=0.007730*exp(-0.504911*lbg)*(1+0.099582*yr)*x/800.;
   lct=log(ct)-7+0.92103404*dm;
   cm1=1.0-0.200904*lbg+0.039005*lbg*lct+0.002108*lct;
   if (cm1<0.153099) cm1=0.153099;
   cm2=0.957704*(yr-0.025474*yr*yr)*exp(-0.449956*lct);
   return pow(10,0.4*(fcorr[filt[img]][IGN[img]][fld]+dm+
		      2.412981*log(exp(0.022389*cm1*y/800.)*(1+cm2)-cm2)
		 ));
}

float CTE(int img,float x,float y,float ct,float bg) {
   float mult,XCTE,YCTE,lct,lbg,yr;
   if (OPTS&32) return 1.;
   if (TEMP[img]==0 && filters[filt[img]].color!='X' && filters[filt[img]].color!='U') return coldCTE(img,x,y,ct,bg);
   if (ct<=0.) return pow(10,0.4*(fcorr_warm[TEMP[img]][filt[img]]+gcorr_warm[fld]));
   //Convert to electrons in original image level;
   mult=EXP0[img]/EXP[img];
   if (GAIN[img]!=7) mult*=14;
   else mult*=7;
   bg-=ct*bgcorr_warm[filt[img]][TEMP[img]][(fld==0)?0:1];
   bg*=mult;
   ct*=mult;
   //Soften background and signal;
   if (bg<0) bg=1.;
   else bg=sqrt(1.+bg*bg);
   if (ct<0) ct=1.;
   else ct=sqrt(1.+ct*ct);
   //HSTphot CTE determination;
   lct=log(ct)-7;
   lbg=log(bg)-1;
   yr=YEAR[img]-2;
   if (TEMP[img]==0) YCTE=0.018009+(0.087972+exp(-0.506635*lct))*exp(-0.042368*bg-0.035149*lbg)*(0.096532+0.041442*yr);
   else YCTE=0.102822+0.027988*exp(-0.958904*lct);
   if (TEMP[img]==0) XCTE=(exp(-0.195966*lct))*exp(-0.126435*lbg)*(0.023881+0.002327*yr);
   else XCTE=0.;
   return pow(10,0.4*(fcorr_warm[TEMP[img]][filt[img]]+gcorr_warm[fld]+YCTE*y/800.+XCTE*x/800.));
}

void eval1(int img,float x,float y,float*m,float*s,float*ss,float*c,float*sh,float*ssky,float*cm,int full) {
   int ix,iy,x1,y1,dx,dy,fx,fy;
   float cs,cc,a,b,A,np,n,w;

   *c=cs=*s=*ss=*sh=cc=a=b=*m=0;
   shift(img,x,y,&x,&y);
   ix=(int)x;
   iy=(int)y;
   *ssky=getsky(img,ix,iy);
   if (full) calc1psf(img,rphot[fld],x,y,&fx,&fy);
   else fx=fy=0;
   for (x1=ix-rphot[fld],dx=-rphot[fld];x1<=ix+rphot[fld];x1++,dx++) for (y1=iy+circ[0][fld][fx][fy][dx];y1<=iy+circ[1][fld][fx][fy][dx];y1++) if (ppixOK(img,x1,y1)) {
      dy=y1-iy;
      n=noise(img,x1,y1);
      if (OPTS&1) w=cwt[fld][fx][fy][dx][dy];
      else if (OPTS&2048) w=cwt[fld][fx][fy][dx][dy]*tpsf[dx][dy]/n*tpsf[dx][dy]/n;
      else w=cwt[fld][fx][fy][dx][dy]*tpsf[dx][dy]/n;
      cc+=w;
      cs+=tpsf[dx][dy]*w;
      if (tpsf[dx][dy]>0) {
	 if (!NOSUBTRACT) (*s)+=(res[img][x1][y1]-(*ssky))*w;
	 else (*s)+=(data[img][x1][y1]-sky[img][x1][y1]-(*ssky))*w;
	 (*c)+=tpsf[dx][dy]*w;
	 (*ss)+=n*w*w;
      }
   }
   if (full && *ss>0 && *c!=0) {
      if (OPTS&1) cs=*c;
      cs/=cc;
      (*s)/=(*c);
      (*ss)=sqrt(*ss)/(*c);
      *c=*sh=A=np=0;
      for (x1=ix-rphot[fld],dx=-rphot[fld];x1<=ix+rphot[fld];x1++,dx++) for (y1=iy+circ[0][fld][fx][fy][dx];y1<=iy+circ[1][fld][fx][fy][dx];y1++) if (ppixOK(img,x1,y1)) {
	 n=noise(img,x1,y1);
	 dy=y1-iy;
	 a=tpsf[dx][dy]*(*s);
	 b=1./(n+dnoise[fld]*(a+(*ssky))*(a+(*ssky)));
	 //b=1./n;
	 a=res[img][x1][y1]-(*ssky)-a;
	 if (OPTS&2048) {
	    if (tpsf[dx][dy]>0) w=cwt[fld][fx][fy][dx][dy]*tpsf[dx][dy]/n;
	    else w=0.;
	 }
	 else w=cwt[fld][fx][fy][dx][dy];
	 (*c)+=a*a*b*w;
	 np+=w;
	 (*sh)+=a/sqrt(n)*(tpsf[dx][dy]-cs)*w;
	 A+=(tpsf[dx][dy]-cs)*(tpsf[dx][dy]-cs)*(*s)/sqrt(n)*w;
      }
      if (A>0) (*sh)/=A;
      else (*sh)=-1;
      (*c)=(*c)/np;
      (*m)=(*s)/((*ss)*sqrt((*c)+0.1)); //maximize S/N;
      //(*m)=1/sqrt((*c)+0.1); //minimize chi;
      if ((*m)<0.0001) (*m)=0.0001;
      (*c)=sqrt(*c);
      if (*c>1) (*ss)*=(*c);
   }
   else if (*ss>0) {
      (*s)/=(*c);
      (*ss)=sqrt(*ss)/(*c);
      (*c)=1;
   }
   *cm=corr[IGN[img]][fld]/EXP[img]*CTE(img,y,x,(*s)*apcor[img],sky[img][ix][iy])*apcor[img];
   (*s)*=(*cm);
   (*ss)*=(*cm);
   (*ssky)=((*ssky)+sky[img][ix][iy])*(*cm);
   return;
}

void t4_eval(int i,int flt,int IMG,float*xs,float*xss,float*xchi,float*xsky) {
   float CR=0,CRV=0,CRS=0,twt=0,st,cr,np,a,b,s,ss,wt,tx,ty,ssky;
   int x1,y1,ix,iy,fx,fy,img;

   *xsky=0;
   for (img=0;img<Nimg;img++) if ((flt<0 || filt[img]==flt) && (IMG<0 || img==IMG)) {
      shift(img,stars[i].x,stars[i].y,&tx,&ty);
      ix=(int)tx; iy=(int)ty;
      ssky=getsky(img,ix,iy);
      calc1psf(img,rphot[fld],tx,ty,&fx,&fy);
      np=s=ss=st=0;
      for (x1=ix-rphot[fld];x1<=ix+rphot[fld];x1++) for (y1=iy+circ[0][fld][fx][fy][x1-ix];y1<=iy+circ[1][fld][fx][fy][x1-ix];y1++) if (ppixOK(img,x1,y1)) {
	 float n;
	 n=noise(img,x1,y1);
	 a=res[img][x1][y1]-tpsf[x1-ix][y1-iy]*stars[i].is[img]/stars[i].icm[img];
	 st+=a*a/n*cwt[fld][fx][fy][x1-ix][y1-iy];
	 np+=cwt[fld][fx][fy][x1-ix][y1-iy];
	 if (tpsf[x1-ix][y1-iy]>0) {
	    a=tpsf[x1-ix][y1-iy];
	    b=a/n*cwt[fld][fx][fy][x1-ix][y1-iy];
	    s+=res[img][x1][y1]*b;
	    a*=b;
	    ss+=a*cwt[fld][fx][fy][x1-ix][y1-iy];
	 }
      }
      if (np>0) {
	 ss=sqrt(ss);
	 if (ss<s*0.01) wt=100./sqrt(st/np+0.25);
	 else wt=s/ss/sqrt(st/np+0.25);
	 if (wt<0) wt=0;
      }
      else wt=0;
      wt+=1;
      twt+=wt;
      a=b=np=cr=0;
      for (x1=ix-rphot[fld];x1<=ix+rphot[fld];x1++) for (y1=iy+circ[0][fld][fx][fy][x1-ix];y1<=iy+circ[1][fld][fx][fy][x1-ix];y1++) if ((x1!=ix || y1!=iy) && ppixOK(img,x1,y1)) {
	 cr+=res[img][x1][y1]*res[img][x1][y1]/noise(img,x1,y1)*cwt[fld][fx][fy][x1-ix][y1-iy];
	 np+=cwt[fld][fx][fy][x1-ix][y1-iy];
      }
      if (np>0) CR+=cr/np*wt;
      if (!NOSUBTRACT) CRV+=res[img][ix][iy]*wt*stars[i].icm[img];
      else CRV+=(data[img][ix][iy]-sky[img][ix][iy])*wt*stars[i].icm[img];
      CRS+=(noise(img,ix,iy)+res[img][ix][iy]/GAIN[img])*stars[i].icm[img]*stars[i].icm[img]*wt*wt;
      (*xsky)+=(sky[img][ix][iy]+ssky)*wt*stars[i].icm[img];
   }
   if (twt>0) {
      CR=sqrt(CR/twt);
      CRV/=twt;
      CRS=sqrt(CRS)/twt;
      (*xsky)/=twt;
   }
   *xs=CRV;
   *xss=CRS;
   *xchi=CR;
}

void t5_eval(int i,int flt,int IMG,float*xs,float*xss,float*xchi,float*xsky) {
   float FL=0,FLV=0,FLS=0,twt=0,st,np,a,b,c,d,s,ss,wt,tx,ty,ssky;
   int x1,y1,ix,iy,fx,fy,img;

   *xsky=0;
   for (img=0;img<Nimg;img++) if ((flt<0 || filt[img]==flt) && (IMG<0 || img==IMG)) {
      shift(img,stars[i].x,stars[i].y,&tx,&ty);
      ix=(int)tx; iy=(int)ty;
      ssky=getsky(img,ix,iy);
      calc1psf(img,rphot[fld],tx,ty,&fx,&fy);
      np=s=ss=st=0;
      for (x1=ix-rphot[fld];x1<=ix+rphot[fld];x1++) for (y1=iy+circ[0][fld][fx][fy][x1-ix];y1<=iy+circ[1][fld][fx][fy][x1-ix];y1++) if (ppixOK(img,x1,y1)) {
	 float n;
	 n=noise(img,x1,y1);
	 a=res[img][x1][y1]-tpsf[x1-ix][y1-iy]*stars[i].is[img]/stars[i].icm[img];
	 st+=a*a/n*cwt[fld][fx][fy][x1-ix][y1-iy];
	 np+=cwt[fld][fx][fy][x1-ix][y1-iy];
	 if (tpsf[x1-ix][y1-iy]>0) {
	    a=tpsf[x1-ix][y1-iy];
	    b=a/n*cwt[fld][fx][fy][x1-ix][y1-iy];
	    s+=res[img][x1][y1]*b;
	    a*=b;
	    ss+=a*cwt[fld][fx][fy][x1-ix][y1-iy];
	 }
      }
      if (np>0) {
	 ss=sqrt(ss);
	 if (ss<s*0.01) wt=100./sqrt(st/np+0.25);
	 else wt=s/ss/sqrt(st/np+0.25);
	 if (wt<0) wt=0;
      }
      else wt=0;
      wt+=1;
      twt+=wt;
      a=b=c=d=np=0;
      for (x1=ix-rphot[fld];x1<=ix+rphot[fld];x1++) for (y1=iy+circ[0][fld][fx][fy][x1-ix];y1<=iy+circ[1][fld][fx][fy][x1-ix];y1++) if (ppixOK(img,x1,y1)) {
	 float n;
	 n=noise(img,x1,y1);
	 a+=res[img][x1][y1]*res[img][x1][y1]*cwt[fld][fx][fy][x1-ix][y1-iy]/n;
	 if (!NOSUBTRACT) b+=res[img][x1][y1]*cwt[fld][fx][fy][x1-ix][y1-iy]/n;
	 else b+=(data[img][x1][y1]-sky[img][x1][y1])*cwt[fld][fx][fy][x1-ix][y1-iy]/n;
	 d+=cwt[fld][fx][fy][x1-ix][y1-iy]/n;
	 c+=cwt[fld][fx][fy][x1-ix][y1-iy]*cwt[fld][fx][fy][x1-ix][y1-iy]/sqrt(n);
	 np+=cwt[fld][fx][fy][x1-ix][y1-iy];
      }
      if (d>0 && np>0) FL+=(a-b*b/d)/np*wt;
      if (d>0) {
	 FLV+=b/d*np*wt*stars[i].icm[img];
	 FLS+=c/d/d*wt*wt*stars[i].icm[img]*stars[i].icm[img];
      }
      (*xsky)+=(sky[img][ix][iy]+ssky)*wt*stars[i].icm[img];
   }
   if (twt>0) {
      FL=sqrt(FL/twt);
      FLV/=twt;
      FLS=sqrt(FLS)/twt;
      (*xsky)/=twt;
   }
   *xs=FLV;
   *xss=FLS;
   *xchi=FL;
}

void eval(int FLT,float x,float y,float*M,float*S,float*SS,float*C,float*SH,float*SSKY,float*IS,float*ISS,float*ICM,int checkstar) {
   int img;
   float m,s,ss,c,sh,ssky,cm,wt,swt,twt=0,tswt=0;

   *S=*SS=*C=*SH=*SSKY=0;
   for (img=0;img<Nimg;img++) if ((FLT<0 || filt[img]==FLT) && (checkstar<0 || starOK(checkstar,img))) {
      eval1(img,x,y,&m,&s,&ss,&c,&sh,&ssky,&cm,1);
      if (m<=0.) wt=swt=0.;
      else if (FLT<0) {
	 wt=fabs(s)/ss/ss;
	 if (s>0) swt=wt;
	 else swt=0.;
      }
      else {
	 wt=1./ss/ss;
	 if (s>0) swt=wt*s;
	 else swt=0.;
      }
      twt+=wt;
      tswt+=swt;
      if (IS) IS[img]=s;
      if (ISS) ISS[img]=ss;
      if (ICM) ICM[img]=cm;
      (*S)+=s*wt;
      (*SS)+=ss*ss*wt*wt;
      (*C)+=c*c*wt;
      (*SH)+=sh*swt;
      (*SSKY)+=ssky*wt;
   }
   if (twt>0.) {
      (*S)/=twt;
      (*SS)=sqrt(*SS)/twt;
      (*C)=sqrt((*C)/twt);
      (*SSKY)/=twt;
      if (*S) (*M)=(*S)/((*SS)*sqrt((*C)*(*C)+0.25));
      else (*M)=0;
   }
   else *S=*SS=*C=*SSKY=*M=0.;
   if (tswt>0.) (*SH)/=tswt;
   else *SH=0.;
}

int photsearch(int IMG,float x,float y,float*sx,float*sy,float*bs,float*bss,float*bsky,float*bc,float*bsh,float*bis,float*biss,float*bcm,int finding) {
   int ix,iy,x1,y1,DONE=0,first=1,bx,by,img;
   float x0,y0,dx,dy,result[5][5],tresult[5][5],m,s,ss,c,sh,ssky,bm=-1;
   static int afirst=1;
   static float *is,*iss,*cm;

   cleargs(IMG);
   *sx=x; *sy=y;
   *bs=*bss=*bsky=*bc=*bsh=0;
   if (afirst) {
      afirst=0;
      is=(float*)calloc(Nimg,sizeof(float));
      iss=(float*)calloc(Nimg,sizeof(float));
      cm=(float*)calloc(Timg,sizeof(float));
      if (!is || !iss || !cm) merr();
   }
   ix=(int)x;
   iy=(int)y;
   if (ix<=XMIN || ix>=XMAX-1 || iy<=YMIN || iy>=YMAX-1) return 1;
   if (finding && (ran[ix][iy] || indx[ix][iy][0])) return 1;
   for (x1=0;x1<5;x1++) for (y1=0;y1<5;y1++) result[x1][y1]=-1000;
   while (!DONE) {
      x0=x; y0=y;
      bx=by=2;
      for (x1=1;x1<4;x1++) for (y1=1;y1<4;y1++) {
	 if (WARM) x1=y1=2;
	 x=x0+(x1-2)/(float)sub[fld];
	 y=y0+(y1-2)/(float)sub[fld];
	 if (x<XMIN || y<YMIN || x>=XMAX || y>=YMAX) result[x1][y1]=-999;
	 else if (result[x1][y1]<=-1000) {
	    if (IMG<0) eval(-1,x,y,&m,&s,&ss,&c,&sh,&ssky,is,iss,cm,-1);
	    else eval1(IMG,x,y,&m,&s,&ss,&c,&sh,&ssky,cm,1);
	    if (m<-999) m=-999;
	    result[x1][y1]=m;
	    if (m>-1000 && (m>bm || first)) {
	       first=0;
	       bm=m;
	       bx=x1;
	       by=y1;
	       *sx=x;
	       *sy=y;
	       *bs=s;
	       *bss=ss;
	       *bsky=ssky;
	       *bc=c;
	       *bsh=sh;
	       if (bis) for (img=0;img<Nimg;img++) {
		  bis[img]=is[img];
		  biss[img]=iss[img];
		  bcm[img]=cm[img];
	       }
	    }
	 }
	 if (WARM) x1=y1=3;
      }
      if (first) return 1;
      x=*sx;
      y=*sy;
      if (finding && (ran[(int)x][(int)y] || indx[(int)x][(int)y][0])) return 1;
      if (bx!=2 || by!=2) {
	 for (x1=0;x1<5;x1++) for (y1=0;y1<5;y1++) if (x1+bx-2>=0 && x1+bx-2<=4 && y1+by-2>=0 && y1+by-2<=4) tresult[x1][y1]=result[x1+bx-2][y1+by-2];
	 else tresult[x1][y1]=-1000;
	 memcpy(result,tresult,sizeof(float[5][5]));
      }
      DONE=1;
      if (!WARM && (result[2][3]<=-1000 || result[3][3]<=-1000 || result[3][2]<=-1000 || result[3][1]<=-1000 || result[2][1]<=-1000 || result[1][1]<=-1000 || result[1][2]<=-1000 || result[1][3]<=-1000)) DONE=0;
   }
   dx=dy=0;
   if (result[2][2]>-1000 && result[2][3]>-1000 && result[2][1]>-1000 && result[1][2]>-1000 && result[3][2]>-1000) {
      if (result[3][2]>=result[2][2]) dx=0.5;
      else if (result[1][2]>=result[2][2]) dx=-0.5;
      else dx=(result[3][2]-result[1][2])/(4*result[2][2]-2*result[3][2]-2*result[1][2]);
      if (dx>0.5) dx=0.5;
      else if (dx<-0.5) dx=-0.5;
      if (result[2][3]>=result[2][2]) dy=0.5;
      else if (result[2][1]>=result[2][2]) dy=-0.5;
      else dy=(result[2][3]-result[2][1])/(4*result[2][2]-2*result[2][3]-2*result[2][1]);
      if (dy>0.5) dy=0.5;
      else if (dy<-0.5) dy=-0.5;
      (*sx)+=dx/(float)sub[fld];
      (*sy)+=dy/(float)sub[fld];
   }
   if (finding && (ran[(int)(*sx)][(int)(*sy)] || indx[(int)(*sx)][(int)(*sy)][0])) return 1;
   if (finding) ran[(int)(*sx)][(int)(*sy)]=1;
   return first;
}

void avgstar(int i,stype *s) {
   stars[i].s=(stars[i].s+s->s)*0.5;
   stars[i].ss=sqrt((stars[i].ss*stars[i].ss+s->ss*s->ss)*0.5);
   stars[i].sky=(s->sky+stars[i].sky)*0.5;
   stars[i].x=(s->x+stars[i].x)*0.5;
   stars[i].y=(s->y+stars[i].y)*0.5;
}

int getclass1(int IMG,float x,float y,float*is,float*icm,float sh) {
   float ST=0,CR=0,CRTEST=0,FL=0,twt=0,st,cr,np,a,b,c,d,s,ss,wt,tx,ty,ssky;
   int x1,y1,ix,iy,fx,fy,img;

   if (OPTS&128) return 0;
   for (img=0;img<Timg;img++) if ((IMG<0 && img<Nimg) || img==IMG) {
      shift(img,x,y,&tx,&ty);
      ix=(int)tx; iy=(int)ty;
      ssky=getsky(img,ix,iy);
      calc1psf(img,rphot[fld],tx,ty,&fx,&fy);
      np=s=ss=st=0;
      for (x1=ix-rphot[fld];x1<=ix+rphot[fld];x1++) for (y1=iy+circ[0][fld][fx][fy][x1-ix];y1<=iy+circ[1][fld][fx][fy][x1-ix];y1++) if (ppixOK(img,x1,y1)) {
	 float n;
	 n=noise(img,x1,y1);
	 if (IMG<0) a=res[img][x1][y1]-ssky-tpsf[x1-ix][y1-iy]*is[img]/icm[img];
	 else a=res[img][x1][y1]-ssky-tpsf[x1-ix][y1-iy]*(*is)/(*icm);
	 st+=a*a/n*cwt[fld][fx][fy][x1-ix][y1-iy];
	 np+=cwt[fld][fx][fy][x1-ix][y1-iy];
	 if (tpsf[x1-ix][y1-iy]>0) {
	    a=tpsf[x1-ix][y1-iy];
	    b=a/n*cwt[fld][fx][fy][x1-ix][y1-iy];
	    s+=(res[img][x1][y1]-ssky)*b;
	    a*=b;
	    ss+=a*cwt[fld][fx][fy][x1-ix][y1-iy];
	 }
      }
      if (np>0) {
	 ss=sqrt(ss);
	 if (ss<s*0.01) wt=100./sqrt(st/np+0.25);
	 else wt=s/ss/sqrt(st/np+0.25);
	 if (wt<0) wt=0;
	 ST+=st/np*wt;
      }
      else wt=0;
      wt+=1;
      twt+=wt;
      a=b=np=cr=0;
      for (x1=ix-rphot[fld];x1<=ix+rphot[fld];x1++) for (y1=iy+circ[0][fld][fx][fy][x1-ix];y1<=iy+circ[1][fld][fx][fy][x1-ix];y1++) if ((x1!=ix || y1!=iy) && ppixOK(img,x1,y1)) {
	 float n;
	 n=noise(img,x1,y1);
	 cr+=(res[img][x1][y1]-ssky)*(res[img][x1][y1]-ssky)/n*cwt[fld][fx][fy][x1-ix][y1-iy];
	 np+=cwt[fld][fx][fy][x1-ix][y1-iy];
      }
      if (np>0) CR+=cr/np*wt;
      CRTEST+=(res[img][ix][iy]-ssky-3.*sqrt(noise(img,ix,iy)+(res[img][ix][iy]-ssky)/GAIN[img]))*wt;
      a=b=c=d=np=0;
      for (x1=ix-rphot[fld];x1<=ix+rphot[fld];x1++) for (y1=iy+circ[0][fld][fx][fy][x1-ix];y1<=iy+circ[1][fld][fx][fy][x1-ix];y1++) if (ppixOK(img,x1,y1)) {
	 float n;
	 n=noise(img,x1,y1);
	 a+=(res[img][x1][y1]-ssky)*(res[img][x1][y1]-ssky)*cwt[fld][fx][fy][x1-ix][y1-iy]/n;
	 b+=(res[img][x1][y1]-ssky)*cwt[fld][fx][fy][x1-ix][y1-iy]/n;
	 d+=cwt[fld][fx][fy][x1-ix][y1-iy]/n;
	 c+=cwt[fld][fx][fy][x1-ix][y1-iy]*cwt[fld][fx][fy][x1-ix][y1-iy]/sqrt(n);
	 np+=cwt[fld][fx][fy][x1-ix][y1-iy];
      }
      if (d>0 && np>0) FL+=(a-b*b/d)/np*wt;
   }
   if (twt>0) {
      ST=sqrt(ST/twt);
      CR=sqrt(CR/twt);
      FL=sqrt(FL/twt);
   }
   if (CR>=0 && CR<ST && sh>0 && CRTEST>=0) return 4;
   if (FL>=0 && FL<ST && sh<0) return 5;
   return 0;
}

int getclass(int i) {
   if (WARM) {
      if (stars[i].type<4) return 0;
      return stars[i].type;
   }
   return getclass1(-1,stars[i].x,stars[i].y,stars[i].is,stars[i].icm,stars[i].sh);
}

void phot(int x,int y,int mark,int iter,int nit) {
   int first,st,i,j,img;
   float bs,bss,bsky,bc,bsh,cx,cy,sx,sy,bss0;
   stype old;
   static float *oldis,*bis,*biss,*bcm;
   static int isfirst=1;

   if (isfirst) {
      isfirst=0;
      oldis=(float*)calloc(Nimg,sizeof(float));
      bis=(float*)calloc(Nimg,sizeof(float));
      biss=(float*)calloc(Nimg,sizeof(float));
      bcm=(float*)calloc(Nimg,sizeof(float));
      if (!oldis || !bis || !biss || !bcm) merr();
   }
   if (mark<0) {
      centroid(-1,x,y,&cx,&cy);
      first=photsearch(-1,cx,cy,&sx,&sy,&bs,&bss,&bsky,&bc,&bsh,bis,biss,bcm,1);
   }
   else {
      memcpy(&old,stars+mark,sizeof(stype));
      memcpy(oldis,stars[mark].is,Nimg*sizeof(float));
      first=photsearch(-1,old.x,old.y,&sx,&sy,&bs,&bss,&bsky,&bc,&bsh,bis,biss,bcm,0);
   }
   if (first) bs=bss=0;
   if (bs<=0) bs=0;
   if (bc>0) bss0=bss/bc;
   else bss0=bss;
   if ((bs>=bss0*findsig && bs>0 && !first) || mark>=0) {
      if (mark<0) {
	 if (Nstars>=NMAX) {
	    printf("****Too many stars\n");
	    exit(-1);
	 }
	 st=Nstars;
	 Nstars++;
	 stars[st].type=1;
      }
      else {
	 st=mark;
	 if (first || bs<=0) {
	    if (old.s!=0) markstars((int)old.x,(int)old.y);
	 }
	 else if ((int)sx!=(int)old.x || (int)sy!=(int)old.y) {
	    markstars((int)sx,(int)sy);
	    markstars((int)old.x,(int)old.y);
	 }
	 else if (fabs(sx-old.x)>1.5/sub[fld] || fabs(sy-old.y)>1.5/sub[fld]) markstars((int)sx,(int)sy);
	 else for (img=0,j=1;img<Nimg && j;img++) if (fabs(bis[j]-oldis[j])>1 && fabs(bis[j]-oldis[j])>(bis[j]+oldis[j])*0.0005) {
	    markstars((int)sx,(int)sy);
	    j=0;
	 }
      }
      stars[st].flag=0;
      stars[st].x=sx;
      stars[st].y=sy;
      stars[st].ss=bss;
      stars[st].sky=bsky;
      stars[st].chi=bc;
      stars[st].sh=bsh;
      for (i=0;i<Nimg;i++) {
	 stars[st].is[i]=bis[i];
	 stars[st].iss[i]=biss[i];
	 stars[st].icm[i]=bcm[i];
      }
      if (first || bs<=0 || indx[(int)sx][(int)sy][0]>=NINDX) {
	 stars[st].s=0;
      }
      else {
	 stars[st].s=bs;
	 if (nit>MaxIT/2 && nit%3==MaxIT%3) avgstar(st,&old);
	 addindx((int)stars[st].x,(int)stars[st].y,st);
      }
   }
}

void imgadd1(int img,int i,int neg) {
   int x1,y1,ix,iy,fx,fy;
   float x,y,p;

   shift(img,stars[i].x,stars[i].y,&x,&y);
   ix=(int)x; iy=(int)y;
   if (ix<0) ix=0; if (ix>799) ix=799;
   if (iy<0) iy=0; if (iy>799) iy=799;
   calc1psf(img,rpsf[fld],x,y,&fx,&fy);
   for (x1=ix-rpsf[fld];x1<=ix+rpsf[fld];x1++) for (y1=iy-rpsf[fld];y1<=iy+rpsf[fld];y1++) if (ppixOK(img,x1,y1)) {
      p=tpsf[x1-ix][y1-iy]+dpsf[img][iy/100][ix/100][fx][fy];
      if (p>0. || neg) res[img][x1][y1]+=stars[i].is[img]/stars[i].icm[img]*p;
   }
}

void imgadd(int i,int neg) {
   int img;
   for (img=0;img<Nimg;img++) imgadd1(img,i,neg);
}

void imgsub1(int img,int i,int neg) {
   int x1,y1,ix,iy,fx,fy;
   float x,y,p;

   shift(img,stars[i].x,stars[i].y,&x,&y);
   ix=(int)x; iy=(int)y;
   if (ix<0) ix=0; if (ix>799) ix=799;
   if (iy<0) iy=0; if (iy>799) iy=799;
   calc1psf(img,rpsf[fld],x,y,&fx,&fy);
   for (x1=ix-rpsf[fld];x1<=ix+rpsf[fld];x1++) for (y1=iy-rpsf[fld];y1<=iy+rpsf[fld];y1++) if (ppixOK(img,x1,y1)) {
      p=tpsf[x1-ix][y1-iy]+dpsf[img][iy/100][ix/100][fx][fy];
      if (p>0. || neg) res[img][x1][y1]-=stars[i].is[img]/stars[i].icm[img]*p;
   }
}

void imgsub(int i,int neg) {
   int img;
   for (img=0;img<Nimg;img++) imgsub1(img,i,neg);
}

/*Quicksort from NR; 0-index by AED*/
#define SWAP(a,b) {memcpy(&temp,a,ssize);memcpy(a,b,ssize);memcpy(b,&temp,ssize);}
#define M 7
#define NSTACK 50

void sortstars() {
   int i,ir,j,k,l=0;
   int jstack=0,*istack,ssize;
   stype temp,a;

   if (WARM) return;
   ir=Nstars-1;
   ssize=sizeof(stype);
   if ((istack=(int*)calloc(sizeof(int),NSTACK))==NULL) merr();
   for (;;) {
      if (ir-l<M) {
	 for (j=l+1;j<=ir;j++) {
	    memcpy(&a,stars+j,ssize);
	    for (i=j-1;i>=l;i--) {
	       if (stars[i].s>=a.s) break;
	       memcpy(stars+i+1,stars+i,ssize);
	    }
	    memcpy(stars+i+1,&a,ssize);
	 }
	 if (jstack==0) break;
	 ir=istack[jstack--];
	 l=istack[jstack--];
      } else {
	 k=(l+ir)>>1;
	 SWAP(stars+k,stars+l+1)
	 if (stars[l+1].s<stars[ir].s) SWAP(stars+l+1,stars+ir)
	 if (stars[l].s<stars[ir].s) SWAP(stars+l,stars+ir)
	 if (stars[l+1].s<stars[l].s) SWAP(stars+l+1,stars+l)
	 i=l+1;
	 j=ir;
	 memcpy(&a,stars+l,ssize);
	 for (;;) {
	    do i++; while (stars[i].s>a.s);
	    do j--; while (stars[j].s<a.s);
	    if (j<i) break;
	    SWAP(stars+i,stars+j);
	 }
	 memcpy(stars+l,stars+j,ssize);
	 memcpy(stars+j,&a,ssize);
	 jstack+=2;
	 if (jstack > NSTACK) {
	    printf("****NSTACK too small in sortstars.");
	    exit(-1);
	 }
	 if (ir-i+1>=j-l) {
	    istack[jstack]=ir;
	    istack[jstack-1]=i;
	    ir=j-1;
	 } else {
	    istack[jstack]=j-1;
	    istack[jstack-1]=l;
	    l=i;
	 }
      }
   }
   free(istack);
}
#undef M
#undef NSTACK
#undef SWAP

int igoodstar(int img,int i,int redge,int max) {
   int x,y,ix,iy,ct=0;
   float tx,ty;

   if (!starOK(i,img)) return 0;
   if (stars[i].is[img]<=0 || stars[i].is[img]<=5*stars[i].iss[img]) return 0;
   shift(img,stars[i].x,stars[i].y,&tx,&ty);
   ix=(int)tx; iy=(int)ty;
   if (ix<XMIN2 || ix-redge<XMIN || ix>=XMAX2 || ix+redge>=XMAX || iy<YMIN2 || iy-redge<YMIN || iy>=YMAX2 || iy+redge>=YMAX) return 0;
   for (x=-redge;x<=redge && ct<max;x++) for (y=-redge;y<=redge && ct<max;y++) if (data[img][ix+x][iy+y]>=DMAX[img]*0.855 || data[img][ix+x][iy+y]<=DMIN[img]) {
      if (abs(x)<2 && abs(y)<2) return 0;
      ct++;
   }
   if (ct<max) return 1;
   return 0;
}

int goodstar(int IMG,int i,int rsep,int redge,int qual) {
   int j,ix,iy;

   if (getclass(i)) return 0;
   if (qual && (stars[i].sh>0.15 || stars[i].sh<-0.15 || stars[i].chi>2.5)) return 0;
   if (!qual && (stars[i].sh>0.15 || stars[i].sh<-0.20)) return 0;
   if (stars[i].s<stars[i].ss*10 || stars[i].s<=0) return 0;
   ix=(int)stars[i].x; iy=(int)stars[i].y;
   for (j=0;j<i;j++) if (abs(ix-(int)stars[j].x)<=rsep && abs(iy-(int)stars[j].y)<=rsep) return 0;
   for (j=i+1;j<Nstars && stars[j].s*2>=stars[i].s;j++) if (abs(ix-(int)stars[j].x)<=redge && abs(iy-(int)stars[j].y)<=redge) return 0;
   if (IMG>=0) return igoodstar(IMG,i,redge,10);
   return 1;
}

int fix1psf(int img,int Np,unsigned short**plist,int it) {
   int x,y,xx,yy,x3,y3,i,N,rad;
   float *list,*sig,av,sd,z,**off,**avpsf,*ssky,toff=0,toffd=0;
   float is;

   ssky=(float*)calloc(Np,sizeof(float));
   list=(float*)calloc(Np,sizeof(float));
   sig=(float*)calloc(Np,sizeof(float));
   off=(float**)calloc(2*rpsf[fld]+1,sizeof(float*));
   avpsf=(float**)calloc(2*rpsf[fld]+1,sizeof(float*));
   if (!ssky || !list || !sig || !off || !avpsf) merr();
   off+=rpsf[fld];
   avpsf+=rpsf[fld];
   rad=(sub[fld]+2)/2;
   for (i=0;i<Np;i++) if (plist[i][img+1] && ((OPTS&2) || (OPTS&512))) {
      float tx,ty;
      shift(img,stars[plist[i][0]].x,stars[plist[i][0]].y,&tx,&ty);
      gs_x[img]=-1000;
      ssky[i]=getsky_norm(img,(int)tx,(int)ty);
   }
   else ssky[i]=0;
   for (x=-rpsf[fld];x<=rpsf[fld];x++) {
      off[x]=(float*)calloc(2*rpsf[fld]+1,sizeof(float));
      avpsf[x]=(float*)calloc(2*rpsf[fld]+1,sizeof(float));
      if (!off[x] || !avpsf[x]) merr();
      off[x]+=rpsf[fld];
      avpsf[x]+=rpsf[fld];
      for (y=-rpsf[fld];y<=rpsf[fld];y++) {
	 N=0;
	 for (i=0;i<Np;i++) if (plist[i][img+1]) {
	    int ii;
	    ii=plist[i][0];
	    if ((is=stars[ii].is[img]/stars[ii].icm[img])>0 && stars[ii].s>0 && stars[ii].s>stars[ii].ss*minsig) {
	       float tx,ty;
	       shift(img,stars[ii].x,stars[ii].y,&tx,&ty);
	       xx=x+(int)tx;
	       yy=y+(int)ty;
	       if (xx>=50 && xx<799 && yy>=50 && yy<799) {
		  if (data[img][xx][yy]>DMIN[img] && data[img][xx][yy]<DMAX[img]*0.855) {
		     sig[N]=is/sqrt(noise(img,xx,yy));
		     if (sky[img][xx][yy]/EXP[img]>0) sig[N]/=1+sky[img][xx][yy]/EXP[img]*100.;
		     list[N++]=(res[img][xx][yy]-ssky[i])/is;
		  }
		  else {
		     float ns=0,rs=0;
		     int ct=0,dx,dy;
		     for (dx=-1;dx<=1;dx++) for (dy=-1;dy<=1;dy++) if (xx+dx>=50 && xx+dx<799 && yy+dy>=50 && yy+dy<799 && data[img][xx+dx][yy+dy]>DMIN[img] && data[img][xx+dx][yy+dy]<DMAX[img]*0.855) {
			ns+=noise(img,xx+dx,yy+dy);
			rs+=res[img][xx+dx][yy+dy];
			ct++;
		     }
		     if (ct) {
			sig[N]=is/sqrt(ns/ct);
			if (sky[img][xx][yy]/EXP[img]>0) sig[N]/=1+sky[img][xx][yy]/EXP[img]*100.;
			list[N++]=(rs/ct-ssky[i])/is;
		     }
		  }
	       }
	    }
	 }
	 avpsf[x][y]=0;
	 for (x3=0;x3<8;x3++) for (y3=0;y3<8;y3++) for (xx=-rad;xx<=rad;xx++) for (yy=-rad;yy<=rad;yy++) avpsf[x][y]+=psflib[img][x3][y3][xx][yy][x][y]+dpsf[img][x3][y3][xx][yy];
	 avpsf[x][y]/=64.*(2*rad+1)*(2*rad+1);
	 av=sd=0;
	 xx=1;
	 while (xx) {
	    av=sd=z=0;
	    for (i=0;i<N;i++) {
	       av+=list[i]*sig[i];
	       z+=sig[i];
	    }
	    //prior of S/N=10 star;
	    av-=poff[img][x][y]*100.;
	    z+=100.;
	    xx=0;
	    if (z>0) {
	       av/=z;
	       for (i=0;i<N;i++) sd+=(list[i]-av)*(list[i]-av)*sig[i];
	       sd=sqrt(sd/z)*3.5;
	       for (i=0;i<N;)
		  if (fabs(list[i]-av)>sd) {
		     list[i]=list[--N];
		     sig[i]=sig[N];
		     xx=1;
		  }
		  else i++;
	    }
	    else av=0;
	 }
	 off[x][y]=av;
	 if (abs(x)<=rphot[fld] && abs(y)<=rphot[fld] && cwt[fld][0][0][x][y]>0) {
	    toff+=avpsf[x][y]*cwt[fld][0][0][x][y];
	    toffd+=(avpsf[x][y]+off[x][y])*cwt[fld][0][0][x][y];
	 }
      }
   }
   toff/=toffd;
   for (x=-rpsf[fld];x<=rpsf[fld];x++) for (y=-rpsf[fld];y<=rpsf[fld];y++) {
      off[x][y]=toff*off[x][y]+(toff-1)*avpsf[x][y];
      if (it>4) off[x][y]/=1+it*0.1;
      poff[img][x][y]+=off[x][y];
   }
   for (i=0;i<Nstars;i++) imgadd1(img,i,1);
   for (x3=0;x3<8;x3++) for (y3=0;y3<8;y3++) for (xx=-rad;xx<=rad;xx++) for (yy=-rad;yy<=rad;yy++) for (x=-rpsf[fld];x<=rpsf[fld];x++) for (y=-rpsf[fld];y<=rpsf[fld];y++) psflib[img][x3][y3][xx][yy][x][y]+=off[x][y];
   getspsf(img,fld);
   for (i=0;i<Nstars;i++) imgsub1(img,i,1);
   free(ssky);
   free(list);
   free(sig);
   i=0;
   for (x=-rpsf[fld];x<=rpsf[fld] && !i;x++) for (y=-rpsf[fld];y<=rpsf[fld] && !i;y++) if (fabs(off[x][y])>=0.001+0.0001*it) i=1;
   for (x=-rpsf[fld];x<=rpsf[fld];x++) {
      free(off[x]-rpsf[fld]);
      free(avpsf[x]-rpsf[fld]);
   }
   free(off-rpsf[fld]);
   free(avpsf-rpsf[fld]);
   return i;
}

int fixpsf(int Np,unsigned short**plist,int it) {
   int img,rval=0;
   for (img=0;img<Nimg;img++) if (fix1psf(img,Np,plist,it)) rval=1;
   return rval;
}

void calcpsf(FILE*fpsfs) {
#define minNpsf 50
#define goodNpsf 150
#define maxNpsf 300
   unsigned short **plist,*tlist;
   float *prank;
   int Np=0,Nt=0,i,j,it=0,img,cont=1,Ng;

   plist=(unsigned short**)calloc(Nstars,sizeof(unsigned short*));
   tlist=(unsigned short*)calloc(Nstars,sizeof(unsigned short));
   prank=(float*)calloc(Nstars,sizeof(float));
   if (!plist || !tlist) merr();
   for (i=0;i<Nstars;i++) {
      plist[i]=(unsigned short*)calloc(Nimg+1,sizeof(unsigned short));
      if (!plist[i]) merr();
   }
   sortstars();
   setindx();
   for (i=0;i<Nstars;i++) {
      addindx((int)stars[i].x,(int)stars[i].y,i);
      stars[i].flag=0;
   }
   for (i=0;i<Nstars;i++) {
      imgadd(i,0);
      if (((stars[i].s>=stars[i].ss*50 && Np<maxNpsf) || (stars[i].s>=stars[i].ss*10 && Np<goodNpsf) || Np<minNpsf) && goodstar(-1,i,rpsf[fld]+rphot[fld],rpsf[fld],0)) {
	 Ng=0;
	 plist[Np][0]=i;
	 fprintf(fpsfs,"%d %6.2f %6.2f",fld,stars[i].y,stars[i].x);
	 for (j=0;j<Nimg;j++) if (igoodstar(j,i,rpsf[fld],5)) {
	    plist[Np][j+1]=1;
	    fprintf(fpsfs," 1");
	    Ng++;
	 }
	 else {
	    plist[Np][j+1]=0;
	    fprintf(fpsfs," 0");
	 }
	 fprintf(fpsfs,"\n");
	 if (Ng>=(Nimg+1)/2) Np++;
      }
      imgsub(i,1);
   }
   while (Np>0 && cont) {
      double mn=0,sd=0;

      cont=0;
      for (i=0;i<Np;i++) mn+=stars[plist[i][0]].sh;
      mn/=Np;
      for (i=0;i<Np;i++) sd+=(stars[plist[i][0]].sh-mn)*(stars[plist[i][0]].sh-mn);
      sd=2.00*sqrt(sd/Np);
      for (i=0;i<Np;i++) if (fabs(stars[plist[i][0]].sh-mn)>sd) {
	 Np--;
	 memcpy(plist[i],plist[Np],2*(Nimg+1));
	 i--;
	 cont=1;
      }
   }
   for (i=0;i<Np;i++) markstars((int)stars[plist[i][0]].x,(int)stars[plist[i][0]].y);
   for (i=0;i<Nstars;i++) if (stars[i].flag) tlist[Nt++]=i;
   printf("%d PSF stars; %d neighbors\n",Np,Nt-Np);
   fflush(stdout);
   while (fixpsf(Np,plist,++it)) {
      for (i=0;i<Nt;i++) {
	 imgadd(tlist[i],1);
	 if (stars[tlist[i]].s>0) subindx((int)stars[tlist[i]].x,(int)stars[tlist[i]].y,tlist[i]);
	 phot((int)stars[tlist[i]].x,(int)stars[tlist[i]].y,tlist[i],0,0);
	 imgsub(tlist[i],1);
      }
   }
   if (it>=20) printf("Poorly converging PSF\n");
   printf("Central pixel PSF adjustments:\n");
   for (img=0;img<Nimg;img++) {
      j=0;
      for (i=0;i<Np;i++) if (plist[i][img+1]) j++;
      printf("image %d: %d stars, %f\n",img,j,poff[img][0][0]);
   }
   for (i=0;i<Nstars;i++) {
      imgadd(i,1);
      imgsub(i,0);
   }
   fflush(stdout);
   for (i=0;i<Nstars;i++) free(plist[i]);
   free(plist);
   free(tlist);
   free(prank);
}

int clean(int it) {
   int i,ii,j,b=0,ct=0,xx,yy,rad,x3,y3;
   float x,y,r,br;

   if (WARM) return 0;
   for (i=1;i<Nstars;i++) if (stars[i].x>=0) {
      rad=(int)(rmin[fld]+0.999);
      br=rmin[fld];
      for (xx=-rad;xx<=rad;xx++) if ((x3=(int)stars[i].x+xx)>=XMIN && x3<XMAX) for (yy=-rad;yy<=rad;yy++) if ((xx || yy) && (y3=(int)stars[i].y+yy)>=YMIN && y3<YMAX) for (ii=1;ii<=indx[x3][y3][0];ii++) {
	 j=indx[x3][y3][ii];
	 if (j<i && (x=fabs(stars[i].x-stars[j].x))<=rmin[fld] && (y=fabs(stars[i].y-stars[j].y))<=rmin[fld]) {
	    r=sqrt(x*x+y*y);
	    if (r<br) {
	       br=r;
	       b=j;
	    }
	 }
      }
      if (br<rmin[fld]) {
	 if (stars[i].s>0) subindx((int)stars[i].x,(int)stars[i].y,i);
	 if (stars[b].s>0) subindx((int)stars[b].x,(int)stars[b].y,b);
	 imgadd(i,0);
	 imgadd(b,0);
	 markstars((int)stars[i].x,(int)stars[i].y);
	 markstars((int)stars[b].x,(int)stars[b].y);
	 avgstar(b,stars+i);
	 stars[b].s*=2;
	 stars[b].ss*=1.4;
	 if (it==1) {
	    if (stars[b].type<4 && stars[i].type<4) stars[b].type=1;
	 }
	 else stars[b].type=2;
	 phot((int)stars[b].x,(int)stars[b].y,b,0,0);
	 imgsub(b,0);
	 stars[i].x=-1;
      }
   }
   ii=0;
   for (i=0;i<Nstars;i++) if (stars[i].x>=0 && stars[i].s>0) {
      if (ii<i) {
	 stype temp;
	 memcpy(&temp,stars+ii,sizeof(stype));
	 memcpy(stars+ii,stars+i,sizeof(stype));
	 memcpy(stars+i,&temp,sizeof(stype));
	 repindx((int)stars[ii].x,(int)stars[ii].y,i,ii);
      }
      ii++;
   }
   ct=Nstars-ii;
   Nstars=ii;
   if (ct) printf("%d stars eliminated\n",ct);
   fflush(stdout);
   return ct;
}

int solve(int it,FILE *fpsfs) {
   int i,reit=0;
   static int newpsf=0;

   if (it==1) newpsf=0;
   setindx();
   for (i=0;i<Nstars;i++) addindx((int)stars[i].x,(int)stars[i].y,i);
   for (i=0;i<Nstars;i++) if (stars[i].flag) {
      reit++;
      imgadd(i,0);
      subindx((int)stars[i].x,(int)stars[i].y,i);
      phot((int)stars[i].x,(int)stars[i].y,i,0,it);
      imgsub(i,0);
   }
   if (clean(it) || it<2) return 2;
   if (!newpsf && !(OPTS&4)) {
      calcpsf(fpsfs);
      newpsf=1;
      for (i=0;i<Nstars;i++) stars[i].flag=1;
      return 2;
   }
   if (reit) return 1;
   return 0;
}

void clearsnmap(void) {
   memset(snmap[0],0,2560000);
}

void setsnmap(int IMG,int x0,int x1,int y0,int y1) {
   int x,y,img;
   float m,s,ss,c,sh,ssky,cm,xx,yy;

   for (img=0;img<Nimg || (img==Nimg && IMG==Nimg);img++) if (IMG<0 || img==IMG) {
      shift(img,(x0+x1)/2+0.5,(y0+y1)/2+0.5,&xx,&yy);
      calc1psf(img,rphot[fld],xx,yy,NULL,NULL);
      for (x=x0;x<=x1;x++) if (x>=51 && x<=799) for (y=y0;y<=y1;y++) if (y>=51 && y<=799) {
	 eval1(img,x+0.5,y+0.5,&m,&s,&ss,&c,&sh,&ssky,&cm,0);
	 if (ss>0) snmap[x][y]+=s*fabs(s)/(ss*ss);
      }
   }
   for (x=x0;x<=x1;x++) if (x>=51 && x<=799) for (y=y0;y<=y1;y++) if (y>=51 && y<=799) {
      if (snmap[x][y]<=0) snmap[x][y]=0.;
      else snmap[x][y]=sqrt(snmap[x][y]);
   }
}

inline float onepeak(int x,int y) {
   if (x<=50 || y<=50 || x>=799 || y>=799) return 0;
   if (snmap[x][y]>0 && snmap[x][y]>=snmap[x-1][y-1] && snmap[x][y]>=snmap[x][y-1] && snmap[x][y]>=snmap[x+1][y-1] && snmap[x][y]>=snmap[x+1][y] && snmap[x][y]>=snmap[x+1][y+1] && snmap[x][y]>=snmap[x][y+1] && snmap[x][y]>=snmap[x-1][y+1]) return snmap[x][y];
   return 0.;
}

inline float peak(int x,int y,int strict) {
   if (x<=50 || y<=50 || x>=799 || y>=799) return 0;
   if (strict) {
      if (ran[x][y] || indx[x][y][0]) return 0;
      if (indx[x-1][y][0] || indx[x-1][y-1][0] || indx[x][y-1][0] || indx[x+1][y-1][0] || indx[x+1][y][0] || indx[x+1][y+1][0] || indx[x][y+1][0] || indx[x-1][y+1][0]) return 0;
   }
   return onepeak(x,y);
}

int alphot(int img,float x0,float y0,float*x,float*y) {
   float dp,s,ss,sky,chi,sh,cm,ssky,t;
   float ispars[4][2][2]={
      {{1.000000,-0.000200},{0.000200,1.000000}},
      {{0.020047,2.184653},{-2.184653,0.020047}},
      {{-2.184800,0.009070},{-0.009070,-2.184800}},
      {{0.012422,-2.185722},{2.185722,0.012422}}
   };
   int finding;

   cleargs(img);
   if (img==Nimg || (img==0 && Timg==Nimg)) {
      finding=1;
      if (Nstars>=NMAX) return 0;
      if (img==0) centroid(-1,(int)x0,(int)y0,x,y);
      else centroid(Nimg,(int)x0,(int)y0,x,y);
   }
   else {
      finding=0;
      *x=x0;
      *y=y0;
   }
   if (photsearch(img,(*x),(*y),x,y,&s,&ss,&sky,&chi,&sh,NULL,NULL,NULL,finding)) return 0;
   if (s<ss*5 || s<=0 || chi>3.5 || fabs(sh)>0.5) return 0;
   dp=1./sub[fld];
   while (dp>0.005) {
      int shrink=1;
      float t0,t1;

      eval1(img,(*x)-dp,(*y),&t0,&s,&ss,&chi,&sh,&ssky,&cm,1);
      eval1(img,(*x),(*y),&t,&s,&ss,&chi,&sh,&ssky,&cm,1);
      eval1(img,(*x)+dp,(*y),&t1,&s,&ss,&chi,&sh,&ssky,&cm,1);
      if (t0>t) {
	 (*x)-=dp;
	 shrink=0;
      }
      else if (t1>t) {
	 (*x)+=dp;
	 shrink=0;
      }
      else if (t0<t || t1<t) {
	 t=(t1-t0)/(4*t-2*t0-2*t1);
	 if (t<-.5) t=-.5;
	 else if (t>.5) t=.5;
	 (*x)+=t*dp;
      }
      if (*x<XMIN) *x=XMIN;
      if (*x>=XMAX) *x=XMAX-0.01;
      eval1(img,(*x),(*y)-dp,&t0,&s,&ss,&chi,&sh,&ssky,&cm,1);
      eval1(img,(*x),(*y),&t,&s,&ss,&chi,&sh,&ssky,&cm,1);
      eval1(img,(*x),(*y)+dp,&t1,&s,&ss,&chi,&sh,&ssky,&cm,1);
      if (t0>t) {
	 (*y)-=dp;
	 shrink=0;
      }
      else if (t1>t) {
	 (*y)+=dp;
	 shrink=0;
      }
      else if (t0<t || t1<t) {
	 t=(t1-t0)/(4*t-2*t0-2*t1);
	 if (t<-.5) t=-.5;
	 else if (t>.5) t=.5;
	 (*y)+=t*dp;
      }
      if (*y<YMIN) *y=YMIN;
      if (*y>=YMAX) *y=YMAX-0.01;
      if (shrink) dp*=.5;
      else dp*=.9;
   }
   eval1(img,(*x),(*y),&t,&s,&ss,&chi,&sh,&ssky,&cm,1);
   if (t<=0) return 0;
   if (s<ss*5 || s<=0 || chi>3.5 || fabs(sh)>0.5) return 0;
   if (getclass1(img,*x,*y,&s,&cm,sh)) return 0;
   if (finding) {
      addindx((int)(*x),(int)(*y),Nstars);
      stars[Nstars].x=*x;
      stars[Nstars].y=*y;
      Nstars++;
   }
   else {
      float dx,dy;
      dx=(*x)-x0;
      dy=(*y)-y0;
      (*x)=dx*ispars[fld][0][0]+dy*ispars[fld][0][1];
      (*y)=dx*ispars[fld][1][0]+dy*ispars[fld][1][1];
      if ((*x)*(*x)+(*y)*(*y)>9.) return 0;
   }
   return 1;
}

void align(void) {
   int img,i,x,y,N;
   float p,dx,dy,sig=0,sigmax=1.e30,sigmin=200;
typedef float dptype[3];
   dptype *list;

   if (Timg>Nimg) img=Nimg;
   else img=0;
   dpos[img][0]=dpos0[img][0];
   dpos[img][1]=dpos0[img][1];
   Nstars=0;
   setindx();
   setran(ran);
   clearsnmap();
   setsnmap(img,XMIN+1,XMAX-2,YMIN+1,YMAX-2);
   while (sigmin>5. && Nstars<250) {
      sigmin*=0.5;
      if (sigmin<5.) sigmin=5.;
      for (x=XMIN+1;x<XMAX-1;x++) for (y=YMIN+1;y<YMAX-1;y++) {
	 if ((p=onepeak(x,y))>=sigmin && p<sigmax) alphot(img,x,y,&dx,&dy);
      }
      sigmax=sigmin;
   }
   printf("%d stars for alignment\n",Nstars);
   fflush(stdout);
   list=(dptype*)calloc(Nstars,sizeof(dptype));
   if (!list) merr();
   for (img=0;img<Nimg;img++) if (img || Timg>Nimg) {
      N=0;
      dpos[img][0]=dpos0[img][0];
      dpos[img][1]=dpos0[img][1];
      setindx();
      for (i=0;i<Nstars;i++) if (alphot(img,stars[i].x,stars[i].y,&dx,&dy)) {
	 list[N][0]=dx;
	 list[N][1]=dy;
	 N++;
      }
      printf("%d matched, ",N);
      fflush(stdout);
      dx=dy=0;
      if (N) do {
	 dx=dy=sig=0;
	 for (i=0;i<N;i++) {
	    dx+=list[i][0];
	    dy+=list[i][1];
	 }
	 dx/=N; dy/=N;
	 for (i=0;i<N;i++) sig+=(list[i][2]=(list[i][0]-dx)*(list[i][0]-dx)+(list[i][1]-dy)*(list[i][1]-dy));
	 sig/=N;
	 if (sig<0.0001) sig=0.0001;
	 x=0;
	 for (i=0;i<N;i++) if (list[i][2]>sig*5.0625) {
	    N--;
	    memcpy(list[i],list[N],sizeof(dptype));
	    x=1;
	    i--;
	 }
      } while (N && x);
      dpos[img][0]+=dx;
      dpos[img][1]+=dy;
      printf("%d used, dx=%6.2f, dy=%6.2f, sig=%4.2f\n",N,dpos[img][1],dpos[img][0],sqrt(sig));
      fflush(stdout);
   }
   free(list);
}

void find(void) {
   int x,y;
   float p,sigmax=1.e30,sigmin=200,fsigm;

   printf("Finding stars: ");
   fflush(stdout);
   setindx();
   if (OPTS&512) {
      if (fld==0) fsigm=findsig*0.85;
      else fsigm=findsig*0.75;
   }
   else {
      if (fld==0) fsigm=findsig*0.9;
      else fsigm=findsig*0.8;
   }
   setran(ran);
   clearsnmap();
   setsnmap(-1,XMIN+1,XMAX-2,YMIN+1,YMAX-2);
   printf("."); fflush(stdout);
   while (sigmin>fsigm) {
      sigmin*=0.5;
      if (sigmin<fsigm) sigmin=fsigm;
      for (x=XMIN+1;x<XMAX-1;x++) for (y=YMIN+1;y<YMAX-1;y++) if ((p=peak(x,y,1))>=sigmin && p<sigmax) phot(x,y,-1,0,0);
      sigmax=sigmin;
   }
   printf("."); fflush(stdout);
   for (x=0;x<Nstars;x++) {
      imgsub(x,0);
      y=stars[x].flag;
      markstars(stars[x].x,stars[x].y);
      stars[x].flag=y;
   }
   printf("\n%d found\n",Nstars);
   fflush(stdout);
   clean(1);
}

void readwarm(char*fn) {
   FILE *f;
   int c,t;
   float x,y;

   setindx();
   setran(ran);
   if ((f=fopen(fn,"r"))==NULL) {
      printf("Cannot open %s\n",fn);
      fflush(stdout);
      return;
   }
   printf("Reading %s ...",fn);
   fflush(stdout);
   fscanf(f,"%d %f %f %d",&c,&x,&y,&t);
   while (!feof(f)) {
      if (c==fld) {
	 if (t<4) t=1;
	 stars[Nstars].x=y;
	 stars[Nstars].y=x;
	 stars[Nstars].type=t;
	 phot((int)x,(int)y,Nstars,0,0);
	 imgsub(Nstars,0);
	 Nstars++;
      }
      fscanf(f,"%d %f %f %d",&c,&x,&y,&t);
   }
   fclose(f);
   printf(" %d stars\n",Nstars);
   fflush(stdout);
}

float fitstar(int i,float x,float y) {
   int j;
   float m,s,ss,c,sh,ssky;
   static int afirst=1;
   static float *is,*iss,*icm;

   if (x<XMIN || y<YMIN || x>=XMAX || y>=YMAX) return 0;
   if (afirst) {
      afirst=0;
      is=(float*)calloc(Nimg,sizeof(float));
      iss=(float*)calloc(Nimg,sizeof(float));
      icm=(float*)calloc(Nimg,sizeof(float));
      if (!is || !iss || !icm) merr();
   }
   eval(-1,x,y,&m,&s,&ss,&c,&sh,&ssky,is,iss,icm,-1);
   if (m<=0) return 0;
   stars[i].s=s;
   stars[i].ss=ss;
   stars[i].chi=c;
   stars[i].sh=sh;
   stars[i].sky=ssky;
   for (j=0;j<Nimg;j++) {
      stars[i].is[j]=is[j];
      stars[i].iss[j]=iss[j];
      stars[i].icm[j]=icm[j];
   }
   return m;
}

void finalphot(int i) {
   int j;
   float dp;

   dp=1./sub[fld];
   cleargs(-1);
   if (!WARM) while (dp>0.005) {
      int shrink=1;
      float t0,t,t1;

      t0=fitstar(i,stars[i].x-dp,stars[i].y);
      t=fitstar(i,stars[i].x,stars[i].y);
      t1=fitstar(i,stars[i].x+dp,stars[i].y);
      if (t0>t) {
	 stars[i].x-=dp;
	 shrink=0;
      }
      else if (t1>t) {
	 stars[i].x+=dp;
	 shrink=0;
      }
      else if (t0<t || t1<t) {
	 t=(t1-t0)/(4*t-2*t0-2*t1);
	 if (t<-.5) t=-.5;
	 else if (t>.5) t=.5;
	 stars[i].x+=t*dp;
      }
      if (stars[i].x<XMIN) stars[i].x=XMIN;
      if (stars[i].x>=XMAX) stars[i].x=XMAX-0.01;
      t0=fitstar(i,stars[i].x,stars[i].y-dp);
      t=fitstar(i,stars[i].x,stars[i].y);
      t1=fitstar(i,stars[i].x,stars[i].y+dp);
      if (t0>t) {
	 stars[i].y-=dp;
	 shrink=0;
      }
      else if (t1>t) {
	 stars[i].y+=dp;
	 shrink=0;
      }
      else if (t0<t || t1<t) {
	 t=(t1-t0)/(4*t-2*t0-2*t1);
	 if (t<-.5) t=-.5;
	 else if (t>.5) t=.5;
	 stars[i].y+=t*dp;
      }
      if (stars[i].y<YMIN) stars[i].y=YMIN;
      if (stars[i].y>=YMAX) stars[i].y=YMAX-0.01;
      if (shrink) dp*=.5;
      else dp*=.9;
   }
   fitstar(i,stars[i].x,stars[i].y);
   j=getclass(i);
   if (j>0) stars[i].type=j;
}

//adapted from Numerical Recipes;
#define ITMAX 1000
#define ZEPS 1.0e-10
#define MOV3(a,b,c, d,e,f) (a)=(d);(b)=(e);(c)=(f);
#define SIGN(a,b) ((b) >= 0.0 ? fabs(a) : -fabs(a))
float dbrent(float ax, float bx, float cx, float (*f)(float,int,float*,float*,float), float (*df)(float,int,float*,float*,float), float tol, float *xmin,int fN,float*fapc,float*fsig,float priorwt) {
   int iter,ok1,ok2;
   float a,b,d=0,d1,d2,du,dv,dw,dx,e=0.0;
   float fu,fv,fw,fx,olde,tol1,tol2,u,u1,u2,v,w,x,xm;

   a=(ax < cx ? ax : cx);
   b=(ax > cx ? ax : cx);
   x=w=v=bx;
   fw=fv=fx=(*f)(x,fN,fapc,fsig,priorwt);
   dw=dv=dx=(*df)(x,fN,fapc,fsig,priorwt);
   for (iter=1;iter<=ITMAX;iter++) {
      xm=0.5*(a+b);
      tol1=tol*fabs(x)+ZEPS;
      tol2=2.0*tol1;
      if (fabs(x-xm) <= (tol2-0.5*(b-a))) {
	 *xmin=x;
	 return fx;
      }
      if (fabs(e) > tol1) {
	 d1=2.0*(b-a);
	 d2=d1;
	 if (dw != dx) d1=(w-x)*dx/(dx-dw);
	 if (dv != dx) d2=(v-x)*dx/(dx-dv);
	 u1=x+d1;
	 u2=x+d2;
	 ok1 = (a-u1)*(u1-b) > 0.0 && dx*d1 <= 0.0;
	 ok2 = (a-u2)*(u2-b) > 0.0 && dx*d2 <= 0.0;
	 olde=e;
	 e=d;
	 if (ok1 || ok2) {
	    if (ok1 && ok2) d=(fabs(d1) < fabs(d2) ? d1 : d2);
	    else if (ok1) d=d1;
	    else d=d2;
	    if (fabs(d) <= fabs(0.5*olde)) {
	       u=x+d;
	       if (u-a < tol2 || b-u < tol2) d=SIGN(tol1,xm-x);
	    }
	    else {d=0.5*(e=(dx >= 0.0 ? a-x : b-x));}
	 }
	 else {d=0.5*(e=(dx >= 0.0 ? a-x : b-x));}
      }
      else {d=0.5*(e=(dx >= 0.0 ? a-x : b-x));}
      if (fabs(d) >= tol1) {
	 u=x+d;
	 fu=(*f)(u,fN,fapc,fsig,priorwt);
      }
      else {
	 u=x+SIGN(tol1,d);
	 fu=(*f)(u,fN,fapc,fsig,priorwt);
	 if (fu > fx) {
	    *xmin=x;
	    return fx;
	 }
      }
      du=(*df)(u,fN,fapc,fsig,priorwt);
      if (fu <= fx) {
	 if (u >= x) a=x; else b=x;
	 MOV3(v,fv,dv, w,fw,dw)
	 MOV3(w,fw,dw, x,fx,dx)
	 MOV3(x,fx,dx, u,fu,du)
      }
      else {
	 if (u < x) a=u; else b=u;
	 if (fu <= fw || w == x) {
	    MOV3(v,fv,dv, w,fw,dw)
	    MOV3(w,fw,dw, u,fu,du)
	 }
	 else if (fu < fv || v == x || v == w) {
	    MOV3(v,fv,dv, u,fu,du)
	 }
      }
   }
   printf("**Aperture corrections failed to converge!\n");
   *xmin=u;
   return fu;
}
#undef SIGN
#undef ITMAX
#undef ZEPS
#undef MOV3

#define APMAX 200
#define APSIG 1.65
#define SIGCAP 10
float funcmin(float x,int N,float*apc,float*sig,float priorwt) {
   int i;
   float q=0.;
   if (priorwt>0.) q=log(1+x*x*0.5*priorwt*priorwt);
   for (i=0;i<N;i++) q+=log(1+(x-apc[i])*(x-apc[i])*0.5*sig[i]*sig[i]);
   return q/N;
}

float dfuncmin(float x,int N,float*apc,float*sig,float priorwt) {
   int i;
   float q=0.;
   if (priorwt>0.) q=x/(1./priorwt/priorwt+0.5*x*x);
   for (i=0;i<N;i++) q+=(x-apc[i])/(1/sig[i]/sig[i]+0.5*(x-apc[i])*(x-apc[i]));
   return q/N;
}

void bigapsky(int img,float x,float y,float*s,float*ss2) {
   int ix,iy,dx,dy,x1,y1,rs,rin,n=0;
   static float *list=NULL;

   ix=(int)x;
   iy=(int)y;
   if (fld) {
      rs=25;
      rin=15;
   }
   else {
      rs=50;
      rin=30;
   }
   if (list==NULL) {
      list=(float*)calloc(sizeof(float),(2*rs+1)*(2*rs+1));
      if (!list) merr();
   }
   for (dx=-rs,x1=(int)x-rs;dx<=rs;dx++,x1++) for (dy=-rs,y1=(int)y-rs;dy<=rs;dy++,y1++) if (ppixOK(img,x1,y1) && (abs(dy)>=rin || abs(dx)>=rin)) list[n++]=res[img][x1][y1]+sky[img][x1][y1];
   *s=calcskyval(n,list,img,ss2);
   if (*s<1.e30) (*s)-=sky[img][ix][iy];
   return;
}

void apsky(int img,float x,float y,float*s,float*ss2) {
   int ix,iy,dx,dy,x1,y1,rs,rin;
   float I=0.,P=0.,R=0.,PR=0.,PP=0.,n,r,p;

   if (!(OPTS&2)) {
      bigapsky(img,x,y,s,ss2);
      return;
   }
   ix=(int)x;
   iy=(int)y;
   if (ix<0) ix=0; if (ix>799) ix=799;
   if (iy<0) iy=0; if (iy>799) iy=799;
   rs=rpsf[fld];
   rin=rpsf[fld]/2;
   calc1psf(img,rpsf[fld],x,y,NULL,NULL);
   for (dx=-rs,x1=(int)x-rs;dx<=rs;dx++,x1++) for (dy=-rs,y1=(int)y-rs;dy<=rs;dy++,y1++) if (ppixOK(img,x1,y1) && (abs(dy)>=rin || abs(dx)>=rin)) {
      n=1./noise(img,x1,y1);
      if (abs(dx)<=rpsf[fld] && abs(dy)<=rpsf[fld]) p=tpsf[dx][dy];
      else p=0.;
      r=res[img][x1][y1]+sky[img][x1][y1];
      I+=n;
      P+=p*n;
      R+=r*n;
      PR+=p*r*n;
      PP+=p*p*n;
   }
   n=I*PP-P*P;
   if (!n) *s=1.e30;
   else *s=(R*PP-PR*P)/n-sky[img][ix][iy];
   if (!n || !P) *ss2=1.e30;
   else *ss2=PP/n/n*(PP*I-P*P);
   return;
}

void getapcor_old(int img,FILE *fapcor) {
   int N=0,i,x,y,ix,iy;
   float xc,yc,apc[APMAX+1],sig[APMAX+1],sq[APMAX+1],mn,r,hisig[SIGCAP],iapc,isig,isq;
   float tx,ty,ssky,dssky2,rtot,fg,ftot;

   if (!img) {
      printf("Aperture corrections:\n");
      fflush(stdout);
   }
   sortstars();
   setindx();
   for (x=0;x<SIGCAP;x++) hisig[x]=0.0;
   for (i=0;i<Nstars;i++) {
      addindx((int)stars[i].x,(int)stars[i].y,i);
      stars[i].flag=0;
      imgadd(i,0);
      if (goodstar(img,i,(int)apsize[fld]+3,(int)apsize[fld]+1,1)) {
	 shift(img,stars[i].x,stars[i].y,&tx,&ty);
	 calc1psf(img,rpsf[fld],tx,ty,NULL,NULL);
	 ix=(int)tx; xc=tx-0.5;
	 iy=(int)ty; yc=ty-0.5;
	 apsky(img,tx,ty,&ssky,&dssky2);
	 iapc=isig=rtot=fg=ftot=0.;
	 for (x=ix-(int)(apsize[fld])-1;x<=ix+(int)(apsize[fld])+1;x++) for (y=iy-(int)(apsize[fld])-1;y<=iy+(int)(apsize[fld])+1;y++) {
	    r=0.465+apsize[fld]-sqrt((x-xc)*(x-xc)+(y-yc)*(y-yc));
	    if (r>1) r=1;
	    if (r>0) {
	       if (abs(x-ix)<=rpsf[fld] && abs(y-iy)<=rpsf[fld]) {
		  if (data[img][x][y]>DMIN[img] && data[img][x][y]<0.855*DMAX[img]) {
		     iapc+=(res[img][x][y]-ssky)*r;
		     isig+=(noise(img,x,y))*r;
		     rtot+=r;
		     fg+=tpsf[x-ix][y-iy]*r;
		  }
		  ftot+=tpsf[x-ix][y-iy]*r;
	       }
	       else if (data[img][x][y]>DMIN[img] && data[img][x][y]<0.855*DMAX[img]) {
		  iapc+=(res[img][x][y]-ssky)*r;
		  isig+=(noise(img,x,y))*r;
		  rtot+=r;
	       }
	       else {
		  float ns=0,rs=0;
		  int ct=0,dx,dy;
		  for (dx=-1;dx<=1;dx++) for (dy=-1;dy<=1;dy++) if (x+dx>=50 && x+dx<799 && y+dy>=50 && y+dy<799 && data[img][x+dx][y+dy]>DMIN[img] && data[img][x+dx][y+dy]<DMAX[img]*0.855) {
		     ns+=noise(img,x+dx,y+dy);
		     rs+=res[img][x+dx][y+dy];
		     ct++;
		  }
		  if (ct) {
		     iapc+=(rs/ct-ssky)*r;
		     isig+=(ns/ct)*r;
		     rtot+=r;
		  }
	       }
	    }
	 }
	 isig+=rtot*rtot*dssky2;
	 if (isig>0 && iapc>0 && stars[i].is[img]>0) {
	    isig=sqrt(isig/iapc/iapc+stars[i].iss[img]*stars[i].iss[img]/stars[i].is[img]/stars[i].is[img])*1.0857+0.01;
	    if (stars[i].chi>1.) isig*=stars[i].chi;
	    iapc*=ftot/fg;
	    iapc=-2.5*log10(iapc/stars[i].is[img]*stars[i].icm[img])-apcor0[fld]-smapcor0[fld]-(appsfm[fld]+smappsfm[fld])*poff[img][0][0];
	    isq=1./isig;
	    if (stars[i].sky>=0) isq/=1+stars[i].sky*0.1;
	    fprintf(fapcor,"%2d %d %6.2f %6.2f %6.3f %6.3f %6.3f %6.2f\n",img,fld,stars[i].y,stars[i].x,-2.5*log10(stars[i].is[img])+filters[filt[img]].zero,iapc,isig,isq);
	    if (fabs(iapc)<=0.25) {
	       isig=1./isig;
	       for (x=N;x>0 && isq>sq[x-1];x--) {
		  apc[x]=apc[x-1];
		  sig[x]=sig[x-1];
		  sq[x]=sq[x-1];
	       }
	       apc[x]=iapc;
	       sig[x]=isig;
	       sq[x]=isq;
	       if (N<APMAX) N++;
	    }
	 }
      }
      imgsub(i,0);
   }
   while (N>5 && sq[N-1]<N*0.75) N--;
   for (i=0;i<N;i++) {
      for (x=SIGCAP;x>0 && sig[i]>hisig[x-1];x--) if (x<SIGCAP) hisig[x]=hisig[x-1];
      if (x<SIGCAP) hisig[x]=sig[i];
   }
   for (x=SIGCAP-1;x>=0 && hisig[x]==0;x--);
   if (x>=0) for (i=0;i<N;i++) if (sig[i]>hisig[x]) sig[i]=hisig[x];
   i=1;
   while (N && i) {
      dbrent(-0.75,0.,0.75,&funcmin,&dfuncmin,0.01,&mn,N,apc,sig,apprior);
      i=0;
      for (x=0;x<N;) if (fabs(apc[x]-mn)>2./sig[x]+0.03) {
	 N--;
	 apc[x]=apc[N];
	 sig[x]=sig[N];
	 i=1;
      }
      else x++;
   }
   printf("image %d: %d stars, %6.3f\n",img,N,mn);
   fflush(stdout);
   fflush(fapcor);
   apcor[img]*=pow(10,-0.4*mn-0.4*(appsfm[fld]+smappsfm[fld])*poff[img][0][0]);
}

void getapcor(int img,FILE *fapcor) {
   int N=0,smN=0,i,x,y,ix,iy;
   float xc,yc,apc[APMAX+1],sig[APMAX+1],sig0[APMAX+1],sq[APMAX+1],mn=0,r,hisig[SIGCAP],iapc,isig,isq,err=0,den;
   float tx,ty,ssky,dssky2,rtot,fg,ftot;
   float smr,smiapc,smisig,smisq,smrtot,smfg,smftot,smapc[APMAX+1],smsig[APMAX+1],smsig0[APMAX+1],smsq[APMAX+1],smmn=0,smerr=0;

   if (!img) {
      printf("Aperture corrections:\n");
      fflush(stdout);
   }
   sortstars();
   setindx();
   for (i=0;i<Nstars;i++) {
      addindx((int)stars[i].x,(int)stars[i].y,i);
      stars[i].flag=0;
      imgadd(i,0);
      if (goodstar(img,i,(int)apsize[fld]+3,(int)apsize[fld]+1,1)) {
	 shift(img,stars[i].x,stars[i].y,&tx,&ty);
	 calc1psf(img,rpsf[fld],tx,ty,NULL,NULL);
	 ix=(int)tx; xc=tx-0.5;
	 iy=(int)ty; yc=ty-0.5;
	 apsky(img,tx,ty,&ssky,&dssky2);
	 iapc=isig=rtot=fg=ftot=0.;
	 smiapc=smisig=smrtot=smfg=smftot=0.;
	 for (x=ix-(int)(apsize[fld])-1;x<=ix+(int)(apsize[fld])+1;x++) for (y=iy-(int)(apsize[fld])-1;y<=iy+(int)(apsize[fld])+1;y++) {
	    r=0.465+apsize[fld]-sqrt((x-xc)*(x-xc)+(y-yc)*(y-yc));
	    smr=0.465+smapsize[fld]-sqrt((x-xc)*(x-xc)+(y-yc)*(y-yc));
	    if (smr>1) smr=1;
	    else if (smr<0) smr=0;
	    if (r>1) r=1;
	    if (r>0) {
	       if (abs(x-ix)<=rpsf[fld] && abs(y-iy)<=rpsf[fld]) {
		  if (data[img][x][y]>DMIN[img] && data[img][x][y]<0.855*DMAX[img]) {
		     iapc+=(res[img][x][y]-ssky)*r;
		     isig+=(noise(img,x,y))*r;
		     rtot+=r;
		     fg+=tpsf[x-ix][y-iy]*r;
		     smiapc+=(res[img][x][y]-ssky)*smr;
		     smisig+=(noise(img,x,y))*smr;
		     smrtot+=smr;
		     smfg+=tpsf[x-ix][y-iy]*smr;
		  }
		  ftot+=tpsf[x-ix][y-iy]*r;
		  smftot+=tpsf[x-ix][y-iy]*smr;
	       }
	       else if (data[img][x][y]>DMIN[img] && data[img][x][y]<0.855*DMAX[img]) {
		  iapc+=(res[img][x][y]-ssky)*r;
		  isig+=(noise(img,x,y))*r;
		  rtot+=r;
		  smiapc+=(res[img][x][y]-ssky)*smr;
		  smisig+=(noise(img,x,y))*smr;
		  smrtot+=smr;
	       }
	       else {
		  float ns=0,rs=0;
		  int ct=0,dx,dy;
		  for (dx=-1;dx<=1;dx++) for (dy=-1;dy<=1;dy++) if (x+dx>=50 && x+dx<799 && y+dy>=50 && y+dy<799 && data[img][x+dx][y+dy]>DMIN[img] && data[img][x+dx][y+dy]<DMAX[img]*0.855) {
		     ns+=noise(img,x+dx,y+dy);
		     rs+=res[img][x+dx][y+dy];
		     ct++;
		  }
		  if (ct) {
		     iapc+=(rs/ct-ssky)*r;
		     isig+=(ns/ct)*r;
		     rtot+=r;
		     smiapc+=(rs/ct-ssky)*smr;
		     smisig+=(ns/ct)*smr;
		     smrtot+=smr;
		  }
	       }
	    }
	 }
	 isig+=rtot*rtot*dssky2;
	 smisig+=smrtot*smrtot*dssky2;
	 if (isig>0 && iapc>0 && stars[i].is[img]>0) {
	    isig=sqrt(isig/iapc/iapc+smisig/smiapc/smiapc)*1.0857+0.01;
	    smisig=sqrt(smisig/smiapc/smiapc+stars[i].iss[img]*stars[i].iss[img]/stars[i].is[img]/stars[i].is[img])*1.0857+0.01;
	    if (stars[i].chi>1.) {
	       isig*=stars[i].chi;
	       smisig*=stars[i].chi;
	    }
	    iapc*=ftot/fg;
	    smiapc/=smfg;
	    iapc=-2.5*log10(iapc/smiapc)-apcor0[fld]-appsfm[fld]*poff[img][0][0];
	    smiapc=-2.5*log10(smiapc/stars[i].is[img]*stars[i].icm[img])-smapcor0[fld]-smappsfm[fld]*poff[img][0][0];
	    isq=1./isig;
	    smisq=1./smisig;
	    if (stars[i].sky>=0) {
	       isq/=1+stars[i].sky*0.1;
	       smisq/=1+stars[i].sky*0.1;
	    }
	    fprintf(fapcor,"%2d %d %6.2f %6.2f %6.3f %6.3f %6.3f %6.2f %6.3f %6.3f %6.2f\n",img,fld,stars[i].y,stars[i].x,-2.5*log10(stars[i].is[img])+filters[filt[img]].zero,iapc,isig,isq,smiapc,smisig,smisq);
	    if (fabs(iapc)<=0.25) {
	       isig=1./isig;
	       for (x=N;x>0 && isq>sq[x-1];x--) {
		  apc[x]=apc[x-1];
		  sig[x]=sig[x-1];
		  sq[x]=sq[x-1];
	       }
	       apc[x]=iapc;
	       sig[x]=isig;
	       sq[x]=isq;
	       if (N<APMAX) N++;
	    }
	    if (fabs(smiapc)<=0.25) {
	       smisig=1./smisig;
	       for (x=smN;x>0 && smisq>smsq[x-1];x--) {
		  smapc[x]=smapc[x-1];
		  smsig[x]=smsig[x-1];
		  smsq[x]=smsq[x-1];
	       }
	       smapc[x]=smiapc;
	       smsig[x]=smisig;
	       smsq[x]=smisq;
	       if (smN<APMAX) smN++;
	    }
	 }
      }
      imgsub(i,0);
   }
   while (N>5 && sq[N-1]<N*0.75) N--;
   while (smN>5 && sq[smN-1]<smN*0.75) smN--;
   for (x=0;x<SIGCAP;x++) hisig[x]=0.0;
   for (i=0;i<N;i++) {
      sig0[i]=sig[i];
      for (x=SIGCAP;x>0 && sig[i]>hisig[x-1];x--) if (x<SIGCAP) hisig[x]=hisig[x-1];
      if (x<SIGCAP) hisig[x]=sig[i];
   }
   for (x=SIGCAP-1;x>=0 && hisig[x]==0;x--);
   if (x>=0) for (i=0;i<N;i++) if (sig[i]>hisig[x]) sig[i]=hisig[x];
   i=1;
   den=SQR(apprior);
   while (N && i) {
      dbrent(-0.75,0.,0.75,&funcmin,&dfuncmin,0.01,&mn,N,apc,sig,apprior);
      i=0;
      err=SQR(mn*apprior);
      den=SQR(apprior);
      for (x=0;x<N;) if (fabs(apc[x]-mn)>2.5/sig[x]+0.05) {
	 N--;
	 apc[x]=apc[N];
	 sig[x]=sig[N];
	 sig0[x]=sig0[N];
	 i=1;
      }
      else {
	 err+=SQR((apc[x]-mn)*sig0[x]);
	 den+=SQR(sig0[x]);
	 x++;
      }
   }
   if (apprior==0.) {
      if (N>1 && err>N-1) err=sqrt(err/den/(N-1.));
      else if (N>1) err=sqrt(1./den);
      else if (N==1) err=1/sig[0];
      else err=9.999;
   }
   else if (N>0 && err>N) err=sqrt(err/den/N);
   else err=sqrt(1./den);
   for (x=0;x<SIGCAP;x++) hisig[x]=0.0;
   for (i=0;i<smN;i++) {
      smsig0[i]=smsig[i];
      for (x=SIGCAP;x>0 && smsig[i]>hisig[x-1];x--) if (x<SIGCAP) hisig[x]=hisig[x-1];
      if (x<SIGCAP) hisig[x]=smsig[i];
   }
   for (x=SIGCAP-1;x>=0 && hisig[x]==0;x--);
   if (x>=0) for (i=0;i<smN;i++) if (smsig[i]>hisig[x]) smsig[i]=hisig[x];
   i=1;
   den=SQR(smapprior);
   while (smN && i) {
      dbrent(-0.75,0.,0.75,&funcmin,&dfuncmin,0.01,&smmn,smN,smapc,smsig,smapprior);
      i=0;
      smerr=SQR(smmn*smapprior);
      den=SQR(smapprior);
      for (x=0;x<smN;) if (fabs(smapc[x]-smmn)>2.5/smsig[x]+0.05) {
	 smN--;
	 smapc[x]=smapc[smN];
	 smsig[x]=smsig[smN];
	 smsig0[x]=smsig0[smN];
	 i=1;
      }
      else {
	 smerr+=SQR((smapc[x]-smmn)*smsig0[x]);
	 den+=SQR(smsig0[x]);
	 x++;
      }
   }
   if (smapprior==0.) {
      if (smN>1 && smerr>smN-1) smerr=sqrt(smerr/den/(smN-1.));
      else if (smN>1) smerr=sqrt(1./den);
      else if (smN==1) smerr=1/smsig[0];
      else smerr=9.999;
   }
   else if (smN>0 && smerr>smN) smerr=sqrt(smerr/den/smN);
   else smerr=sqrt(1./den);
   printf("image %d: %d %5.3f +/- %5.3f small + %d %5.3f +/- %5.3f large = %6.3f\n",img,smN,smmn,smerr,N,mn,err,mn+smmn);
   fflush(stdout);
   fflush(fapcor);
   apcor[img]*=pow(10,-0.4*(mn+smmn)-0.4*(appsfm[fld]+smappsfm[fld])*poff[img][0][0]);
}
#undef APMAX
#undef APSIG0
#undef APSIG1
#undef APSIG

void calcapcor(int img) {
   float mn;

   if (!img) {
      printf("Aperture corrections:\n");
      fflush(stdout);
   }
   mn=(appsfm[fld]+smappsfm[fld])*poff[img][0][0];
   printf("image %d: %6.3f\n",img,mn);
   fflush(stdout);
   apcor[img]*=pow(10,-0.4*mn);
}

#define PASTEP 5
#define NPA 18
typedef float rndtype[NPA];
void get1round(int i,int img,rndtype rnd,rndtype wt) {
   int ix,iy,iix,iiy,x1,y1,dx,dy,fx,fy,pa;
   float x,y,mr,mr2,trnd,ssky,s=0.,ss=0.,c=0.,fpa,a,b;

   shift(img,stars[i].x,stars[i].y,&x,&y);
   ix=iix=(int)x;
   iy=iiy=(int)y;
   if (iix<0) iix=0; if (iix>799) iix=799;
   if (iiy<0) iiy=0; if (iiy>799) iiy=799;
   ssky=getsky(img,iix,iiy);
   calc1psf(img,rpsf[fld],x,y,&fx,&fy);
   for (x1=ix-rphot[fld],dx=-rphot[fld];x1<=ix+rphot[fld];x1++,dx++) for (y1=iy+circ[0][fld][fx][fy][dx];y1<=iy+circ[1][fld][fx][fy][dx];y1++) if (ppixOK(img,x1,y1)) {
      dy=y1-iy;
      if (OPTS&1) {
	 c+=tpsf[dx][dy]*cwt[fld][fx][fy][dx][dy];
	 s+=(res[img][x1][y1]-ssky)*cwt[fld][fx][fy][dx][dy];
	 ss+=noise(img,x1,y1)*cwt[fld][fx][fy][dx][dy];
      }
      else {
	 a=tpsf[dx][dy];
	 b=a*cwt[fld][fx][fy][dx][dy]/noise(img,x1,y1);
	 if (a>0) {
	    s+=(res[img][x1][y1]-ssky)*b;
	    a*=b;
	    c+=a;
	    ss+=a*cwt[fld][fx][fy][dx][dy];
	 }
      }
   }
   if (s<=0 || ss<=0 || c==0) {
      for (pa=0;pa<NPA;pa++) {
	 if (wt) wt[pa]=0.;
	 rnd[pa]=0.;
      }
      return;
   }
   s/=c;
   ss=sqrt(ss)/c;
   for (pa=0;pa<NPA;pa++) {
      fpa=pa*0.017453293*PASTEP;
      trnd=mr=mr2=0.;
      for (x1=ix-rpsf[fld],dx=-rpsf[fld];x1<=ix+rpsf[fld];x1++,dx++) for (y1=iy-rpsf[fld],dy=-rpsf[fld];y1<=iy+rpsf[fld];y1++,dy++) if ((dx!=0 || dy!=0) && dx*dx+dy*dy<(rpsf[fld]+0.465)*(rpsf[fld]+0.465)) {
	 if (ppixOK(img,x1,y1)) {
	    a=tpsf[dx][dy]*s;
	    if (a>0) {
	       mr+=a;
	       mr2+=res[img][x1][y1]-ssky;
	       trnd+=(res[img][x1][y1]-ssky-a)*cos(2*(atan2(dx,dy)-fpa));
	    }
	 }
	 else if (ppixOK(img,ix-dx,iy-dy)) {
	    a=tpsf[-dx][-dy]*s;
	    if (a>0) {
	       mr+=a;
	       mr2+=res[img][ix-dx][iy-dy]-ssky;
	       trnd+=(res[img][ix-dx][iy-dy]-ssky-a)*cos(2*(atan2(-dx,-dy)-fpa));
	    }
	 }
      }
      if (mr2>mr) mr=mr2;
      if (mr>0) {
	 rnd[pa]=trnd/mr;
	 if (wt) wt[pa]=s/ss/ss;
      }
      else {
	 rnd[pa]=0.;
	 if (wt) wt[pa]=0.;
      }
   }
}

void getround(int i,int*pa,float*rnd,float*irnd,float*frnd) {
   int img,ip,f,pm=1;
   float tr,wt;
   rndtype *rsave,*wsave;

   rsave=(rndtype*)calloc(sizeof(rndtype),Nimg);
   wsave=(rndtype*)calloc(sizeof(rndtype),Nimg);
   if (!rsave || !wsave) merr();
   *pa=-1;
   *rnd=-1.;
   for (img=0;img<Nimg;img++) irnd[img]=-1.;
   for (img=0;img<NFILTERS;img++) frnd[img]=-1.;
   for (img=0;img<Nimg;img++) get1round(i,img,rsave[img],wsave[img]);
   for (ip=0;ip<NPA;ip++) {
      tr=wt=0.;
      for (img=0;img<Nimg;img++) {
	 tr+=rsave[img][ip]*wsave[img][ip];
	 wt+=wsave[img][ip];
      }
      if (wt>0 && (*pa<0 || fabs(tr/wt)>*rnd)) {
	 if (tr>=0) {
	    *rnd=tr/wt;
	    *pa=ip*PASTEP;
	    pm=1;
	 }
	 else {
	    *rnd=-tr/wt;
	    *pa=ip*PASTEP+90;
	    pm=-1;
	 }
	 for (img=0;img<Nimg;img++) irnd[img]=pm*rsave[img][ip];
	 for (f=0;f<NFILTERS;f++) {
	    tr=wt=0.;
	    for (img=0;img<Nimg;img++) if (filt[img]==f) {
	       tr+=rsave[img][ip]*wsave[img][ip];
	       wt+=wsave[img][ip];
	    }
	    if (wt<=0) frnd[f]=-1.;
	    else frnd[f]=pm*tr/wt;
	 }
      }
   }
   free(rsave);
   free(wsave);
}
#undef PASTEP
#undef NPA

void eval1star(FILE*fall,int img,float x,float y) {
   int ix,iy,x1,y1,dx,dy,fx,fy;
   float a,b,n,d,ssky;

   shift(img,x,y,&x,&y);
   ix=(int)x;
   iy=(int)y;
   ssky=getsky(img,ix,iy);
   calc1psf(img,rphot[fld],x,y,&fx,&fy);
   //normal PSF fit;
   n=d=0;
   for (x1=ix-rphot[fld],dx=-rphot[fld];x1<=ix+rphot[fld];x1++,dx++) for (y1=iy+circ[0][fld][fx][fy][dx];y1<=iy+circ[1][fld][fx][fy][dx];y1++) if (ppixOK(img,x1,y1)) {
      dy=y1-iy;
      if ((a=tpsf[dx][dy])>0) {
	 b=a*cwt[fld][fx][fy][dx][dy]/noise(img,x1,y1);
	 n+=(res[img][x1][y1]-ssky)*b;
	 d+=a*b;
      }
   }
   if (n>0 && d>0) n/=d;
   else n=0;
   if (n<999999.9) fprintf(fall," %8.1f",n);
   else fprintf(fall," %8.2e",n);
   //small aperture;
   n=d=0;
   for (x1=ix-rphot[fld],dx=-rphot[fld];x1<=ix+rphot[fld];x1++,dx++) for (y1=iy+circ[0][fld][fx][fy][dx];y1<=iy+circ[1][fld][fx][fy][dx];y1++) if (ppixOK(img,x1,y1)) {
      dy=y1-iy;
      n+=(res[img][x1][y1]-ssky)*cwt[fld][fx][fy][dx][dy];
      d+=tpsf[dx][dy]*cwt[fld][fx][fy][dx][dy];
   }
   a=n;
   if (n>0 && d>0) n/=d;
   else n=0;
   if (n<999999.9) fprintf(fall," %8.1f",n);
   else fprintf(fall," %8.2e",n);
   //plain small aperture;
   if (a<0) fprintf(fall,"      0.0");
   else if (a<999999.9) fprintf(fall," %8.1f",a);
   else fprintf(fall," %8.2e",a);
}

int out1star(int i,FILE*fout,FILE*fall,int fake) {
   int j,k,NFF,SBAD=0,pa;
   stype t;
   double mag[NFILTERS],dmag[NFILTERS],tmag[NFILTERS];
   float tx,ty,rnd,frnd[NFILTERS];
   static int NPF[NFILTERS],first=1;
   static float *irnd=NULL;

   if (irnd==NULL) {
      irnd=(float*)calloc(sizeof(float),Nimg);
      if (!irnd) merr();
   }
   if (first) {
      for (j=0;j<NFILTERS;j++) NPF[j]=0;
      for (j=0;j<Nimg;j++) NPF[filt[j]]++;
      first=0;
   }
   if (stars[i].s<=0 || stars[i].s<minsig*stars[i].ss || stars[i].x<XMIN2 || stars[i].x>=XMAX2 || stars[i].y<YMIN2 || stars[i].y>=YMAX2) {
      if (fake || WARM) SBAD=1;
      else return 0;
   }
   if (stars[i].type<4 && !starOK(i,-1)) stars[i].type=3;
   memcpy(&t,stars+i,sizeof(stype));
   NFF=0;
   cleargs(-1);
   if (!SBAD) for (j=0;j<NFILTERS;j++) {
      mag[j]=tmag[j]=dmag[j]=99.999;
      if (NPF[j]) {
	 float m,s,ss,c,sh,ssky;
	 if (t.type>3) {
	    if (t.type==4) t4_eval(i,j,-1,&s,&ss,&c,&ssky);
	    else t5_eval(i,j,-1,&s,&ss,&c,&ssky);
	    sh=t.sh;
	 }
	 else eval(j,t.x,t.y,&m,&s,&ss,&c,&sh,&ssky,NULL,NULL,NULL,i);
	 if (s>0 && s/ss>=outsig) {
	    NFF++;
	    mag[j]=-2.5*log10(s)+filters[j].zero;
	    dmag[j]=ss/s*1.0857362;
	 }
      }
   }
   if (!SBAD && !NFF) {
      if (fake || WARM) SBAD=1;
      else return 0;
   }
   if (!SBAD) transform(mag,dmag,tmag);
   cleargs(-1);
   getround(i,&pa,&rnd,irnd,frnd);
   if (t.s<=0) fprintf(fout,"%d %6.2f %6.2f   0.00     0.0  0.000  0.000   0 1",fld,t.y,t.x);
   else fprintf(fout,"%d %6.2f %6.2f %6.2f %7.1f %6.3f %6.3f %3d %d",fld,t.y,t.x,t.chi,t.s/t.ss,t.sh,rnd,pa,t.type);
   for (j=0;j<NFILTERS;j++) if (NPF[j]>1) {
      float m,s,s0,ss,c,sh,ssky,cm=0;
      if (SBAD) s=0;
      else if (t.type>3) {
	 NOSUBTRACT=1;
	 if (t.type==4) t4_eval(i,j,-1,&s0,&ss,&c,&ssky);
	 else t5_eval(i,j,-1,&s0,&ss,&c,&ssky);
	 NOSUBTRACT=0;
	 if (t.type==4) t4_eval(i,j,-1,&s,&ss,&c,&ssky);
	 else t5_eval(i,j,-1,&s,&ss,&c,&ssky);
	 sh=t.sh;
      }
      else {
	 NOSUBTRACT=1;
	 eval(j,t.x,t.y,&m,&s0,&ss,&c,&sh,&ssky,NULL,NULL,NULL,i);
	 NOSUBTRACT=0;
	 eval(j,t.x,t.y,&m,&s,&ss,&c,&sh,&ssky,NULL,NULL,NULL,i);
      }
      if (!SBAD) for (k=0;k<Nimg;k++) if (filt[k]==j) cm+=EXP[k]/corr[IGN[k]][fld];
      cm=1./cm;
      if (s/cm<9999.995) fprintf(fout,"  %8.3f",s/cm);
      else if (s/cm<99999.95) fprintf(fout,"  %8.2f",s/cm);
      else if (s/cm<999999.5) fprintf(fout,"  %8.1f",s/cm);
      else fprintf(fout,"  %8.2e",s/cm);
      if (ssky/cm<99.9) fprintf(fout," %6.3f",ssky/cm);
      else if (ssky/cm<999.9) fprintf(fout," %6.2f",ssky/cm);
      else fprintf(fout," %6.1f",ssky/cm);
      if (s>0 && s/ss>=outsig) fprintf(fout," %6.3f %6.3f %6.3f %6.2f %7.1f %6.3f %6.3f %6.3f",-2.5*log10(s)+filters[j].zero,tmag[j],ss/s*1.0857362,c,s/ss,sh,frnd[j],2.5*log10(s0/s));
      else fprintf(fout," 99.999 99.999  0.000   0.00     0.0  0.000  0.000  0.000");
   }
   for (j=0;j<Nimg;j++) if (NPF[filt[j]]) {
      cleargs(j);
      shift(j,t.x,t.y,&tx,&ty);
      if (!SBAD && starOK(i,j) && tx>=50 && tx<799 && ty>=50 && ty<799) {
	 float m,s,s0,ss,c,sh,ssky,cm,m2;
	 if (t.type>3) {
	    NOSUBTRACT=1;
	    if (t.type==4) t4_eval(i,-1,j,&s0,&ss,&c,&ssky);
	    else t5_eval(i,-1,j,&s0,&ss,&c,&ssky);
	    NOSUBTRACT=0;
	    if (t.type==4) t4_eval(i,-1,j,&s,&ss,&c,&ssky);
	    else t5_eval(i,-1,j,&s,&ss,&c,&ssky);
	    sh=t.sh;
	    cm=stars[i].icm[j];
	 }
	 else {
	    NOSUBTRACT=1;
	    eval1(j,t.x,t.y,&m,&s0,&ss,&c,&sh,&ssky,&cm,1);
	    NOSUBTRACT=0;
	    eval1(j,t.x,t.y,&m,&s,&ss,&c,&sh,&ssky,&cm,1);
	 }
	 if (s/cm<9999.995) fprintf(fout,"  %8.3f",s/cm);
	 else if (s/cm<99999.95) fprintf(fout,"  %8.2f",s/cm);
	 else if (s/cm<999999.5) fprintf(fout,"  %8.1f",s/cm);
	 else fprintf(fout,"  %8.2e",s/cm);
	 if (ssky/cm<99.9) fprintf(fout," %6.3f",ssky/cm);
	 else if (ssky/cm<999.9) fprintf(fout," %6.2f",ssky/cm);
	 else fprintf(fout," %6.1f",ssky/cm);
	 if (s>0 && s/ss>=outsig) {
	    m=-2.5*log10(s)+filters[filt[j]].zero;
	    if (tmag[filt[j]]>90) m2=99.999;
	    else m2=m+tmag[filt[j]]-mag[filt[j]];
	    fprintf(fout," %6.3f %6.3f %6.3f %6.2f %7.1f %6.3f %6.3f %6.3f",m,m2,ss/s*1.0857362,c,s/ss,sh,irnd[j],2.5*log10(s0/s));
	 }
	 else fprintf(fout," 99.999 99.999  0.000   0.00     0.0  0.000  0.000  0.000");
	 if (fall) eval1star(fall,j,t.x,t.y);
      }
      else if (tx>=50 && tx<799 && ty>=50 && ty<799) {
	 float m,s,ss,c,sh,ssky,cm;
	 if (t.type>3) {
	    if (t.type==4) t4_eval(i,-1,j,&s,&ss,&c,&ssky);
	    else t5_eval(i,-1,j,&s,&ss,&c,&ssky);
	    sh=t.sh;
	    cm=stars[i].icm[j];
	 }
	 else eval1(j,t.x,t.y,&m,&s,&ss,&c,&sh,&ssky,&cm,1);
	 if (s/cm<9999.995) fprintf(fout,"  %8.3f",s/cm);
	 else if (s/cm<99999.95) fprintf(fout,"  %8.2f",s/cm);
	 else if (s/cm<999999.5) fprintf(fout,"  %8.1f",s/cm);
	 else fprintf(fout,"  %8.2e",s/cm);
	 if (ssky/cm<99.9) fprintf(fout," %6.3f",ssky/cm);
	 else if (ssky/cm<999.9) fprintf(fout," %6.2f",ssky/cm);
	 else fprintf(fout," %6.1f",ssky/cm);
	 fprintf(fout," 99.999 99.999  0.000   0.00     0.0  0.000  0.000  0.000");
	 if (fall) fprintf(fall,"      0.0      0.0      0.0");
      }
      else {
	 fprintf(fout,"       0.0  0.000 99.999 99.999  0.000   0.00     0.0  0.000  0.000  0.000");
	 if (fall) fprintf(fall,"      0.0      0.0      0.0");
      }
   }
   fprintf(fout,"\n");
   if (fall) fprintf(fall,"\n");
   return 1;
}

void setsmag(double*smag) {
   int f0=-1,i;
   float cui=0,mlt=1,dcol[5]={2.677834,1.930288,1.00000,0.484694,0.000000};

   for (i=0;i<5;i++) if (smag[i]<99.5) {
      if (f0==-1) {
	 f0=i;
	 cui=smag[i];
	 mlt=dcol[i];
      }
      else {
	 cui-=smag[i];
	 mlt-=dcol[i];
      }
   }
   cui/=mlt;
   for (i=0;i<5;i++) smag[i]=smag[f0]+(dcol[i]-dcol[f0])*cui;
}

inline double frand(void) {
   return (rand()/10%100)*.01+(rand()/10%100)*.0001+(rand()/10%100+.5)*.000001;
}

inline int poiss(double m) {
   int n,mmin,mmax,pmax;
   double logm;

   if (m<=0.) return 0;
   mmin=(int)(m-5*sqrt(m)-1);
   if (mmin<0) mmin=0;
   mmax=(int)(m+5*sqrt(m)+2)-mmin+1;
   logm=log(m);
   if (m>0.5) pmax=(m-0.5)*logm-lgamma(m+0.5);
   else pmax=0.;
   do {
      n=mmin+(int)(mmax*frand());
   } while (frand()>=exp(n*logm-lgamma(n+1)-pmax));
   return n;
}

void addfake(float X,float Y,double*hmag,float*S) {
   int img,x1,y1,ix,iy,fx,fy,it;
   float x,y,cts,cts0,s,ss,wt,TWT=0;

   *S=0;
   for (img=0;img<Nimg;img++) if (hmag[filt[img]]<99.999) {
      shift(img,X,Y,&x,&y);
      ix=(int)x; iy=(int)y;
      if (ix<0) ix=0; if (ix>799) ix=799;
      if (iy<0) iy=0; if (iy>799) iy=799;
      cts=(s=exp(-0.921034*(hmag[filt[img]]-filters[filt[img]].zero)))*EXP[img]/corr[IGN[img]][fld]/apcor[img];
      it=0;
      do {
	 cts0=cts;
	 cts=exp(-0.921034*(hmag[filt[img]]-filters[filt[img]].zero))*EXP[img]/corr[IGN[img]][fld]/CTE(img,y,x,cts0*apcor[img],sky[img][ix][iy])/apcor[img];
	 it++;
      } while (it<5 && fabs(cts0/cts-1)>0.01);
      ss=s/sqrt(cts*GAIN[img]);
      TWT+=(wt=1./ss/ss);
      (*S)+=s*wt;
      calc1psf(img,rpsf[fld],x,y,&fx,&fy);
      //Add random scatter
      cts*=1+(frand()+frand()-frand()-frand())/0.577562*0.011;
      for (x1=ix-rpsf[fld];x1<=ix+rpsf[fld];x1++) for (y1=iy-rpsf[fld];y1<=iy+rpsf[fld];y1++) if (ppixOK(img,x1,y1)) {
	 float pc0,pc;
	 pc0=cts*(tpsf[x1-ix][y1-iy]+dpsf[img][iy/100][ix/100][fx][fy]);
	 pc=(int)(poiss(pc0*GAIN[img])/(double)GAIN[img]+0.5);
	 res[img][x1][y1]+=pc;
	 data[img][x1][y1]+=pc;
      }
   }
   if (TWT>0) (*S)/=TWT;
   else (*S)=0;
}

int fakestarOK(float x,float y,float s0) {
   float r,br=0,dx,dy;
   int i;

   dx=stars[Nstars].x-x;
   dy=stars[Nstars].y-y;
   if (!fld) {
      dx/=1.5;
      dy/=1.5;
   }
   r=dx*dx+dy*dy;
   if (r<=9) br=s0*exp(-0.5*r);
   else return 0;
   for (i=Nstars-1;i>=0;i--) {
      dx=stars[Nstars].x-stars[i].x;
      dy=stars[Nstars].y-stars[i].y;
      if (!fld) {
	 dx/=1.5;
	 dy/=1.5;
      }
      r=dx*dx+dy*dy;
      if (r<=9 && stars[i].s*exp(-0.5*r)>br) return 0;
   }
   return 1;
}

inline int anypeak(int x,int y) {
   clearsnmap();
   setsnmap(-1,x-2,x+2,y-2,y+2);
   if (peak(x,y,0)>0 || peak(x-1,y-1,0)>0 || peak(x-1,y,0)>0 || peak(x-1,y+1,0)>0 || peak(x,y-1,0)>0 || peak(x,y+1,0)>0 || peak(x+1,y-1,0)>0 || peak(x+1,y,0)>0 || peak(x+1,y+1,0)>0) return 1;
   return 0;
}

void fakestars(FILE*ffake) {
   double smag[5],hmag[NFILTERS],dmag[NFILTERS],tmag[NFILTERS],**cmd;
   int i,j,x,y,r,USED[NFILTERS],m,c,nm,nc,xx,yy,ii,nfake=0;
   float ***res0,***data0,xc,yc,nf[32][32],nt=0,s0;
   char UBVRI[6]="UBVRI";

   if (fmult[fld]==0) return;
   printf("Running fake stars ... ");
   fflush(stdout);
   if (OPTS&4096) {
      for (x=0;x<32;x++) for (y=0;y<32;y++) {
	 if (x<2 || y<2) nf[x][y]=0.;
	 else nf[x][y]=1./900.;
      }
   }
   else {
      for (x=0;x<32;x++) for (y=0;y<32;y++) nf[x][y]=0.;
      for (x=50;x<800;x++) for (y=50;y<800;y++) for (i=0;i<Nimg;i++) {
	 shift(i,x,y,&xc,&yc);
	 if (ppixOK2(i,(int)xc,(int)yc)) {
	    float z;
	    z=data[i][(int)xc][(int)yc]-res[i][(int)xc][(int)yc]*0.75-sky[i][(int)xc][(int)yc];
	    if (z<0) z=0;
	    z+=sky[i][(int)xc][(int)yc]*0.25;
	    nt+=z;
	    nf[x/25][y/25]+=z;
	 }
      }
      for (x=2;x<32;x++) for (y=2;y<32;y++) {
	 nf[x][y]/=nt;
	 if (nf[x][y]<0) nf[x][y]=0;
      }
   }
   nm=(int)((flim[0][1]-flim[0][0])*2+0.5);
   if (nm<1) nm=1;
   nc=(int)((flim[1][1]-flim[1][0])*4+0.5);
   if (nc<1) nc=1;
   for (i=0;i<NFILTERS;i++) USED[i]=0;
   res0=(float***)calloc(sizeof(float**),Nimg);
   data0=(float***)calloc(sizeof(float**),Nimg);
   if (!res0 || !data0) merr();
   r=2*rpsf[fld]+1;
   for (i=0;i<Nimg;i++) {
      USED[filt[i]]=1;
      res0[i]=(float**)calloc(sizeof(float*),r);
      data0[i]=(float**)calloc(sizeof(float*),r);
      if (!res0[i] || !data0[i]) merr();
      res0[i]+=rpsf[fld];
      data0[i]+=rpsf[fld];
      for (x=-rpsf[fld];x<=rpsf[fld];x++) {
	 res0[i][x]=(float*)calloc(sizeof(float),r);
	 data0[i][x]=(float*)calloc(sizeof(float),r);
	 if (!res0[i][x] || !data0[i][x]) merr();
	 res0[i][x]+=rpsf[fld];
	 data0[i][x]+=rpsf[fld];
      }
   }
   cmd=(double**)calloc(sizeof(double*),nm+1);
   if (!cmd) merr();
   for (m=0;m<=nm;m++) {
      cmd[m]=(double*)calloc(sizeof(double),nc);
      if (!cmd[m]) merr();
   }
   if (fmult[fld]<0) {
      float mm,s,ss,ss0,ch,sh,ssky;

      for (m=0;m<=nm;m++) for (c=0;c<nc;c++) cmd[m][c]=0;
      for (i=0;i<Nstars;i++) if (stars[i].type<4) {
	 imgadd(i,0);
	 for (m=0;m<NFILTERS;m++) {
	    hmag[m]=99.999;

	    for (j=0;j<Nimg && m!=filt[j];j++);
	    if (j<Nimg) {
	       eval(m,stars[i].x,stars[i].y,&mm,&s,&ss,&ch,&sh,&ssky,NULL,NULL,NULL,-1);
	       if (ch>1) ss0=ss/ch;
	       else ss0=ss;
	       if (s>0 && s/ss0>=findsig) {
		  hmag[m]=-2.5*log10(s)+filters[m].zero;
		  dmag[m]=ss/s*1.0857362;
	       }
	    }
	 }
	 imgsub(i,0);
	 transform(hmag,dmag,tmag);
	 for (j=0;j<5;j++) smag[j]=findcolor(tmag,UBVRI[j],NULL);
	 if (smag[ffilt[0]]<90 && smag[ffilt[1]]<90 && smag[ffilt[0]]>=flim[0][0] && smag[ffilt[0]]-smag[ffilt[1]]>=flim[1][0]) {
	    m=(int)((smag[ffilt[0]]-flim[0][0])*2.);
	    c=(int)((smag[ffilt[0]]-smag[ffilt[1]]-flim[1][0])*4.);
	    if (m<=nm && c<nc) cmd[m][c]++;
	 }
      }
      for (m=0;m<=nm;m++) {
	 for (c=0;c<nc;c++) {
	    if (m>0 && cmd[m][c]<cmd[m-1][c]) cmd[m][c]=cmd[m-1][c];
	    if (m<nm && cmd[m][c]<0.5*cmd[m+1][c]) cmd[m][c]=0.5*cmd[m+1][c];
	 }
	 for (c=0;c<nc;c++) if (c<nc-1 && cmd[m][c]<0.5*cmd[m][c+1]) cmd[m][c]=0.5*cmd[m][c+1];
	 for (c=nc-1;c>=nc;c++) if (c>0 && cmd[m][c]<0.5*cmd[m][c-1]) cmd[m][c]=0.5*cmd[m][c-1];
      }
      for (m=0;m<=nm;m++) for (c=0;c<nc;c++) {
	 if (cmd[m][c]<25) cmd[m][c]=25;
	 cmd[m][c]*=fabs(fmult[fld]);
      }
   }
   else for (m=0;m<=nm;m++) for (c=0;c<nc;c++) cmd[m][c]=900*fmult[fld];
   for (xx=2;xx<32;xx++) for (yy=2;yy<32;yy++) for (m=0;m<=nm;m++) for (c=0;c<nc;c++) {
      int nfb;
      double tmpx;

      tmpx=nf[xx][yy]*cmd[m][c];
      nfb=(int)tmpx;
      if (tmpx-nfb>frand()) nfb++;
      for (ii=0;ii<nfb;ii++) {
	 xc=(xx+frand())*25;
	 yc=(yy+frand())*25;
	 if (xc>=XMIN2 && xc<XMAX2-rpsf[fld] && yc>=YMIN2 && yc<YMAX2-rpsf[fld]) {
	    for (i=0;i<5;i++) smag[i]=99.999;
	    smag[ffilt[0]]=flim[0][0]+(m+frand())*0.5;
	    smag[ffilt[1]]=smag[ffilt[0]]-(flim[1][0]+(c+frand())*0.25);
	    setsmag(smag);
	    stars[Nstars].x=xc;
	    stars[Nstars].y=yc;
	    for (i=0;i<NFILTERS;i++) {
	       if (USED[i]) hmag[i]=untransform(i,smag);
	       else hmag[i]=99.999;
	    }
	    if (m<nm) {
	       for (i=0;i<Nimg;i++) {
		  float tx,ty;
		  shift(i,xc,yc,&tx,&ty);
		  for (x=-rpsf[fld];x<=rpsf[fld];x++) for (y=-rpsf[fld];y<=rpsf[fld];y++) if ((int)tx+x>=0 && (int)tx+x<800 && (int)ty+y>=0 && (int)ty+y<800) {
		     res0[i][x][y]=res[i][x+(int)tx][y+(int)ty];
		     data0[i][x][y]=data[i][x+(int)tx][y+(int)ty];
		  }
	       }
	       stars[Nstars].type=1;
	       addfake(xc,yc,hmag,&s0);
	       if (!anypeak((int)xc,(int)yc) || photsearch(-1,xc,yc,&stars[Nstars].x,&stars[Nstars].y,&stars[Nstars].s,&stars[Nstars].ss,&stars[Nstars].sky,&stars[Nstars].chi,&stars[Nstars].sh,stars[Nstars].is,stars[Nstars].iss,stars[Nstars].icm,0)) stars[Nstars].s=0;
	       if (fakestarOK(xc,yc,s0) && stars[Nstars].s>0 && stars[Nstars].s>=findsig*stars[Nstars].ss/(stars[Nstars].chi>1.?stars[Nstars].chi:1.) && stars[Nstars].x>=XMIN2 && stars[Nstars].x<XMAX2 && stars[Nstars].y>=YMIN2 && stars[Nstars].y<YMAX2 && starOK(Nstars,-1)) finalphot(Nstars);
	       else stars[Nstars].s=0;
	    }
	    else stars[Nstars].s=0;
	    nfake++;
	    fprintf(ffake,"%d %6.2f %6.2f ",fld,yc,xc);
	    for (i=0;i<5;i++) fprintf(ffake,"%6.3f ",smag[i]);
	    for (i=0;i<NFILTERS;i++) if (USED[i]) fprintf(ffake,"%6.3f ",hmag[i]);
	    fprintf(ffake," ");
	    out1star(Nstars,ffake,NULL,1);
	    if (m<nm) for (i=0;i<Nimg;i++) {
	       float tx,ty;
	       shift(i,xc,yc,&tx,&ty);
	       for (x=-rpsf[fld];x<=rpsf[fld];x++) for (y=-rpsf[fld];y<=rpsf[fld];y++) if ((int)tx+x>=0 && (int)tx+x<800 && (int)ty+y>=0 && (int)ty+y<800) {
		  res[i][x+(int)tx][y+(int)ty]=res0[i][x][y];
		  data[i][x+(int)tx][y+(int)ty]=data0[i][x][y];
	       }
	    }
	 }
      }
   }
   for (i=0;i<Nimg;i++) {
      for (x=-rpsf[fld];x<=rpsf[fld];x++) {
	 free(res0[i][x]-rpsf[fld]);
	 free(data0[i][x]-rpsf[fld]);
      }
      free(res0[i]-rpsf[fld]);
      free(data0[i]-rpsf[fld]);
   }
   res0=(float***)calloc(sizeof(float**),Nimg);
   data0=(float***)calloc(sizeof(float**),Nimg);
   for (m=0;m<=nm;m++) free(cmd[m]);
   free(cmd);
   fflush(ffake);
   printf("%d written\n",nfake);
   fflush(stdout);
}

void outpt(FILE*fout,FILE*fall,FILE**fres) {
   int i,Nelim[2]={0,0};

   sortstars();
   for (i=0;i<Nstars;i++) if (stars[i].s>0 && stars[i].s>=minsig*stars[i].ss && stars[i].x>=XMIN2 && stars[i].x<XMAX2 && stars[i].y>=YMIN2 && stars[i].y<YMAX2 && starOK(i,-1)) {
      imgadd(i,0);
      finalphot(i);
      imgsub(i,0);
   }
   for (i=0;i<Nimg;i++) ffwrite(res[i],800*800,4,fres[i]);
   for (i=0;i<Nstars;i++) {
      imgadd(i,0);
      Nelim[out1star(i,fout,fall,0)]++;
      imgsub(i,0);
   }
   printf("%d stars written; %d stars deleted\n",Nelim[1],Nelim[0]);
   fflush(stdout);
   fflush(fout);
   fflush(fall);
}

void outptinfo(FILE *finfo,char**argv) {
   int NPF[NFILTERS],j,N;

   fprintf(finfo,"%f detection S/N threshold\n",findsig);
   fprintf(finfo,"%f output S/N threshold\n",outsig);
   for (j=0;j<NFILTERS;j++) NPF[j]=0;
   for (j=0;j<Nimg;j++) NPF[filt[j]]++;
   N=Nimg;
   for (j=0;j<NFILTERS;j++) if (NPF[j]>1) N++;
   fprintf(finfo,"%d sets of output data\n",N);
   for (j=0;j<NFILTERS;j++) if (NPF[j]>1) fprintf(finfo,"%s\n",filters[j].name);
   for (j=0;j<Nimg;j++) {
      fprintf(finfo,"%s %s\n",filters[filt[j]].name,argv[10+j*3]);
      fprintf(finfo,"%16.8f\n",EPOCH[j]);
   }
   fprintf(finfo,"\n");
   fflush(finfo);
}

void outptalign(FILE*finfo) {
   int i;
   fprintf(finfo,"Chip %d alignment\n",fld);
   for (i=0;i<Nimg;i++) fprintf(finfo,"  %f %f\n",dpos[i][0],dpos[i][1]);
   fflush(finfo);
}

void outptapcor(FILE*finfo) {
   int i;
   fprintf(finfo,"Chip %d aperture corrections\n",fld);
   for (i=0;i<Nimg;i++) fprintf(finfo,"  %f\n",-2.5*log10(apcor[i]));
   fprintf(finfo,"\n");
   fprintf(finfo,"Chip %d default aperture corrections\n",fld);
   for (i=0;i<Nimg;i++) fprintf(finfo,"  %f\n",apcor0[fld]+smapcor0[fld]+(appsfm[fld]+smappsfm[fld])*poff[i][0][0]);
   fprintf(finfo,"\n");
   fflush(finfo);
}

int ferr(FILE *f,char *str) {
   printf("****%s\n",str);
   fclose(f);
   return 1;
}

int getfunc(char*str) {
   if (!strcasecmp(str,"gauss") || !strcasecmp(str,"g")) return 0;
   else if (!strcasecmp(str,"hubble") || !strcasecmp(str,"h")) return 1;
   else if (!strcasecmp(str,"moffat10") || !strcasecmp(str,"m10")) return 1;
   else if (!strcasecmp(str,"moffat15") || !strcasecmp(str,"m15")) return 2;
   else if (!strcasecmp(str,"moffat20") || !strcasecmp(str,"m20")) return 3;
   else if (!strcasecmp(str,"moffat25") || !strcasecmp(str,"m25")) return 4;
   else {
      printf("Unknown psf shape\n");
      exit(-1);
   }
   return -1;
}

int main(int argc,char**argv) {
   FILE **fdata,*fout,*fall,*finfo,*fapcor,*fpsfs,**fpsf,**fres,*ffake=NULL;
   char str[82];
   int MODE,line,tmp,it,x,y,img,HPP;
   float texp[NFILTERS];

   OPTS=atoi(argv[9]);
   if (OPTS&1024) argc--;
   if (OPTS&64) argc-=10;
   if (OPTS&256) {
      Nimg=(argc-11)/15;
      argc-=12*Nimg;
   }
   if (argc<14 || argc%3!=2) {
      printf("****Usage: %s <output> <detection sig> <output sig> <fld> <xmin> <xmax> <ymin> <ymax> <options>\n (<fits base> <dX> <dY>) * #exposures <ref base>\n\n",*argv);
      return 1;
   }
   findsig=minsig=atof(argv[2]);
   outsig=atof(argv[3]);
   if (outsig>minsig) minsig=outsig;
   if (atoi(argv[4])>=0 && atoi(argv[4])<4) {
      MMIN=MMAX=atoi(argv[4]);
      YMIN2=atoi(argv[5]);
      YMAX2=atoi(argv[6]);
      XMIN2=atoi(argv[7]);
      XMAX2=atoi(argv[8]);
   }
   if (OPTS&1) rap[0]=2.1;
   else rap[0]=3.;
   rpsf[0]=6;
   sub[0]=5;
   rmin[0]=2.25;
   Nimg=(argc-11)/3;
   if (OPTS&32) {apprior=smapprior=0.;}
   if (argv[argc-1][0]) Timg=Nimg+1;
   else Timg=Nimg;
   getpars();
   initrows();
   fdata=(FILE**)calloc(Timg,sizeof(FILE*));
   fpsf=(FILE**)calloc(Nimg,sizeof(FILE*));
   fres=(FILE**)calloc(Nimg,sizeof(FILE*));
   if (!fdata || !fpsf || !fres) merr();
   initfilters();
   fout=fopen(argv[1],"w");
   if (!fout) {
      printf("****Error opening %s\n",argv[1]);
      exit(-1);
   }
   if ((OPTS&1) && (OPTS&2)) {
      //cheating; don't actually have a solution for this parameter combination
      float new_smapcor0[4]={-0.005,-0.008,-0.008,-0.007};
      float new_smappsfm[4]={0.022,0.222,0.270,0.211};
      float new_apcor0[4]={-0.011,-0.014,-0.091,-0.037};
      float new_appsfm[4]={0.300,1.947,2.295,1.929};
      for (x=0;x<4;x++) {
	 smapcor0[x]=new_smapcor0[x];
	 smappsfm[x]=new_smappsfm[x];
	 apcor0[x]=new_apcor0[x];
	 appsfm[x]=new_appsfm[x];
      }
      smapprior=600.;
      apprior=35.;
   }
   else if (OPTS&1) {
      float new_smapcor0[4]={-0.005,-0.008,-0.008,-0.007};
      float new_smappsfm[4]={0.022,0.222,0.270,0.211};
      float new_apcor0[4]={-0.011,-0.014,-0.091,-0.037};
      float new_appsfm[4]={0.300,1.947,2.295,1.929};
      for (x=0;x<4;x++) {
	 smapcor0[x]=new_smapcor0[x];
	 smappsfm[x]=new_smappsfm[x];
	 apcor0[x]=new_apcor0[x];
	 appsfm[x]=new_appsfm[x];
      }
      smapprior=600.;
      apprior=35.;
   }
   else if ((OPTS&2048) && (OPTS&2)) {
      //cheating; don't actually have a solution for this parameter combination
      float new_smapcor0[4]={-0.025,-0.030,-0.033,-0.029};
      float new_smappsfm[4]={ 0.052,-0.009,-0.033,-0.013};
      float new_apcor0[4]={-0.003,-0.006,-0.075,-0.027};
      float new_appsfm[4]={0.343,0.697,1.043,0.751};
      for (x=0;x<4;x++) {
	 smapcor0[x]=new_smapcor0[x];
	 smappsfm[x]=new_smappsfm[x];
	 apcor0[x]=new_apcor0[x];
	 appsfm[x]=new_appsfm[x];
      }
   }
   else if (OPTS&2048) {
      float new_smapcor0[4]={-0.025,-0.030,-0.033,-0.029};
      float new_smappsfm[4]={ 0.052,-0.009,-0.033,-0.013};
      float new_apcor0[4]={-0.003,-0.006,-0.075,-0.027};
      float new_appsfm[4]={0.343,0.697,1.043,0.751};
      for (x=0;x<4;x++) {
	 smapcor0[x]=new_smapcor0[x];
	 smappsfm[x]=new_smappsfm[x];
	 apcor0[x]=new_apcor0[x];
	 appsfm[x]=new_appsfm[x];
      }
   }
   else if (OPTS&2) {
   }
   if (OPTS&64) {
      sprintf(str,"%s.fake",argv[1]);
      ffake=fopen(str,"w");
      if (!ffake) {
	 printf("****Error opening %s\n",str);
	 exit(-1);
      }
      srand(time(NULL));
      if (OPTS&1024) tmp=1;
      else tmp=0;
      for (x=0;x<2;x++) {
	 char sm[6]={"UBVRI"};
	 for (y=0;y<5;y++) if (sm[y]==argv[argc+tmp+x][0]) {
	    ffilt[x]=y;
	    y=6;
	 }
	 if (y==5) {
	    printf("Illegal standard color %s\n",argv[argc+tmp+x]);
	    exit(-1);
	 }
      }
      if (ffilt[0]==ffilt[1]) {
	 printf("Same standard color %s\n",argv[argc+tmp]);
	 exit(-1);
      }
      for (x=0;x<2;x++) {
	 for (y=0;y<2;y++) flim[x][y]=atof(argv[argc+tmp+2+x*2+y]);
	 if (flim[x][0]>=flim[x][1] || flim[x][1]>99) {
	    printf("Illegal magnitude limits\n");
	    exit(-1);
	 }
      }
      for (x=0;x<4;x++) fmult[x]=atof(argv[argc+tmp+6+x]);
   }
   sprintf(str,"%s.allmags",argv[1]);
   fall=fopen(str,"w");
   if (!fall) {
      printf("****Error opening %s\n",str);
      exit(-1);
   }
   sprintf(str,"%s.info",argv[1]);
   finfo=fopen(str,"w");
   if (!finfo) {
      printf("****Error opening %s\n",str);
      exit(-1);
   }
   sprintf(str,"%s.apcor",argv[1]);
   fapcor=fopen(str,"w");
   if (!fapcor) {
      printf("****Error opening %s\n",str);
      exit(-1);
   }
   sprintf(str,"%s.psfs",argv[1]);
   fpsfs=fopen(str,"w");
   if (!fpsfs) {
      printf("****Error opening %s\n",str);
      exit(-1);
   }
   for (x=0;x<NFILTERS;x++) texp[x]=0;
   for (img=0;img<Timg;img++) {
      if (img<Nimg) {
	 dpos[img][0]=dpos0[img][0]=atof(argv[12+img*3]);
	 dpos[img][1]=dpos0[img][1]=atof(argv[11+img*3]);
      }
      else dpos[img][0]=dpos0[img][0]=dpos[img][1]=dpos0[img][1]=0;
      sprintf(str,"%s.fits",argv[10+img*3]);
      if ((fdata[img]=fopen(str,"rb"))==NULL) {
	 printf("****File %s not found\n",str);
	 return 1;
      }
      MODE=line=HPP=0;
      while (!feof(fdata[img]) && (!MODE || line%36)) {
	 fgets(str,81,fdata[img]);
	 line++;
	 if (!MODE) {
	    if (!strncmp(str,"END     ",8)) MODE=1;
	    else if ((!strncmp(str,"SIMPLE  =",9)) && (str[29]!='T')) return ferr(fdata[img],"Illegal format type");
	    else if ((!strncmp(str,"BITPIX  =",9)) && (strncmp(str+27,"-32",3))) return ferr(fdata[img],"Illegal bits per pixel");
	    else if ((!strncmp(str,"NAXIS   =",9)) && (strncmp(str+27,"  3",3))) return ferr(fdata[img],"Illegal number of axes");
	    else if (!strncmp(str,"NAXIS",5)) {
	       tmp=atoi(str+9);
	       if (str[5]=='1' && tmp!=800) return ferr(fdata[img],"Illegal X axis size");
	       else if (str[5]=='2' && tmp!=800) return ferr(fdata[img],"Illegal Y axis size");
	       else if (str[5]=='3' && tmp!=4) return ferr(fdata[img],"Illegal Z axis size");
	    }
	    else if (!strncmp(str,"ATODGAIN=",9)) {
	       GAIN[img]=atof(str+9);
	       if (GAIN[img]==7) IGN[img]=0;
	       else if (GAIN[img]==14) IGN[img]=1;
	    }
	    else if (!strncmp(str,"RNOISE  =",9)) RN[img]=atof(str+9);
	    else if (!strncmp(str,"EPOCH   =",9)) {
	       YEAR[img]=((EPOCH[img]=atof(str+9))-49462.5)/365.25;
	       if (YEAR[img]<0) YEAR[img]=0;
	    }
	    else if (!strncmp(str,"EXPTIME =",9)) EXP[img]=atof(str+9);
	    else if (!strncmp(str,"EXP0    =",9)) EXP0[img]=atof(str+9);
	    else if (!strncmp(str,"RSDPFILL=",9)) DMIN[img]=atoi(str+9);
	    else if (!strncmp(str,"SATURATE=",9)) DMAX[img]=atoi(str+9);
	    else if (!strncmp(str,"TEMP    =",9)) TEMP[img]=atoi(str+9);
	    else if (!strncmp(str,"FILTNAM1=",9)) {
	       int i=0;
	       while ((i<19) && (str[i+11]!=32)) {
		  FILT[img][i]=str[i+11];
		  i++;
	       }
	       FILT[img][i]=0;
	    }
	    else if (!strncmp(str,"HSTPHOT =",9)) HPP=1;
	 }
      }
      if (!HPP) return ferr(fdata[img],"Not pre-processed by mask");
      printf("Reading FITS file: %s.fits\n",argv[10+3*img]);
      printf("Exp time=%fs, Gain=%f e/ADU, Noise=%f ADU\n",EXP[img],GAIN[img],RN[img]);
      printf("Saturation at %d, min value=%d, temp=%d\n",DMAX[img],DMIN[img],TEMP[img]);
      printf("Filter=\"%s\"\n",FILT[img]);
      fflush(stdout);
      filt[img]=findfilt(FILT[img]);
      if (img<Nimg) {
	 texp[filt[img]]+=EXP[img];
	 if (OPTS&256) {
	    char fname[5][6]={"gauss","mof10","mof15","mof20","mof25"};
	    x=argc+12*img;
	    if (OPTS&1024) x++;
	    if (OPTS&64) x+=10;
	    printf("Lib Sigma:");
	    for (y=0;y<4;y++) printf(" %5.3f",sigpsf[img][0][y]=atof(argv[x+3*y]));
	    printf("\n");
	    printf("New Sigma:");
	    for (y=0;y<4;y++) printf(" %5.3f",sigpsf[img][1][y]=atof(argv[x+1+3*y]));
	    printf("\n");
	    printf("Functions:");
	    for (y=0;y<4;y++) printf(" %5s",fname[sigfpsf[img][y]=getfunc(argv[x+2+3*y])]);
	    printf("\n");
	 }
	 sprintf(str,"%s.%d.psf",argv[1],img);
	 fpsf[img]=fopen(str,"wb");
	 if (!fpsf[img]) {
	    printf("****Error opening %s\n",str);
	    exit(-1);
	 }
	 sprintf(str,"%s.res",argv[10+img*3]);
	 fres[img]=fopen(str,"wb");
	 if (!fres[img]) {
	    printf("****Error opening %s\n",str);
	    exit(-1);
	 }
      }
   }
   outptinfo(finfo,argv);
   for (fld=0;fld<4;fld++) {
      for (img=0;img<Timg;img++) {
	 ffread(data[img],sizeof(dtype),1,fdata[img]);
	 poff[img]=(float**)calloc(2*rpsf[fld]+1,sizeof(float*));
	 if (!poff[img]) merr();
	 poff[img]+=rpsf[fld];
	 for (tmp=-rpsf[fld];tmp<=rpsf[fld];tmp++) {
	    poff[img][tmp]=(float*)calloc(2*rpsf[fld]+1,sizeof(float));
	    if (!poff[img][tmp]) merr();
	    poff[img][tmp]+=rpsf[fld];
	 }
      }
      if (fld>=MMIN && fld<=MMAX) {
	 WARM=0;
	 if (!fld) printf("** PC **\n");
	 else printf("** WFC%d **\n",fld+1);
	 fflush(stdout);
	 getpsf(fld);
	 XMIN=XMIN2-rpsf[fld]-rphot[fld];
	 if (XMIN<50) XMIN=50;
	 XMAX=XMAX2+rpsf[fld]+rphot[fld];
	 if (XMAX>800) XMAX=800;
	 YMIN=YMIN2-rpsf[fld]-rphot[fld];
	 if (YMIN<50) YMIN=50;
	 YMAX=YMAX2+rpsf[fld]+rphot[fld];
	 if (YMAX>800) YMAX=800;
	 initpixmap();
	 for (img=0;img<Timg;img++) {
	    sprintf(str,"%s.sky",argv[10+img*3]);
	    readsky(img,str);
	    apcor[img]=pow(10,-0.4*apcor0[fld]-0.4*smapcor0[fld]);
	 }
	 NO_SKY=1;
	 if (!(OPTS&16) && Nimg>1) align();
	 else for (img=0;img<Nimg;img++) {
	    dpos[img][0]=dpos0[img][0];
	    dpos[img][1]=dpos0[img][1];
	 }
	 outptalign(finfo);
	 Nstars=0;
	 if (OPTS&1024) {
	    WARM=1;
	    readwarm(argv[argc]);
	 }
	 else find();
	 NO_SKY=0;
	 if (OPTS&512) for (x=0;x<Nstars;x++) stars[x].flag=1;
	 sortstars();
	 line=0;
	 it=1;
	 while (it && line<MaxIT) {
	    it=solve(++line,fpsfs);
	    if (it>1 && line>MaxIT/2) line=MaxIT/2;
	 }
	 for (img=0;img<Nimg;img++) {
	    cleansat(img);
	    if (!(OPTS&8)) getapcor(img,fapcor);
	    else calcapcor(img);
	 }
	 outptapcor(finfo);
	 if (OPTS&64) fakestars(ffake);
	 outpt(fout,fall,fres);
      }
      for (img=0;img<Nimg;img++) {
	 for (tmp=-rpsf[fld];tmp<=rpsf[fld];tmp++) {
	    ffwrite(poff[img][tmp]-rpsf[fld],sizeof(float),rpsf[fld]*2+1,fpsf[img]);
	    free(poff[img][tmp]-rpsf[fld]);
	 }
	 free(poff[img]-rpsf[fld]);
      }
   }
   fclose(fout);
   fclose(fall);
   fclose(finfo);
   fclose(fapcor);
   fclose(fpsfs);
   if (OPTS&64) fclose(ffake);
   for (img=0;img<Timg;img++) {
      fclose(fdata[img]);
      if (img<Nimg) fclose(fpsf[img]);
      if (img<Nimg) fclose(fres[img]);
   }
   return 0;
}
