#include <stdio.h>
#include <stdlib.h>
#include <math.h>
#include <string.h>
#include <unistd.h>
#ifdef LINUX
#include "linux.h"
#else
#include "sun.h"
#endif

typedef float dtype[4][800][800];
typedef short dqtype[4][800][800];

int X,Y,Z,BITS,DMAX,DMIN,TEMP;
double EXP,GAIN,RN,EPOCH=0.;
float scale=1.,zero=0.;
dtype data;
dqtype dq;
char FILT[20]={0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0};

void merr(void) {
   printf("Mem error\n");
   exit(0);
}

void ferr(FILE *f,char *str) {
   printf("%s\n",str);
   fclose(f);
   exit(1);
}

int main(int argc,char**argv) {
   FILE *f;
   char str[162],tfn[161];
   int MODE=0,line=0,tmp,fld,x,y;

   if (argc!=3) {
      printf("Usage: %s <data image> <DQ image>\n",*argv);
      return 1;
   }
   if ((f=fopen(argv[1],"rb"))==NULL) {
      printf("%s not found\n",argv[1]);
      return 1;
   }
   while ((!feof(f)) && ((!MODE) || (line%36))) {
      fread(str,1,80,f); str[80]=0;
      line++;
      if (!MODE) {
	 if (!strncmp(str,"END     ",8)) MODE=1;
	 else if ((!strncmp(str,"SIMPLE  =",9)) && (str[29]!='T')) ferr(f,"Illegal format type");
	 else if (!strncmp(str,"BITPIX  =",9)) {
	    BITS=atoi(str+9);
	    if (BITS!=32 && BITS!=-32) ferr(f,"Illegal bits per pixel");
	 }
	 else if ((!strncmp(str,"NAXIS   =",9)) && (strncmp(str+27,"  3",3))) ferr(f,"Illegal number of axes");
	 else if (!strncmp(str,"NAXIS",5)) {
	    tmp=atoi(str+9);
	    if (str[5]=='1') X=tmp;
	    else if (str[5]=='2') Y=tmp;
	    else if (str[5]=='3') Z=tmp;
	 }
	 else if (!strncmp(str,"BSCALE  =",9)) scale=atof(str+9);
	 else if (!strncmp(str,"BZERO   =",9)) zero=atof(str+9);
	 else if (!strncmp(str,"ATODGAIN=",9)) GAIN=atof(str+9);
	 else if (!strncmp(str,"EXPSTART=",9)) EPOCH+=atof(str+9)*0.5;
	 else if (!strncmp(str,"EXPEND  =",9)) EPOCH+=atof(str+9)*0.5;
	 else if (!strncmp(str,"EXPTIME =",9)) EXP=atof(str+9);
	 else if (!strncmp(str,"RSDPFILL=",9)) DMIN=atoi(str+9);
	 else if (!strncmp(str,"UCH1CJTM=",9)) {
	    if (atof(str+9)<-82) TEMP=0;
	    else TEMP=1;
	 }
	 else if (!strncmp(str,"SATURATE=",9)) DMAX=atoi(str+9);
	 else if (!strncmp(str,"FILTNAM1=",9)) {
	    int i=0;
	    while ((i<19) && (str[i+11]!=32)) {
	       FILT[i]=str[i+11];
	       i++;
	    }
	 }
      }
   }
   if (GAIN==7) RN=5/GAIN;
   else if (GAIN==14) RN=7.5/GAIN;
   else if (GAIN==15) {
      GAIN=14;
      RN=7.5/GAIN;
   }
   else {
      printf("Illegal Gain\n");
      exit(1);
   }
   if (X!=800 || Y!=800 || Z!=4) ferr(f,"Illegal axis size");
   if (BITS==-32) {
      ffread(data,4*800*800,4,f);
      if (scale!=1. || zero!=0.) for (fld=0;fld<4;fld++) for (x=0;x<800;x++) for (y=0;y<800;y++) data[fld][x][y]=data[fld][x][y]*scale+zero;
   }
   else for (fld=0;fld<4;fld++) for (x=0;x<800;x++) for (y=0;y<800;y++) {
      long ii;
      fread(&ii,1,4,f);
      data[fld][x][y]=(float)ii*scale+zero;
   }
   fclose(f);
   MODE=line=0;
   if ((f=fopen(argv[2],"rb"))==NULL) printf("%s not found, converting headers only\n",argv[2]);
   else {
      while ((!feof(f)) && ((!MODE) || (line%36))) {
	 fread(str,1,80,f); str[80]=0;
	 line++;
	 if (!MODE) {
	    if (!strncmp(str,"END     ",8)) MODE=1;
	    else if ((!strncmp(str,"SIMPLE  =",9)) && (str[29]!='T')) ferr(f,"Illegal format type");
	    else if ((!strncmp(str,"BITPIX  =",9)) && (strncmp(str+27," 16",3))) ferr(f,"Illegal bits per pixel");
	    else if ((!strncmp(str,"NAXIS   =",9)) && (strncmp(str+27,"  3",3))) ferr(f,"Illegal number of axes");
	    else if (!strncmp(str,"NAXIS",5)) {
	       tmp=atoi(str+9);
	       if (str[5]=='1') X=tmp;
	       else if (str[5]=='2') Y=tmp;
	       else if (str[5]=='3') Z=tmp;
	    }
	 }
      }
      if ((X!=800) || (Y!=800) || (Z!=4)) ferr(f,"Illegal axis size");
      sfread(dq,4*800*800,2,f);
      fclose(f);
      for (fld=0;fld<4;fld++) for (x=0;x<800;x++) for (y=0;y<800;y++) {
	 if ((dq[fld][x][y]&8) || data[fld][x][y]>=3500) data[fld][x][y]=DMAX+1;
	 else if ((dq[fld][x][y]&1) || (dq[fld][x][y]&2) || (dq[fld][x][y]&4) || (dq[fld][x][y]&16) || (dq[fld][x][y]&32) || (dq[fld][x][y]&128) || (dq[fld][x][y]&256) || (dq[fld][x][y]&512)) data[fld][x][y]=DMIN;
	 else if (x==799 || y==799) data[fld][x][y]=DMIN;
	 if ((dq[fld][x][y]&64) || (dq[fld][x][y]&2048) || (dq[fld][x][y]&4096)) printf("Error! %d %d\n",(dq[fld][x][y]&64),(dq[fld][x][y]&4096));
      }
   }
   sprintf(tfn,"temp.%d",(int)getpid());
   if ((f=fopen(tfn,"wb"))==NULL) {
      printf("Error on output file\n");
      exit(1);
   }
   fputs("SIMPLE  =                    T / Standard FITS format                           ",f);
   fputs("BITPIX  =                  -32 / 32 bit IEEE floating point numbers             ",f);
   fputs("NAXIS   =                    3 / Number of axes                                 ",f);
   fputs("NAXIS1  =                  800 /                                                ",f);
   fputs("NAXIS2  =                  800 /                                                ",f);
   fputs("NAXIS3  =                    4 /                                                ",f);
   fputs("EXTEND  =                    T / There may be standard extensions               ",f);
   sprintf(str,"ATODGAIN=       %14f / Analog to Digital Gain (Electrons/DN)          ",GAIN);
   fputs(str,f);
   sprintf(str,"RNOISE  =       %14f / Readout Noise (DN)                             ",RN);
   fputs(str,f);
   sprintf(str,"EPOCH   =     %16.8f / exposure average time (Modified Julian Date)   ",EPOCH);
   fputs(str,f);
   sprintf(str,"EXPTIME =       %14f / exposure duration (seconds)--calculated        ",EXP);
   fputs(str,f);
   sprintf(str,"EXP0    =       %14f / weighted average initial exposure time         ",EXP);
   fputs(str,f);
   sprintf(str,"RSDPFILL=       %14d / bad data fill value for calibrated images      ",DMIN);
   fputs(str,f);
   sprintf(str,"SATURATE=       %14d / Data value at which saturation occurs          ",DMAX);
   fputs(str,f);
   sprintf(str,"TEMP    =       %14d / Temperature (0=cold, 1=warm)                   ",TEMP);
   fputs(str,f);
   sprintf(str,"FILTNAM1= \'%-18s\' / first filter name                              ",FILT);
   fputs(str,f);
   fputs("HSTPHOT =                    T / Preprocessed by HSTphot/mask                   ",f);
   fputs("END                                                                             ",f);
   for (line=18;line<36;line++) fputs("                                                                                ",f);
   ffwrite(data,4*800*800,4,f);
   fclose(f);
   sprintf(str,"mv %s %s.old ; mv %s %s",argv[1],argv[1],tfn,argv[1]);
   if ((f=popen(str,"w"))!=NULL) pclose(f);
   else printf("Cannot replace old fits file\n");
   return 0;
}
