#include <stdio.h>
#include <stdlib.h>
#include <string.h>
#include <math.h>
#include "filters.h"

typedef char fntype[81];
typedef double dtype[9];
typedef struct {
   int f,rth,tp;
   double X,Y,chi,sn,sh,rnd;
   dtype*data;
} stype;
double dX,dY,dT,*epoch[2],fsig[2];
int *filt[2],*map[2],tnc=0,tnfc=0,nc[2]={0,0},nfc[2]={0,0},N[2]={0,0},MODE=0;
fntype *base[2];
stype *s[2];

void merr(void) {
   printf("Memory allocation error\n");
   exit(-1);
}

void ferr(char *fn) {
   printf("Could not open %s\n",fn);
   exit(-1);
}

void readinfo(char*mbase,int i) {
   fntype fn;
   FILE *f;
   int j;
   char *ptr;

   sprintf(fn,"%s.info",mbase);
   if ((f=fopen(fn,"r"))==NULL) ferr(fn);
   fgets(fn,81,f);
   if (!i) fsig[0]=atof(fn);
   fgets(fn,81,f);
   if (!i) fsig[1]=atof(fn);
   fgets(fn,81,f);
   nc[i]=atoi(fn);
   epoch[i]=(double*)calloc(sizeof(double),nc[i]);
   filt[i]=(int*)calloc(sizeof(int),nc[i]);
   base[i]=(fntype*)calloc(sizeof(fntype),nc[i]);
   if (!epoch[i] || !filt[i] || !base[i]) merr();
   for (j=0;j<nc[i];j++) {
      fgets(fn,81,f);
      fn[strlen(fn)-1]=0;
      for (ptr=fn;*ptr && *ptr!=32;ptr++);
      if (*ptr) {
	 *ptr=0;
	 filt[i][j]=findfilt(fn);
	 strcpy(base[i][j],ptr+1);
	 fgets(fn,81,f);
	 epoch[i][j]=atof(fn);
      }
      else {
	 filt[i][j]=findfilt(fn);
	 epoch[i][j]=0;
	 base[i][j][0]=0;
	 nfc[i]++;
      }
   }
   fclose(f);
}

void setmap(void) {
   int i,j,k,ct;

   for (i=0;i<2;i++) {
      map[i]=(int*)calloc(sizeof(int),nc[0]+nc[1]);
      if (!map[i]) merr();
   }
   for (i=0;i<NFILTERS;i++) {
      ct=0;
      for (j=0;j<2;j++) for (k=0;k<nc[j];k++) if (base[j][k][0] && filt[j][k]==i) ct++;
      if (ct>1) {
	 for (j=0;j<2;j++) {
	    map[j][tnc]=-1;
	    for (k=0;k<nc[j] && filt[j][k]!=i;k++);
	    if (k<nc[j]) map[j][tnc]=k;
	 }
	 tnc++;
	 tnfc++;
      }
   }
   for (j=0;j<2;j++) for (k=nfc[j];k<nc[j];k++) {
      map[j][tnc]=k;
      map[1-j][tnc]=-1;
      tnc++;
   }
}

void writeinfo(char*mbase) {
   fntype fn;
   FILE *f;
   int i;

   sprintf(fn,"%s.info",mbase);
   if ((f=fopen(fn,"w"))==NULL) ferr(fn);
   fprintf(f,"%f individual S/N threshold\n%f cumulative S/N threshold\n%d sets of output data\n",fsig[0],fsig[1],tnc);
   for (i=0;i<tnfc;i++) {
      if (map[0][i]>=0) fprintf(f,"%s\n",filters[filt[0][map[0][i]]].name);
      else fprintf(f,"%s\n",filters[filt[1][map[1][i]]].name);
   }
   for (;i<tnc;i++) {
      if (map[0][i]>=0) fprintf(f,"%s %s\n%16.8f\n",filters[filt[0][map[0][i]]].name,base[0][map[0][i]],epoch[0][map[0][i]]);
      else fprintf(f,"%s %s\n%16.8f\n",filters[filt[1][map[1][i]]].name,base[1][map[1][i]],epoch[1][map[1][i]]);
   }
   fclose(f);
}

//uses Holtzman et at (1995), PASP 107, 156
double CC[10][4]=
{{3.54356E+02,-8.12003E+02,-8.07068E+02,7.72904E+02},
 {1.00021E+00,1.95172E-02,-2.18757E+00,1.37619E-02},
 {9.79758E-04,-2.18805E+00,-8.51284E-03,2.18899E+00},
 {9.84222E-08,-1.51015E-06,-3.21458E-07,1.33326E-06},
 {-6.31327E-09,5.06693E-06,2.77459E-06,-4.00828E-06},
 {-7.19983E-07,1.04063E-06,-2.06874E-06,-5.71282E-07},
 {-3.73922E-08,-4.55094E-10,7.39223E-08,-2.00520E-09},
 {6.65101E-10,7.50249E-08,-1.18833E-09,-7.86673E-08},
 {-3.51470E-08,-1.51652E-09,7.73056E-08,-3.37491E-09},
 {-2.55003E-09,7.34568E-08,-3.72971E-10,-7.81490E-08}};
double DD[10][4]=
{{3.43646E+02,7.66592E+02,-7.71489E+02,-7.74638E+02},
 {1.00544E-03,2.18607E+00,7.40319E-03,-2.18688E+00},
 {9.99786E-01,2.02006E-02,-2.18554E+00,1.29831E-02},
 {-5.80870E-07,-3.79469E-06,-1.49841E-06,1.30814E-06},
 {-5.07221E-07,-2.99518E-06,2.96650E-06,2.01760E-06},
 {3.41574E-07,1.05789E-07,-2.02756E-07,-8.47656E-07},
 {-1.08091E-09,-7.47857E-08,-1.45720E-09,7.70408E-08},
 {-3.42070E-08,-1.67666E-09,7.72421E-08,-1.68514E-09},
 {1.17819E-09,-7.55302E-08,8.52633E-10,7.75257E-08},
 {-4.48966E-08,3.02579E-10,7.68023E-08,-3.69210E-10}};
double cc[10][4]=
{{4.28529E+01,4.99901E+01,2.91305E+01,4.06423E+01},
 {1.01839E+00,3.13580E-04,-4.65410E-01,6.89657E-03},
 {8.19691E-03,4.64781E-01,-2.53882E-03,-4.65172E-01},
 {-4.07640E-05,-2.44422E-06,-8.01738E-06,-2.85229E-06},
 {-2.44982E-05,5.28887E-06,-5.00071E-06,5.16479E-06},
 {-1.48524E-05,-7.39546E-06,-2.92348E-06,-7.91820E-06},
 {3.85393E-08,7.48860E-11,-3.32528E-09,2.55717E-11},
 {-8.32054E-10,3.41559E-09,8.12087E-11,-3.50591E-09},
 {3.65245E-08,-1.64561E-11,-3.50117E-09,-3.28654E-11},
 {2.58167E-09,3.37577E-09,4.45278E-11,-3.47401E-09}};
double dd[10][4]=
{{5.33594E+01,2.27199E+01,4.25019E+01,4.60263E+01},
 {7.27504E-03,-4.64924E-01,-5.86315E-03,4.64946E-01},
 {1.02076E+00,8.16708E-03,-4.65858E-01,-1.43623E-03},
 {-1.27390E-05,-7.93511E-06,-2.74006E-06,-8.16812E-06},
 {-2.37200E-05,4.68366E-06,-5.44820E-06,5.29464E-06},
 {-4.77746E-05,-2.98490E-06,-8.09822E-06,-2.99135E-06},
 {1.04890E-09,-3.30055E-09,4.25231E-11,3.51142E-09},
 {3.55851E-08,-7.84816E-12,-3.49983E-09,-1.12344E-10},
 {-1.40528E-09,-3.39413E-09,-5.98929E-11,3.55813E-09},
 {4.65028E-08,4.01436E-11,-3.46944E-09,-4.76932E-11}};

void gtransform(int c,double x,double y,double*xc,double*yc) {
   *xc=CC[0][c]+CC[1][c]*x+CC[2][c]*y+CC[3][c]*x*x+CC[4][c]*x*y+CC[5][c]*y*y+CC[6][c]*x*x*x+CC[7][c]*x*x*y+CC[8][c]*x*y*y+CC[9][c]*y*y*y;
   *yc=DD[0][c]+DD[1][c]*x+DD[2][c]*y+DD[3][c]*x*x+DD[4][c]*x*y+DD[5][c]*y*y+DD[6][c]*x*x*x+DD[7][c]*x*x*y+DD[8][c]*x*y*y+DD[9][c]*y*y*y;
}

void itransform(int c,double x,double y,double*xc,double*yc) {
   *xc=cc[0][c]+cc[1][c]*x+cc[2][c]*y+cc[3][c]*x*x+cc[4][c]*x*y+cc[5][c]*y*y+cc[6][c]*x*x*x+cc[7][c]*x*x*y+cc[8][c]*x*y*y+cc[9][c]*y*y*y;
   *yc=dd[0][c]+dd[1][c]*x+dd[2][c]*y+dd[3][c]*x*x+dd[4][c]*x*y+dd[5][c]*y*y+dd[6][c]*x*x*x+dd[7][c]*x*x*y+dd[8][c]*x*y*y+dd[9][c]*y*y*y;
}

double ratechip(double x,double y) {
   double rx,ry;
   rx=fabs(x-375);
   ry=fabs(y-375);
   if (rx>=ry) return -rx;
   return -ry;
}

void ungtransform(double x,double y,int*c,double*xc,double*yc) {
   double tx,ty,r,br;
   int k;
   itransform(*c,x,y,xc,yc);
   if (*xc<50 || *xc>799 || *yc<50 || *yc>799) {
      r=br=ratechip(*xc,*yc);
      for (k=0;k<4;k++) if (k!=*c) {
	 itransform(k,x,y,&tx,&ty);
	 if ((r=ratechip(tx,ty))>br) {
	    br=r;
	    *xc=tx;
	    *yc=ty;
	    *c=k;
	 }
      }
   }
}

void readphot(char*fn,int i) {
   FILE *f;
   char str[80001];
   int j,k,l;
   double sdT,cdT;

   if ((f=fopen(fn,"r"))==NULL) ferr(fn);
   while (fgets(str,80001,f)) N[i]++;
   fclose(f);
   s[i]=(stype*)calloc(sizeof(stype),N[i]);
   if (!s[i]) merr();
   sdT=sin(dT*M_PI/180);
   cdT=cos(dT*M_PI/180);
   if ((f=fopen(fn,"r"))==NULL) ferr(fn);
   for (j=0;j<N[i];j++) {
      fscanf(f,"%d %lf %lf %lf %lf %lf %lf %d %d",&s[i][j].f,&s[i][j].X,&s[i][j].Y,&s[i][j].chi,&s[i][j].sn,&s[i][j].sh,&s[i][j].rnd,&s[i][j].rth,&s[i][j].tp);
      s[i][j].data=(dtype*)calloc(sizeof(dtype),nc[i]);
      if (!s[i][j].data) merr();
      for (k=0;k<nc[i];k++) for (l=0;l<9;l++) fscanf(f,"%lf",s[i][j].data[k]+l);
      if (i==1) {
	 double tx,ty;
	 if (MODE) printf("%d %6.2f %6.2f ",s[i][j].f,s[i][j].X,s[i][j].Y);
	 gtransform(s[i][j].f,s[i][j].X-400,s[i][j].Y-400,&tx,&ty);
	 s[i][j].X=tx*cdT-ty*sdT+dX;
	 s[i][j].Y=tx*sdT+ty*cdT+dY;
	 if (MODE) printf("%7.2f %7.2f ",s[i][j].X,s[i][j].Y);
	 ungtransform(s[i][j].X,s[i][j].Y,&s[i][j].f,&tx,&ty);
	 if (MODE) printf("%d %6.2f %6.2f\n",s[i][j].f,tx,ty);
      }
   }
   fclose(f);
}

void align(void) {
   int i,j,Nlist=0;
   double list[10000][3],dx,dy,sig;

   for (i=0;i<N[0] && Nlist<10000;i++) if (s[0][i].tp<4 && s[0][i].chi<=3.5 && s[0][i].sn>=5 && fabs(s[0][i].sh)<=0.5) {
      double X,Y;
      gtransform(s[0][i].f,s[0][i].X-400,s[0][i].Y-400,&X,&Y);
      for (j=0;j<N[1] && Nlist<10000;j++) if (s[1][j].tp<4 && s[1][j].chi<=3.5 && s[1][j].sn>=5 && fabs(s[1][j].sh)<=0.5) {
	 dx=X-s[1][j].X;
	 dy=Y-s[1][j].Y;
	 if (dx*dx+dy*dy<=9) {
	    list[Nlist][0]=dx;
	    list[Nlist][1]=dy;
	    Nlist++;
	 }
      }
   }
   printf("%d stars matched for alignment\n",Nlist);
   dx=dy=sig=0;
   if (Nlist) do {
      dx=dy=sig=0;
      for (i=0;i<Nlist;i++) {
	 dx+=list[i][0];
	 dy+=list[i][1];
      }
      dx/=Nlist;
      dy/=Nlist;
      for (i=0;i<Nlist;i++) sig+=(list[i][2]=(list[i][0]-dx)*(list[i][0]-dx)+(list[i][1]-dy)*(list[i][1]-dy));
      sig=2.25*sqrt(sig/Nlist);
      j=0;
      for (i=0;i<Nlist;i++) if (list[i][2]>sig) {
	 Nlist--;
	 memcpy(list[i],list[Nlist],24);
	 j=1;
	 i--;
      }
   } while (Nlist && j);
   printf("Alignment adjusted to %5.2f, %5.2f, sig=%4.2f (%d)\n",dX+dx,dY+dy,sig/2.25,Nlist);
   for (i=0;i<N[1];i++) {
      s[1][i].X+=dx;
      s[1][i].Y+=dy;
   }
   for (i=0;i<N[1];i++) ungtransform(s[1][i].X,s[1][i].Y,&s[1][i].f,&s[1][i].X,&s[1][i].Y);
}

void out1star(FILE *f,int i0,int i1) {
   double m0,m1,mm0,mm1,*d0,*d1,d[9],tr[NFILTERS],hmag[NFILTERS],dmag[NFILTERS];
   int i,j,th;

   if (i1<0) fprintf(f,"%d %7.2f %7.2f %6.2f %7.1f %6.3f %6.3f %3d %d",s[0][i0].f,s[0][i0].X,s[0][i0].Y,s[0][i0].chi,s[0][i0].sn,s[0][i0].sh,s[0][i0].rnd,s[0][i0].rth,s[0][i0].tp);
   else if (i0<0) fprintf(f,"%d %7.2f %7.2f %6.2f %7.1f %6.3f %6.3f %3d %d",s[1][i1].f,s[1][i1].X,s[1][i1].Y,s[1][i1].chi,s[1][i1].sn,s[1][i1].sh,s[1][i1].rnd,s[1][i1].rth,s[1][i1].tp);
   else {
      m0=s[0][i0].sn*s[0][i0].sn/(s[0][i0].sn*s[0][i0].sn+s[1][i1].sn*s[1][i1].sn);
      m1=1-m0;
      if (s[0][i0].rth-s[1][i1].rth>90) th=5*(int)(0.2*(s[0][i0].rth-90)*m0+0.2*s[1][i1].rth*m1+0.5);
      else if (s[0][i0].rth-s[1][i1].rth>=-90) th=5*(int)(0.2*s[0][i0].rth*m0+0.2*s[1][i1].rth*m1+0.5);
      else th=5*(int)(0.2*s[0][i0].rth*m0+0.2*(s[1][i1].rth-90)*m1+0.5);
      while (th<0) th+=180;
      while (th>=180) th-=180;
      fprintf(f,"%d %7.2f %7.2f %6.2f %7.1f %6.3f %6.3f %3d ",s[0][i0].f,s[0][i0].X*m0+s[1][i1].X*m1,s[0][i0].Y*m0+s[1][i1].Y*m1,s[0][i0].chi*m0+s[1][i1].chi*m1,sqrt(s[0][i0].sn*s[0][i0].sn+s[1][i1].sn*s[1][i1].sn),s[0][i0].sh*m0+s[1][i1].sh*m1,s[0][i0].rnd*m0+s[1][i1].rnd*m1,th);
      if (s[0][i0].tp>s[1][i1].tp) fprintf(f,"%d",s[0][i0].tp);
      else fprintf(f,"%d",s[1][i1].tp);
   }
   for (i=0;i<NFILTERS;i++) hmag[i]=dmag[i]=tr[i]=99.999;
   for (i=0;i<tnc;i++) {
      if (i0>=0 && map[0][i]>=0 && s[0][i0].data[map[0][i]][0]>0) mm0=s[0][i0].data[map[0][i]][2];
      else mm0=99.999;
      if (i1>=0 && map[1][i]>=0 && s[1][i1].data[map[1][i]][0]>0) mm1=s[1][i1].data[map[1][i]][2];
      else mm1=99.999;
      if (mm0<90 && mm1>90 && hmag[filt[0][map[0][i]]]>90) {
	 hmag[filt[0][map[0][i]]]=mm0;
	 dmag[filt[0][map[0][i]]]=s[0][i0].data[map[0][i]][4];
      }
      if (mm1<90 && mm0>90 && hmag[filt[1][map[1][i]]]>90) {
	 hmag[filt[1][map[1][i]]]=mm1;
	 dmag[filt[1][map[1][i]]]=s[1][i1].data[map[1][i]][4];
      }
      if (mm0<90 && mm1<90 && hmag[filt[0][map[0][i]]]>90) {
	 m0=s[0][i0].data[map[0][i]][6]*s[0][i0].data[map[0][i]][6]/(s[0][i0].data[map[0][i]][6]*s[0][i0].data[map[0][i]][6]+s[1][i1].data[map[1][i]][6]*s[1][i1].data[map[1][i]][6]);
	 m1=1-m0;
	 hmag[filt[0][map[0][i]]]=m0*mm0+m1*mm1;
	 dmag[filt[0][map[0][i]]]=m0*s[0][i0].data[map[0][i]][4]+m1*s[1][i1].data[map[1][i]][4];
      }
   }
   transform(hmag,dmag,tr);
   for (i=0;i<NFILTERS;i++) if (tr[i]<90) tr[i]-=hmag[i];
   for (i=0;i<tnc;i++) {
      if (i0>=0 && map[0][i]>=0 && s[0][i0].data[map[0][i]][0]>0) d0=s[0][i0].data[map[0][i]];
      else d0=NULL;
      if (i1>=0 && map[1][i]>=0 && s[1][i1].data[map[1][i]][0]>0) d1=s[1][i1].data[map[1][i]];
      else d1=NULL;
      if (d0==NULL && d1==NULL) fprintf(f,"       0.0   0.00 99.999 99.999  0.000   0.00     0.0  0.000  0.000");
      else if (d1==NULL) {
	 if (d0[0]<999999.9) fprintf(f,"  %8.1f",d0[0]);
	 else fprintf(f,"  %8.2e",d0[0]);
	 if (d0[1]<999.9) fprintf(f," %6.2f",d0[1]);
	 else fprintf(f," %6.1f",d0[1]);
	 fprintf(f," %6.3f ",d0[2]);
	 if (tr[filt[0][map[0][i]]]<90) fprintf(f,"%6.3f",d0[2]+tr[filt[0][map[0][i]]]);
	 else fprintf(f,"99.999");
	 fprintf(f," %6.3f %6.2f %7.1f %6.3f %6.3f",d0[4],d0[5],d0[6],d0[7],d0[8]);
      }
      else if (d0==NULL) {
	 if (d1[0]<999999.9) fprintf(f,"  %8.1f",d1[0]);
	 else fprintf(f,"  %8.2e",d1[0]);
	 if (d1[1]<999.9) fprintf(f," %6.2f",d1[1]);
	 else fprintf(f," %6.1f",d1[1]);
	 fprintf(f," %6.3f ",d1[2]);
	 if (tr[filt[1][map[1][i]]]<90) fprintf(f,"%6.3f",d1[2]+tr[filt[1][map[1][i]]]);
	 else fprintf(f,"99.999");
	 fprintf(f," %6.3f %6.2f %7.1f %6.3f %6.3f",d1[4],d1[5],d1[6],d1[7],d1[8]);
      }
      else {
	 m0=d0[6]*d0[6]/(d0[6]*d0[6]+d1[6]*d1[6]);
	 m1=1-m0;
	 for (j=0;j<9;j++) d[j]=d0[j]*m0+d1[j]*m1;
	 if (d[0]<999999.9) fprintf(f,"  %8.1f",d[0]);
	 else fprintf(f,"  %8.2e",d[0]);
	 if (d[1]<999.9) fprintf(f," %6.2f",d[1]);
	 else fprintf(f," %6.1f",d[1]);
	 fprintf(f," %6.3f ",d[2]);
	 if (tr[filt[0][map[0][i]]]<90) fprintf(f,"%6.3f",d[2]+tr[filt[0][map[0][i]]]);
	 else fprintf(f,"99.999");
	 fprintf(f," %6.3f %6.2f %7.1f %6.3f %6.3f",d[4],d[5],d[6],d[7],d[8]);
      }
   }
   fprintf(f,"\n");
}

void writephot(char *fn) {
   FILE *f;
   int i,j,bj;
   double r,br=0;
   char *used;

   used=(char*)calloc(N[1],1);
   if (!used) merr();
   for (j=0;j<N[1];j++) used[j]=0;
   if ((f=fopen(fn,"w"))==NULL) ferr(fn);
   for (i=0;i<N[0];i++) {
      bj=-1;
      for (j=0;j<N[1];j++) if (!used[j] && s[0][i].f==s[1][j].f) {
	 double dx,dy;
	 dx=s[0][i].X-s[1][j].X;
	 dy=s[0][i].Y-s[1][j].Y;
	 r=dx*dx+dy*dy;
	 if (r<=1 && (bj<0 || r<br)) {
	    br=r;
	    bj=j;
	 }
      }
      if (bj>=0) used[bj]=1;
      out1star(f,i,bj);
   }
   for (j=0;j<N[1];j++) if (!used[j]) out1star(f,-1,j);
   fclose(f);
}

int main(int argc,char**argv) {
   if (argc!=7 && argc!=8) {
      printf("Usage: %s <mphot output 1> <mphot output 2> <output> <dXo> <dYo> <dT> <<verbose>>\n",*argv);
      return -1;
   }
   initfilters();
   readinfo(argv[1],0);
   readinfo(argv[2],1);
   setmap();
   writeinfo(argv[3]);
   dX=atof(argv[4]);
   dY=atof(argv[5]);
   dT=atof(argv[6]);
   if (argc==8) MODE=atoi(argv[7]);
   readphot(argv[2],1);
   readphot(argv[1],0);
   align();
   writephot(argv[3]);
   return 0;
}
