#include <stdio.h>
#include <stdlib.h>
#include <math.h>
#include <string.h>
#ifdef LINUX
#include "../linux.h"
#else
#include "../sun.h"
#endif

int N,fld,rpsf,*img;
float ***opsf,**psf;

void merr(void) {
   printf("Memory error\n");
   exit(-1);
}

void calcpsf(void) {
   int i,x,y;
   for (x=-rpsf;x<=rpsf;x++) for (y=-rpsf;y<=rpsf;y++) {
      psf[x][y]=0;
      for (i=0;i<N;i++) psf[x][y]+=opsf[i][x][y]/N;
   }
}

void showpsf(void) {
   int x,y;

   printf("Chip %d\n",fld);
   for (y=rpsf;y>=-rpsf;y--) {
      for (x=-rpsf;x<=rpsf;x++) printf("%6d",(int)(psf[y][x]*10000+0.5));
      printf("\n");
   }
   printf("\n");
}

int main(int argc,char**argv) {
   FILE **fin,*fout,*finfo;
   int i,j,x,y;
   char str[801],scmp[161];

   if (argc<4) {
      printf("Usage: %s <output base> <base> <img1> ... <imgN>\n",*argv);
      exit(-1);
   }
   N=argc-3;
   img=(int*)calloc(sizeof(int),N);
   fin=(FILE**)calloc(sizeof(FILE*),N);
   opsf=(float***)calloc(sizeof(float**),N);
   if (!img || !fin || !opsf) merr();
   for (i=0;i<N;i++) {
      img[i]=atoi(argv[3+i]);
      sprintf(str,"%s.%d.psf",argv[2],img[i]);
      if ((fin[i]=fopen(str,"rb"))==NULL) {
	 printf("Cannot open %s\n",str);
	 exit(-1);
      }
   }
   sprintf(str,"%s.dpsf",argv[1]);
   if ((fout=fopen(str,"wb"))==NULL) {
      printf("Cannot open %s\n",str);
      exit(-1);
   }
   for (fld=0;fld<4;fld++) {
      if (!fld) {
	 rpsf=6;
	 y=13;
	 psf=(float**)calloc(sizeof(float*),y);
	 if (!psf) merr();
	 psf+=rpsf;
	 for (x=-rpsf;x<=rpsf;x++) {
	    psf[x]=(float*)calloc(sizeof(float),y);
	    if (!psf[x]) merr();
	    psf[x]+=rpsf;
	 }
	 for (i=0;i<N;i++) {
	    opsf[i]=(float**)calloc(sizeof(float*),y);
	    if (!opsf[i]) merr();
	    opsf[i]+=rpsf;
	    for (x=-rpsf;x<=rpsf;x++) {
	       opsf[i][x]=(float*)calloc(sizeof(float),y);
	       if (!opsf[i][x]) merr();
	       opsf[i][x]+=rpsf;
	    }
	 }
      }
      else rpsf=4;
      for (i=0;i<N;i++) for (x=-rpsf;x<=rpsf;x++) ffread(opsf[i][x]-rpsf,sizeof(float),rpsf*2+1,fin[i]);
      calcpsf();
      showpsf();
      for (x=-rpsf;x<=rpsf;x++) ffwrite(psf[x]-rpsf,sizeof(float),rpsf*2+1,fout);
   }
   fclose(fout);
   for (i=0;i<N;i++) fclose(fin[i]);
   sprintf(str,"%s.apcor",argv[1]);
   if ((fout=fopen(str,"w"))==NULL) {
      printf("Cannot open %s\n",str);
      exit(-1);
   }
   for (fld=0;fld<4;fld++) {
      double apcor;
      sprintf(str,"%s.info",argv[2]);
      if ((finfo=fopen(str,"r"))==NULL) {
	 printf("Cannot open %s\n",str);
	 return -1;
      }
      sprintf(scmp,"Chip %d aperture corrections\n",fld);
      while (fgets(str,801,finfo) && strcmp(str,scmp));
      if (!strcmp(str,scmp)) {
	 i=-1;
	 j=0;
	 apcor=0;
	 while (j<N) {
	    while (i<img[j]) {
	       fgets(str,801,finfo);
	       i++;
	    }
	    apcor+=atof(str)/N;
	    j++;
	 }
	 printf("%f\n",apcor);
	 fprintf(fout,"%f\n",apcor);
      }
      else {
	 printf("Aperture corrections not found in info file\n");
	 return -1;
      }
      fclose(finfo);
   }
   fclose(fout);
   return 0;
}
